{*******************************************************************
*               Top-down RPC compiler - Main program
*


        28 May 1986     Written Antonio Pastore CERN/DD
        27 Jun 86       Last update by Antonio
         2 Nov 86       One output file used at a time. New options.
                        Explicitly ask for each stub module.
        30 Jan 87       .ext file produced with CLIENT stub, not server.
        10 Aug 87       Use globals ser_mode & cli_mode - Nici.
         1 May 88       Directives under MSDOS conditional for Turbo version
           Nov 88       PILS and FORTRAN generation added

Include options for the environment in which the compiler will run:

        VAXVMS          VMS operting system
        UNIXBSD         Berkley Unix 4.2 or 4.3, or DEc Ultrix
        MSDOS           Microsoft pascal compiler or Turbo pascal
        PCTURBO         Turbo pascal (use with MSDOS option)

**********************************************************************
*               SOME TECHNICAL NOTES                            - AP
**********************************************************************
 This compiler is organized in two parts:

        1) the PARSER
        2) the OUTPUT GENERATOR

    The PARSER checks the input for SYNTACTICAL and LEXICAL errors and
 produces as output two trees, the TYPEs and BLOCKs trees.

    The TYPEs tree (pointed by TYPEPTR) is a linked list of record each
 describing a defined type. To each of these records is associated a 'NAME',
 as defined by USER, and a sub-tree describing the structure of this type.

    The BLOCKs tree (pointed by BLOCKPTR) is a linked list of record each
 describing a block, that is a PROCEDURE or FUNCTION. Each BLOCK owns a
 NAME, (as given by the user), and a list of PARAMETERS, pointed by '.LIST'.
 Each record of this list has a NAME, the parameter's name (as given by
 the user), and a TYPE associated to it. This is a pointer, 'type', to a
 TYPE, so that the BLOCKs list points to the TYPEs list if BLOCKs have
 parameters. The same routine as used to build a TYPE definition is used to
 analize a parameter declaration inside a BLOCK.

    The OUTPUT GENERATOR reads the BLOCKs tree and produces as output two
 files, the CLIENT and the SERVER stubs. These are ASCII files containing
 procedures/functions that pack and unpack parameters and call
 the RPC Run-Time System.

    One output generator will produce Pascal or C stubs; there is also
 one to produce FORTRAN, and one for PILS.

**********************************************************************}

program rpcucompiler(input,output);


{* Global declaration *}
{*******************************************************************

                Top-down RPC compiler - Declaration part

History:
        28 May 86       Written, Antonio Pastore, technical Student, DD/OC
        11 Jul 86       Last update by Antonio
         2 Nov 86       Options concurrent, sm6809, cm6809 added
        10 Aug 87       genericC options added; structural changes - Nici.
        17 Aug 87       byvalue option added - Nici.
        26 Aug 87       Options timeout and version added - Nici.
         1 Sep 87       Options types and <s,c>pcturbo added - Nici.
        26 Aug 88       Options <s,c>macturbo added. Roberto Bagnara, DD/OC
         1 Nov 88       Options <s,c>pils added. TBL, LT.
        30 Mar 89       CAST option added - TBL
}
const
        rpcunameulength = 40;   { Must tally with RPC$CONST! }

        maxstring = 255;
        expressionulength = 80; { expression for compenent of composite type}
        templateulength =   10; { Length of expression templates }
        errorulength =      48; { Length of code generator error string }
        maxidlen  = 25;
        numkeyword= 27;         { Must = number of keywords in INIT  }
        numomodes = 11;         {  "     "     "    "   "  output_mode }
        maxdim    = 9;

        nonufragmentationulimit = 1488; { RPC_BUFFER_SIZE-CALL_HEADER_LENGTH
                                          in the worst case }

        fragmentationuthreshold = 150;  { Can break up types over this size }

        rpcudefaultutimeout = -1;   { Must agree with include files }

{ The following constant should be set to take parameters from the
 command line.
}
        commandline = true;


type

        longstring = packed array[1..maxstring] of char;
        charuname  = packed array [1..maxidlen] of char;
        char3      = packed array [1..3] of char;
        optuname   = packed array [1..10] of char;
        templateutype
                   = packed array [1..templateulength] of char;
        errorustring
                   = packed array [1..errorulength] of char;
        astring = record
                        str : packed array [1..maxstring] of char;
                        len : integer;
                  end;
        pilstring = record
                        str : packed array [1..maxstring] of char;
                        start, len : integer;
                    end;
        iduname = record
                        str : charuname;
                        len : integer;
                  end;

        expression = record
                        str : packed array[1..expressionulength] of char;
                        len : integer;
                  end;

        {* Types related to lexical analizer and tokens *}
        {*****
        * WARNING!!!
        * Be careful when changing the order of the following declaration!!
        ******}
        typeutoken = (
        chartok,                    { First token, simple type, courier type }
        bytetok,
        shortok, integertok,
        real32tok, real48tok,
        real64tok, real128tok,
        longtok,                    { Last simple type }
        arraytok,
        recordtok,                  { RECORD            added 23 Oct TBL }
        accesstok,                  { POINTER            .    .   .      }
        sequence,
        stringtok,
        substring,                  { Last courier type }
        package,
        semicolon,
        istok, typetok, endtok,
        colon, comma, opnround,
        clsround, oftok,
        intok,                      { First attr_type }
        outok,
        inoutok,                    { Last attr_type }
        dot, ident, number,
        proctok, functok, return,
        pragmatok,
        { add new tokens here }
        nultok);                    { Last type }

        tokenudescriptor = record
                                case kind : typeutoken of
                                ident : (name : iduname);
                                number: (value: integer);
                           end;
        courierutype = chartok .. substring;

        simpleutype  = chartok .. longtok;

        blockutype   = proctok .. functok;

        attrutype    = intok .. inoutok;

        ptrudefinedutype= ^definedutype;
        ptrunamedutype  = ^namedutype;
        ptruidlist      = ^idulist;
        ptrublockutable = ^blockutable;

{   Describes:  an abstract data type }

        definedutype = record
            typuname:       ptrunamedutype;     { The name if any }
            typuminusize:   integer;            { the minimum size in bytes }
            typumaxusize:   integer;            { The maximum size in bytes }
            typunesting:    integer;            { Number of indeces needed }
            typusubtype:    ptrudefinedutype;   { Subtype for complex types }
            typuexternal:   boolean;            { External marshalling }
            case typubasicutype: courierutype   { The type represented by this }
            of
                chartok, bytetok,
                 shortok, integertok,
                 real32tok, real48tok,
                 real64tok, real128tok,
                 longtok,
                 accesstok:    ();

                sequence,           { low is always 1 }
                 stringtok,         { low is always 1 }
                 substring,         { low is always 1 }
                arraytok: (         { low and high can vary }
                    typulow,
                    typuhigh:   integer);               { array bounds }
                recordtok: (
                    typufields: ptrunamedutype);        { List of subtypes }
        end {defined_type};

{   Descibes:   a Named type
}
        namedutype = record
                        ntyunext:   ptrunamedutype;
                        ntyuname:   iduname;
                        ntyutype:   ptrudefinedutype;
                      end;

{   Describes:   one parameter of a procedure/function
}
        idulist = record
                        next : ptruidlist;
                        name : iduname;
                        attr : attrutype;
                        idutype : ptrudefinedutype;
                  end;

{   Describes:  one procedure or function
}
        blockutable = record
                        next   : ptrublockutable;
                        name   : iduname;
                        butype : blockutype;
                        return : simpleutype;
                        list   : ptruidlist;
                        inuonly:        boolean;        { no data returned }
                        blkunesting:    integer;        { Number of indeces }
                        blkuminuin,
                        blkumaxuin:     integer;        { Tot Parameter size }
                        blkuminuout,
                        blkumaxuout:    integer;        { Tot Parameter size }
                        blkutimeout:    integer;        { Overall timeout }
                        blkustatusuparam: ptruidlist;   { Status parameter }
                        blkuconcurrent,                 { Concurrent execution}
                        blkucast:       boolean;        { CAST(one way message)}
                     end;

{   Error Codes:
}
        {* WARNING!
        * Do not change the first and the last element of the following list!
        *}
        errutype = (illubasicutype, numberureq, oftokumiss, simtypeureq,
                blocksureq, identureq, twiceudeclared, semicolumiss,
                varuattrureq, colonumiss, idunotudeclared, clsroundumiss,
                opnroundumiss, packageumiss, endtokumiss, charuignored,
                paramsumiss, inputumiss, commaumiss, semicuroundumiss,
                toomanyudim, toomanyudig, procuexp, cantuopnuinput,
                cantuopnuclient, cantuopnuserver, cantuclsuclient,
                cantuclsuserver, dotumiss, positiveureq, cantuclsuinput,
                identureserved, istokumiss, recordumiss,
                invalidurange, baduprocudecl,
                baduname, typeumiss, returnumiss, cantuopnuext,
                internaluerror,
                baduinputuname, cantuclsuext, unexpueof);

        setuofutoken = set of typeutoken;

        openumode =     (rewriting, resetting);

{ *** WARNING:  IF YOU MODIFY THIS DECLARATION:-

1.  Check that NUMOMODES in the constant section matches the number of entries
2.  Check that order of the the server and client options below matches - Nici
}

        outputumode =   (cerncross, m6809, monolith,    { ** SEE Note: }
                        vaxvms, vaxpas, unixbsd, pcturbo,
                        macturbo, pils, vaxfor, genericc);

{*****
* WARNING!!!
* Think carefully before changing the order of the following declaration!!! (AP)
******}
     options = (ccerncross, cm6809, cmonolith,
                    cvaxvms, cvaxpas, cunixbsd, cpcturbo,
                    cmacturbo, cpils, cvaxfor, cgenericc,   { See output_mode }
                scerncross, sm6809, smonolith,
                    svaxvms, svaxpas, sunixbsd, spcturbo,
                    smacturbo, spils, svaxfor, sgenericc,
                dlex, dlexhot, dtree,
                shortint, stdescr, concurrent, noautoinit,
                byvalue, timeout, version, types,
        {       add new options here }
                invalid);

        rangeoptions = ccerncross .. invalid;

        {* deb_lex      For lexical analizer                    *}
        {* deb_lex_hot  Prints each character read from input   *}
        {* deb_tree     To print trees created by parse *}

{
        cerncross       Cern Cross Software enviroment
        m6809           Omegasoft Pascal for the m6809
        monolith        Any monolithic Pascal compiler
        vaxvms          Vax/Vms pascal fortran-like
        vaxpas          Vax/Vms pascal
        unixbsd         Unix BSD enviroment
        pcturbo         IBM PC Turbo Pascal
        macturbo        Macintosh Turbo Pascal
        pils            Any standard PILS environonment
        vaxfor          VAX/FORTRAN under VMS (might be portable)
        genericc        Any generic C compiler
        shortint        forces short (16 bit) rpc_integer
        stdescr         pass strings by descriptor (not implemented yet)
        concurrent      return before processing if all parameters IN
        noautoinit      supress automatic initialisation of stubs (VMS)
        byvalue         pass all simple type parameters by value - Nici.
        timeout         specify value of timeout parameter - Nici.
        version         control stub version number handling - Nici.
        types           include user types in ".ext" file - Nici.
}

var
        {* This variable is the HEAD pointer to the TYPE's list table *}
        typeptr : ptrunamedutype;

        {* This variable is the HEAD pointer to the BLOCK's list table *}
        blockptr : ptrublockutable;

        unitname : iduname; {* Name of the unit under analysis *}

        {* Vars related to lexical analizer and tokens *}
        checksum : integer;     {* used for stub version number *}
        isublank : boolean;     {* for skipping multiple blanks *}
        chuthere : char;        {* for read-ahead in getchar(); *}
        lastcar  : char;        {* Last char read *}
        oldchar  : char;        {* Char ready for next request *}
        charuready : boolean;   {* Is a char ready to be returned? *}
        incomment : boolean;    {* We are checking for comment *}
        lastindex : integer;    {* Index to the last token read *}
        token, lastoken : tokenudescriptor;
        tokupresent : boolean;  {* If a token already present in lex-analizer *}
        lineread : integer;     {* Number of line read *}
        maxkeyword : integer;   {* Number of keyword,
                                should be the same as NUMKEYWORD *}
        keyword : array [1..numkeyword] of record
                                name : charuname;
                                symbol: typeutoken;

                        end;

        upkpck :    array[boolean] of packed array[1..3] of char; {'upk','pck'}

        simpleudescriptor: array[simpleutype] of ptrudefinedutype;
                                { Reference definitions of simple types }


{ Variables set by parser:
}
        externalumarshalling:       boolean;    { has been invoked }

        numberuofuinventedutypes:       integer;   { Used by ensure_named_type }

        {* General flag for errors *}
        errorfound : integer;

        {* File variables *}
        inpufile : text;        {* Input file *}
        inpuname : astring;     {* Input file name *}
        inpuline : pilstring;   {* Last line read *}

        opufile:        text;   {* Output file for stub &c *}

        cliuname : astring;     {* Client file name *}
        seruname : astring;     {* Server file name *}
        extuname : astring;     {* External declaration file *}

{   Variables for use by Code generator:
}
        nextulabel: integer;    {* Next label value for use in FORTRAN *}
        sizeusoufar: integer;   {* Number of bytes packed/unpacked so far *}
        fragmentationuused: boolean; {* have we resorted to fragmentation? *}

{   General options
}
        deref, drutmp : boolean;        {* dereferencing flag for C params *}
        cliuspec, seruspec : boolean;   {* forbid multiple stub generation *}
        omode, serumode, cliumode : outputumode;
        client:     boolean;            {* We are making a client stub now *}
        cmode :  set of outputumode;    {* must contain all C output modes *}
        timeoutuval : integer;          {* value of timeout, if specified  *}
        versionunum : integer;          {* stub version number, if given   *}

        {* Debugging variables *}
        runoptions : array [rangeoptions] of record
                                name : optuname;
                                value: boolean;
                        end;



{* Error management *}
{*******************************************************************
*               Top-down RPC compiler - Error management
*
*       Author:
*               Antonio Pastore, Tec. Student 1986, DD/OC, CERN

    28 May 1986     Written, AP
    23 Nov 1988     'expected' written, find_tok made less verbose. TBL

**********************************************************************}

procedure getoken; forward;
procedure backtoken; forward;

procedure error(which : errutype);
begin
write('ERROR: ');
errorfound := errorfound + 1;
if not(which in [illubasicutype..unexpueof]) then
        writeln(
'RPCC: Panic: compiler internal error, notify the compiler''s Administrator.')
else
case which of
illubasicutype  : writeln('Illegal basic type.');
invalidurange   : writeln('Upper bound must be greater then lower.');
numberureq      : writeln('Number requested.');
oftokumiss      : writeln('Keyword "of" missing.');
simtypeureq     : writeln('Simple type requested.');
positiveureq    : writeln('Index must be positive.');
identureq       : writeln('Identifier requested.');
blocksureq      : writeln('At least a procedure/function declaration requested.');
baduprocudecl   : writeln('Parameter must be simple or already declared in type section.');
twiceudeclared  : writeln('Identifier already declared.');
semicolumiss    : writeln('Semicolon ";" missing.');
semicuroundumiss: writeln('Semicolon ";" or round bracket ")" missing.');
commaumiss      : writeln('Comma "," missing.');
varuattrureq    : writeln('"in","out" or "in out" required.');
colonumiss      : writeln('Colon ":" missing.');
dotumiss        : writeln('Dot "." missing.');
idunotudeclared : writeln('Identifier not declared.');
clsroundumiss   : writeln('Round bracket ")" missing.');
opnroundumiss   : writeln('Round bracket "(" missing.');
packageumiss    : writeln('Keyword "package" missing.');
recordumiss     : writeln('Keyword "record" missing.');
returnumiss     : writeln('Keyword "return" missing.');
istokumiss      : writeln('Keyword "is" missing.');
endtokumiss     : writeln('Keyword "end" missing.');
typeumiss       : writeln('Keyword "type", "procedure" or "function" missing.');
charuignored    : writeln('Illegal character:"',lastcar,'", ignored.');
toomanyudig     : writeln('Too many digits in this number.');
toomanyudim     : writeln('Too many dimensions in this array.');
identureserved  : writeln('Identifier reserved for compiler use.');
baduname        : writeln('Identifier does not match already declared package name.');
procuexp        : writeln('Keyword "procedure" or "function" expected.');
paramsumiss     : writeln('Parameters missing.');
inputumiss      : writeln('Input file missing, can''t go on.');
baduinputuname  : writeln('Bad input file name: ".ext" extension reserved.');
cantuopnuinput  : writeln('Can''t open input file.');
cantuopnuext    : writeln('Can''t open ".ext" file.');
cantuopnuclient : writeln('Can''t open client file.');
cantuopnuserver : writeln('Can''t open server file.');
cantuclsuinput  : writeln('Can''t close input file.');
cantuclsuclient : writeln('Can''t close client file.');
cantuclsuserver : writeln('Can''t close server file.');
cantuclsuext    : writeln('Can''t close ".ext" file.');
internaluerror  : writeln('Sorry - Unexpected internal condition in compiler.');
unexpueof       : writeln('Unexpected end of input file.');
end; {End_CASE}
end; {ERROR}

{       Write a token name
        ------------------
}
procedure writeutoken(what: typeutoken);
begin
    if not(what in[chartok..nultok]) then
        write('(Unknown token!)')
    else
    case what of
        chartok :       write('RPC_CHAR');
        bytetok :       write('RPC_BYTE');
        shortok :       write('RPC_SHORT');
        integertok:     write('RPC_INTEGER');
        real32tok:      write('RPC_REAL32');
        real48tok:      write('RPC_REAL48');
        real64tok:      write('RPC_REAL64');
        real128tok:     write('RPC_REAL128');
        longtok :       write('RPC_LONG');
        arraytok:       write('ARRAY');
        sequence:       write('SEQUENCE');
        stringtok:      write('STRING');
        substring:      write('SUBSTRING');
        package :       write('PACKAGE');
        semicolon:      write('SEMICOLON');
        typetok :       write('TYPE');
        endtok  :       write('END');
        colon   :       write('colon ":"');
        comma   :       write('comma ","');
        opnround:       write('"("');
        clsround:       write('")"');
        oftok   :       write('OF');
        intok   :       write('IN');
        outok   :       write('OUT');
        inoutok :       write('INOUT');
        istok   :       write('IS');
        return  :       write('RETURN');
        dot     :       write('"."');
        ident   :       write('identifier');
        number  :       write('number');
        proctok :       write('PROCEDURE');
        functok :       write('FUNCTION ');
        nultok  :       write('(Null Token!)');
    end;
end; {write_token}

{       Print last line read from declaration file
        ------------------------------------------
On entry:
    'lastindex' is the index of the last token read.
}
procedure printulastuline;
var
   a : integer;
begin
        write('INPUT:');
        for a := 1 to inpuline.len do
                write(inpuline.str[a]);
        writeln;
        for a := 1 to lastindex+4 do
                write('-');
        write('^');
        writeln;
end;

procedure linerror(which : errutype);
begin
    writeln('RPCC: Error found on line ',lineread:3,':');
    printulastuline;
    error(which);
    writeln;
end;

procedure findutok(kind : setuofutoken);
begin
{*    writeln('RPCC: Skipping ...');        *}
    while not(token.kind in kind) do
        getoken;
{*
    print_last_line;
    writeln('RPCC: Restarting analysis here.');
    writeln;
*}
end;

procedure errfind(which : errutype; kind : setuofutoken);
begin
    linerror(which);
    findutok(kind);
end;

procedure abortupgm(which : errutype);
begin
    error(which);
    writeln('Fatal error, program aborted.');
    halt;
end;

{* Debugging module *}
{*******************************************************************
*               Top-down RPC compiler - Debugging module
*
*       Author:
*               Antonio Pastore, Tec. Student 1986, DD/OC, CERN
*
*       History:
*               28 May 1986     first written (AP)
*               27 Jun 1986     last update by AP
*               26 Aug 1987     bug fix in print_tab_types - Nici.
*
**********************************************************************}

procedure debuguprintuname(var where: text; name: iduname);
var i:  integer;
begin
  with name do
    for i := 1 to len do write(where, str[i]);
end;

procedure printok(what: typeutoken);
begin
    if not(what in[chartok..nultok]) then
        write('?????? (bad type token) ???')
    else
    case what of
        chartok :       write('RPC_CHAR');
        bytetok :       write('RPC_BYTE');
        shortok :       write('RPC_SHORT');
        integertok:     write('RPC_INTEGER');
        real32tok:      write('RPC_REAL32');
        real48tok:      write('RPC_REAL48');
        real64tok:      write('RPC_REAL64');
        real128tok:     write('RPC_REAL128');
        longtok :       write('RPC_LONG');
        arraytok:       write('ARRAY');
        sequence:       write('SEQUENCE');
        stringtok:      write('STRING');
        substring:      write('SUBSTRING');
        package :       write('PACKAGE');
        semicolon:      write('SEMICOLON');
        typetok :       write('TYPE');
        endtok  :       write('END');
        colon   :       write('COLON');
        comma   :       write('COMMA');
        opnround:       write('OPNROUND');
        clsround:       write('CLSROUND');
        oftok   :       write('OF');
        intok   :       write('IN     ');
        outok   :       write('OUT    ');
        inoutok :       write('IN OUT ');
        istok   :       write('IS');
        return  :       write('RETURN');
        dot     :       write('DOT');
        ident   :       write('IDENT');
        number  :       write('NUMBER');
        proctok :       write('PROCEDURE');
        functok :       write('FUNCTION ');
        nultok  :       write('NULTOKEN');
    end;
end; {PRINT}
{_____________________________________________________________________________

                Print out a type definition

Prints out whole lines.
}
{   indenting function }

function indent(level: integer): integer;
begin
    indent := level*4 + 16;
end;

procedure printudefutype(ptr: ptrudefinedutype; level: integer);
var
    a : integer;
    scan:   ptrunamedutype;
begin
    if ptr = nil then write('???? NIL pointer ????')
    else if not(ptr^.typubasicutype in[chartok..substring]) then
        write('???? Bad basic type ????')
    else with ptr^ do
    if ((level>1) or (typubasicutype in [chartok..longtok]))
        and (typuname <> nil) then begin
        write(' ':indent(level));           { Use predefined name if possible }
        debuguprintuname(output, typuname^.ntyuname);
        writeln(';');
    end else case typubasicutype of
        chartok,
        bytetok,
        shortok,
        integertok,
        real32tok,
        real48tok,
        real64tok,
        real128tok,
        longtok:    begin
                        write(' ':indent(level));
                        printok(typubasicutype);
                        writeln('; -- ??? unnamed basic type ???');
                    end;

        stringtok:  writeln(' ':indent(level), 'STRING (',typuhigh:1,');');

        arraytok:   begin
                        writeln(' ':indent(level), 'ARRAY (', typulow:1,
                                    '..',typuhigh:1,') OF');
                        printudefutype(typusubtype, level+1);
                    end;

        accesstok:  begin
                        writeln(' ':indent(level), 'ACCESS');
                        printudefutype(typusubtype, level+1);
                    end;
        recordtok:  begin
                        writeln(' ':indent(level), 'RECORD');
                        scan := typufields;
                        while scan<> nil do with scan^ do begin
                            write(' ':indent(level+1));
                            debuguprintuname(output, ntyuname);
                            writeln(': ');
                            printudefutype(ntyutype, level+2);
                            scan := scan^.ntyunext;
                        end {while};
                        writeln(' ':indent(level), 'END RECORD;');
                    end;

        sequence:   begin
                        writeln(' ':indent(level), 'SEQUENCE (',typuhigh:1,
                                    ') OF ');
                        printudefutype(typusubtype, level+1);
                    end;

        substring:  writeln(' ':indent(level), 'SUBSTRING (',typuhigh:1,');');

    end; {end_case}
end; {PRINT_DEF_TYPE}

procedure printutabutypes;
{***
* This procedure prints on standard output the tree of the type's declaration.
***}
var
        scan : ptrunamedutype;
        a : options;

begin
    writeln;
    writeln('--     Compiler options requested:');
    writeln;
    for a := ccerncross to invalid do
        if runoptions[a].value then
                writeln('-- Option ',runoptions[a].name,' is set.');
    writeln;

    writeln;
    writeln('--     Tree of types (alphabetical order)');
    writeln;
    scan := typeptr;
    while scan <> nil do begin
        write('TYPE ');
        debuguprintuname(output,scan^.ntyuname);
        writeln(output, ' IS ');
        printudefutype(scan^.ntyutype, 1);
        with scan^.ntyutype^ do
          if typumaxusize=typuminusize
            then writeln(' -- Representation: ', typumaxusize:1, ' bytes.')
            else writeln(' -- Representation: ',
                        typuminusize:1, ' bytes minimum, ',
                        typumaxusize:1, ' bytes maximum.');
        writeln;
        scan := scan^.ntyunext;
    end; {end_while}
end; {PRINT_TAB_TYPES}

procedure printutabublocks;
{***
* This procedure prints on standard output the tree of the block's declaration.
***}
var
        scan : ptrublockutable;

procedure printulist(ptr: ptruidlist);
begin
writeln('with parameters:');
repeat
        write('    ',ptr^.name.str,' : ');
        printok(ptr^.attr);
        if ptr^.idutype^.typuname <> nil then
                write(ptr^.idutype^.typuname^.ntyuname.str)
        else
                printudefutype(ptr^.idutype, 1);
        writeln;
        ptr := ptr^.next;
until ptr = nil;
write('(end of parameters)');
end;

begin {PRINT_TAB_BLOCKS}
    writeln;
    writeln('################## TREE OF BLOCKS #######################');
    writeln;
    writeln('PACKAGE ',unitname.str, ' IS');
    scan := blockptr;
    while scan <> nil do begin
        writeln;
        printok(scan^.butype);
        write(' ',scan^.name.str);
        if scan^.list <> nil then
                printulist(scan^.list);
        if scan^.butype = functok then begin
                write(' RETURN ');
                printok(scan^.return);
        end;
        writeln;
        writeln('--  Minimum ', scan^.blkuminuin:1, ' bytes in, ',
                                scan^.blkuminuout:1,  ' out.');
        writeln('--  Maximum ', scan^.blkumaxuin:1, ' bytes in, ',
                                scan^.blkumaxuout:1,  ' out.');
        writeln('-- _______________________________________________________');
        scan := scan^.next;
    end; {end_while}
    writeln('END ',unitname.str, ';');
    writeln;
end;

procedure printutoken;
begin
write('     Token read: ');
if token.kind = ident then
        write('IDENT:"',token.name.str,'"')
else
        if token.kind = number then
                write('NUMBER: ',token.value)
        else
                printok(token.kind);
writeln;
end; {PRINT_TOKEN}


{* Global initialization *}
{*******************************************************************
*               Top-down RPC compiler - Initialization code
*
*       Author:
*               Antonio Pastore, Tec. Student 1986, DD/OC, CERN
*

History:
        28 May 86       Written, Antonio Pastore, Technical student, DD/OC
        11 Jul 86       Last update by antonio
         2 Nov 86       Extra options added (TBL)
        10 Aug 87       genericC options added; structural changes - Nici.
        17 Aug 87       byvalue option added - Nici.
        26 Aug 87       Options timeout and version added - Nici.
         1 Sep 87       Options types and <s,c>pcturbo added - Nici.
        26 Aug 88       Options <s,c>macturbo added. Roberto Bagnara, DD/OC
        23 Oct 88       RECORD and ACCESS added - Tim BL
         1 Nov 88       PILS output options added (Louis Tremblet)

**********************************************************************}

procedure getokenuinit;

procedure putukeyword(name: charuname; tok: typeutoken);
begin
        maxkeyword := maxkeyword + 1;
        keyword[maxkeyword].name    := name;
        keyword[maxkeyword].symbol  := tok;
end;

begin {GETOKEN_INIT}
        {* Vars in getoken *}
        inpuline.len := 0;
        inpuline.start := 0;
        lastindex := 1;
        lineread := 0;
        tokupresent := false;
        lastcar := ' ';
        charuready := false;
        incomment := false;

        {* The following MUST be in alphabetical order !!! *}

        maxkeyword := 0;
        putukeyword('access                   ', accesstok);
        putukeyword('array                    ', arraytok);
        putukeyword('end                      ', endtok);
        putukeyword('function                 ', functok);
        putukeyword('in                       ', intok);
        putukeyword('inout                    ', inoutok);
        putukeyword('is                       ', istok);
        putukeyword('of                       ', oftok);
        putukeyword('out                      ', outok);
        putukeyword('package                  ', package);
        putukeyword('pragma                   ', pragmatok);
        putukeyword('procedure                ', proctok);
        putukeyword('record                   ', recordtok);
        putukeyword('return                   ', return);
        putukeyword('rpc_byte                 ', bytetok);
        putukeyword('rpc_char                 ', chartok);
        putukeyword('rpc_integer              ', integertok);
        putukeyword('rpc_long                 ', longtok);
        putukeyword('rpc_real128              ', real128tok);
        putukeyword('rpc_real32               ', real32tok);
        putukeyword('rpc_real48               ', real48tok);
        putukeyword('rpc_real64               ', real64tok);
        putukeyword('rpc_short                ', shortok);
        putukeyword('sequence                 ', sequence);
        putukeyword('string                   ', stringtok);
        putukeyword('substring                ', substring);
        putukeyword('type                     ', typetok);

        upkpck[false] := 'upk';     { Part of output of code generator }
        upkpck[true]  := 'pck';
end;

procedure inpuoptions;
var     opt: options;
begin
        for opt := ccerncross to invalid do
                runoptions[opt].value:= false;

        runoptions[cmonolith].name      := 'cmonolith ';
        runoptions[ccerncross].name     := 'ccerncross';
        runoptions[cm6809].name         := 'cm6809    ';
        runoptions[cvaxvms].name        := 'cvaxvms   ';
        runoptions[cvaxpas].name        := 'cvaxpas   ';
        runoptions[cunixbsd].name       := 'cunixbsd  ';
        runoptions[cpcturbo].name       := 'cpcturbo  ';
        runoptions[cmacturbo].name      := 'cmacturbo ';
        runoptions[cpils].name          := 'cpils     ';
        runoptions[cvaxfor].name        := 'cfortran  ';
        runoptions[cgenericc].name      := 'cgenericc ';
        runoptions[smonolith].name      := 'smonolith ';
        runoptions[scerncross].name     := 'scerncross';
        runoptions[sm6809].name         := 'sm6809    ';
        runoptions[svaxvms].name        := 'svaxvms   ';
        runoptions[svaxpas].name        := 'svaxpas   ';
        runoptions[sunixbsd].name       := 'sunixbsd  ';
        runoptions[spcturbo].name       := 'spcturbo  ';
        runoptions[smacturbo].name      := 'smacturbo ';
        runoptions[spils].name          := 'spils     ';
        runoptions[svaxfor].name        := 'sfortran  ';
        runoptions[sgenericc].name      := 'sgenericc ';
        runoptions[dlex].name           := 'dlex      ';
        runoptions[dlexhot].name        := 'dlexhot   ';
        runoptions[dtree].name          := 'dtree     ';
        runoptions[shortint].name       := 'shortint  ';
        runoptions[stdescr].name        := 'stdescr   ';
        runoptions[concurrent].name     := 'concurrent';
        runoptions[noautoinit].name     := 'noautoinit';
        runoptions[byvalue].name        := 'byvalue   ';
        runoptions[timeout].name        := 'timeout   ';
        runoptions[version].name        := 'version   ';
        runoptions[types].name          := 'types     ';

        cmode := [genericc];
        seruname.len := 0;
        serumode := cerncross;
        seruspec := false;
        cliuname.len := 0;
        cliumode := cerncross;
        cliuspec := false;

        checksum := 0;
        chuthere := ' ';
        isublank := true;
end;

procedure pointersuinit;
begin
        typeptr  := nil;
        blockptr := nil;
end;

procedure inituglobal;
begin
        getokenuinit;
        inpuoptions;
        pointersuinit;
        errorfound := 0;
end;


{* Input parameters management *}
{*******************************************************************
*               Top-down RPC compiler - Parameters management
*
*       Author:
*               Antonio Pastore, Tec. Student 1986, DD/OC, CERN
*
        28 May 1986     Written (Antonio)
        27 Jun 1986     Last update by Antonio
        12 Feb 87       No longer overwrites existing files: makes new version
        10 Aug 87       get_argv (UNIXBSD): string length bug fixed;
                        options "[s,c]<mode>=<filename>" implemented;
                        bug fix: disallow problematic MAGIC characters;
                        catch multiple server/client specifications - Nici.
        26 Aug 87       Options timeout and version added - Nici.
        20 Mar 88       For loop vaiable was used after loop in Unix file open
                        Subsctring function now used in VMS file open.
        15 Dec 88       Bug fix: filename with 10 char long option was ignored

**********************************************************************}



{       M A C H I N E           D E P E N D A N T       R O U T I N E S
        =============           =================       ===============

These are:

        get_argc(count)                 Return number of arguments
        get_argv(index, str, len)       Return one argument as a string
        file_open(file, name, mode)     Open a file
        file-close(file)                Close a file
}


procedure getuargc(var count: integer);
begin
        count := argc - 1; {* Predefined identifier *}
end;

procedure getuargv(what: integer; var towhere: longstring; var len : integer);
var
        loop : boolean;
begin
        argv(what, towhere);
        {*
        * Remember:
        * command line parameters are truncated or padded with blanks!!
        *}
        len := 0;
        loop := true;
        while (loop) and  (len <= maxstring) do
                if towhere[len+1] <> ' ' then
                        len := len + 1
                else
                        loop := false;
end;

function fileuopen(var  fileudes: text;
                   var  name: astring;
                        mode: openumode) : boolean;
{**
* Try to open a file with 'name' name, returns true if succesful,
* false otherwise.
**}
var
        a : integer;
        str : longstring;
begin
        for a := 1 to name.len do
                str[a] := name.str[a];

        a := name.len;                                          { 80320 }
        if a < maxstring then
                a := a + 1;
        str[a] := chr(0);

        if mode = rewriting then
                rewrite(fileudes, str)
        else
                reset(fileudes, str);
        fileuopen := true;
end;

function fileuclose(var fileudes : text): boolean;
begin
        flush(fileudes);
        fileuclose := true;
end;



{       E N D           O F     M A C H I N E   D E P E N D A N T       B I T
******************************************************************************
}

{       Analize an option:
        -----------------

If interactive skip MAGIC and don't set input file name.
The spelling of the name of this procedure is historical.

}
procedure analize(var arg: longstring; len: integer; interactive: boolean);

const

        magic1 = '-';   { ugly! This disallows magic charcter '-' for }

        magic2 = '-';

var
        locuarg :       optuname;
        a, i :          integer;
        found :         boolean;
        scan :          options;
        present:        boolean;        { Parameter present }

begin {ANALIZE}

{ Options are at most 10 characters long, but may be followed by an '='
   sign which must be scanned and may be in position 11.
}
    if (arg[1] = magic1) or (arg[1] = magic2) or (interactive)
    then begin {* Option *}
        locuarg := '          ';
        a := 1;
        present := false;
        while (a <= 11) and (a < len) and not present do begin
            if arg[a+1] = '=' then present := true
            else if a<=10 then begin
                if arg[a+1] in['A'..'Z'] then
                        locuarg[a] := chr(ord('a') + ord(arg[a+1]) - ord('A'))
                else
                        locuarg[a] := arg[a+1];
            end;
            a := a + 1;
        end;

        {* Find options *}
        scan := ccerncross;
        found := false;
        {* if present then a := a + 1;   skip over "="    out 90926 *}
        repeat
            if locuarg = runoptions[scan].name then
            begin
                found := true;
                if ord(scan) < numomodes then
                begin            {client specification}
                    if cliuspec then
                    begin
                        write('RPCC: ');
                        for a := 1 to len do write(arg[a]);
                        writeln(': client already specified - option ignored.')
                    end
                    else begin
                        cliuspec := true;
                        for i := 1 to ord(scan) do cliumode := succ(cliumode);
                        if present then
                        begin
                            cliuname.len := len - a;
                            for i := 1 to len - a do
                                cliuname.str[i] := arg[a+i]
                        end
                    end
                end
                else if ord(scan) < 2*numomodes then
                begin                 {server specification}
                    if seruspec then
                    begin
                        write('RPCC: ');
                        for a := 1 to len do write(arg[a]);
                        writeln(': server already specified - option ignored.')
                    end
                    else begin
                        seruspec := true;
                        for i := 1 to ord(scan) - numomodes do
                            serumode := succ(serumode);
                        if present then
                        begin
                            seruname.len := len - a;
                            for i := 1 to len - a do
                                seruname.str[i] := arg[a+i]
                        end
                    end
                end
                else begin      {a REAL option}
                    runoptions[scan].value := true;
                    if scan = timeout then
                    begin
                        if a >= len then
                        begin
                            writeln('RPCC: timeout value missing, default will be used.');
                            runoptions[timeout].value := false;
                        end
                        else begin
                            timeoutuval := 0;
                            for i := 1 to len - a do
                                if arg[a+i] in ['0'..'9']
                                    then timeoutuval := 10*timeoutuval + ord(arg[a+i]) - ord('0')
                                    else begin
                                        write('RPCC timeout option: ');
                                        abortupgm(numberureq);
                                    end;
                             if timeoutuval = 0 then writeln('RPCC: timeout is zero, hope that''s okay.');
                        end;
                    end
                    else if scan = version then
                    begin
                        if a >= len then
                        begin
                            writeln('RPCC: supplying missing version number.');
                            runoptions[version].value := false;
                        end
                        else begin
                            versionunum := 0;
                            for i := 1 to len - a do
                                if arg[a+i] in ['0'..'9']
                                    then versionunum := 10*versionunum + ord(arg[a+i]) - ord('0')
                                    else begin
                                        write('RPCC version option: ');
                                        abortupgm(numberureq);
                                    end;
                             if versionunum = 0 then writeln('RPCC: stub version check disabled.');
                        end;
                    end;
                end
            end else
                scan := succ(scan)
        until found or (scan = invalid);

        if scan = invalid then begin
                write('RPCC: ');
                for a := 1 to len do
                        write(arg[a]);
                        writeln(': unrecognized option, ignored.')
        end;

    end {* Options *} else begin {* File name *}

        inpuname.len := len;
        for a := 1 to len do
                inpuname.str[a] := arg[a];
    end;

end; {ANALIZE}

{   Make a filename out of the input file name and a prefix
    -------------------------------------------------------

    The prefix is put on the front of the name part of the filename.
This is taken to be the last bit which contains only alphanumerics and
dots and underscores. Other characters are assumed to be directory
information, which must be left intact.

}
procedure makeufilename(var filename: astring;
                            prefix:     char3);

var i, rdp:     integer;    { read pointer }
    done:       boolean;
begin
    rdp := inpuname.len;    { point to last char }
    done := false;  { Find start of name - a horrible Pascal loop }
    repeat          { Oh, to be programming in C at this point! }
        if (rdp>=1)
        then begin
            if (inpuname.str[rdp] in ['.', '0'..'9', 'A'..'Z', '_', 'a'..'z'])
            then rdp:=rdp-1
            else done := true;
        end else
            done := true;
    until done;

    { rdp is the number of characters of directory etc preceding the filename}

    filename.len := 0;
    for i := 1 to rdp do begin
        filename.len := filename.len + 1;
        filename.str[filename.len] := inpuname.str[i];
    end;

    for i := 1 to 3 do begin
        filename.len := filename.len + 1;
        filename.str[filename.len] := prefix[i];
    end;

    for i := rdp+1 to inpuname.len do begin
        filename.len := filename.len + 1;
        filename.str[filename.len] := inpuname.str[i];
    end;

end;


procedure getuparameters;
{****
* On entry: parameters on the command line
* On exit:
* 'inp_name' filled with input file name;
* 'inp_file' opened using 'inp_name' name;
* 'cli_name' filled with client file name; ('cli'+inp_name)
* 'ser_name' filled with server file name; ('ser'+inp_name)
* 'ext_name' filled with the external file name; (inp_name.'ext')
******}
var
        a, b, count, puargc, pulen : integer;
        puargv : longstring;
        ext : packed array[1..3] of char;       { TBL 14-8-86 }

begin {GET_PARAMETERS}
inpuname.len := 0;

if commandline then begin
        count := 1;
        getuargc(puargc);
        while count <= puargc do begin
                getuargv(count, puargv, pulen);
                count := count + 1;
                {* Analize option *}
                analize(puargv, pulen, false);
        end; {end_while}
end else begin
        {* Get file name *}
        write('Input file name: ');
        with inpuname do begin
        len := 0;
        while not(eoln(input))  and (len <= maxstring) do begin
                len := len + 1;
                read(str[len]);
        end;
        {* Skip end of line *}
        readln;
        end;
        {* Get options *}
        repeat
                write('Debugging options: ');
                count := 1;
                while not(eoln(input)) and (count < maxstring) do begin
                        count := count + 1;
                        read(puargv[count]);
                end;
                readln;
                {* Analize option *}
                if count > 1 then
                        analize(puargv, count, true);
        until count = 1;
end;

if inpuname.len = 0 then begin
        writeln('RPCC: usage: source_file [options]');
        abortupgm(inputumiss);
end;

{* Try to open input file *}
if not(fileuopen(inpufile, inpuname, resetting)) then
        abortupgm(cantuopnuinput);

{* Build client, server and external default file names *}

if cliuname.len = 0 then makeufilename(cliuname, 'cli');

if seruname.len = 0 then makeufilename(seruname, 'ser');

{   Make the external declaration file name:

    Th extension is changed to '.ext'
}
    b := inpuname.len;
    while (b>=1)
    and {then} (inpuname.str[b] in ['A'..'Z', 'a'..'z'])
        do b := b-1;
    if b=0 then b:= inpuname.len;   { No '.' found }
    if inpuname.str[b]<> '.' then b:= inpuname.len; { Found directory }

    for a := 1 to b  do begin
        if a <= maxstring - 3 then
                extuname.str[a] := inpuname.str[a];
    end;

    if (b <= inpuname.len - 3) then begin       { extension was specified }
        ext[1] := inpuname.str[b+1];
        ext[2] := inpuname.str[b+2];
        ext[3] := inpuname.str[b+3];
        if (ext = 'ext') or (ext = 'EXT') then
                abortupgm(baduinputuname);
    end else begin
        b := b+1;                                { Add a '.' if it wasn't }
        extuname.str[b] := '.';
    end{if};

    extuname.str[b + 1] := 'e';
    extuname.str[b + 2] := 'x';
    extuname.str[b + 3] := 't';
    extuname.len := b + 3;
end; {GET_PARAMETERS}

{* Getoken, lexical analizer *}
{*******************************************************************
*               Top-down RPC compiler - Lexical analizer
*
*       Author:
*               Antonio Pastore, Tec. Student 1986, DD/OC, CERN
*
*       History:
*               28 May 1986     first written (AP)
*               27 Jun 1986     last update by AP
*               26 Aug 1987     checksum (for version) included - Nici.
*               16 Nov 1988     Bug fix: crashed with 26 char identifier (TBL)
*
**********************************************************************}

{GETOKEN returns in TOKEN (globally declared) the last token read}

procedure getoken;
var
        i, j, k, sign : integer;

{**
* Note: 'lastcar' is declared global to be printed in error messages.
**}

function apugetchar : char;     {* renamed - Nici, 26 Aug 87 *}
const
        tab = 9; {* ASCII TAB character *}
var
        a : integer;
        localcar : char;

begin {AP_GETCHAR}
if charuready then begin
        apugetchar := oldchar;
        charuready := false;
end else begin

        with inpuline do begin

        while (start > len) or (len = 0) do begin
                len := 0;
                start := 1;
                lastindex := 1;
                if eof(inpufile) then
                        abortupgm(unexpueof)
                else begin
                        while not(eoln(inpufile)) do begin
                                len := len + 1;
                                read(inpufile,str[len]);
                                if ord(str[len]) = tab then
                                        str[len] := ' ' {* Convert TAB to space *}
                                else
                                        if ord(str[len]) < 32 then
                                                len := len - 1;
                        end;
                        len := len + 1;
                        str[len] := ' ';
                        readln(inpufile); {* Skip END_OF_LINE *}
                        lineread := lineread + 1; {* Line counter *}
                        if runoptions[dlex].value then begin
                                write('LINE READ: ');
                                for a := 1 to len do
                                        write(str[a]);
                                writeln;
                        end;
                end;
        end;

        localcar := str[start];
        start := start + 1;

        end; {END_WITH}

        {* Make it lower *}
        if localcar in ['A'..'Z'] then
                localcar := chr(ord('a') + ord(localcar) - ord('A'));
        apugetchar := localcar;

        {* Skip comment if any *}
        if (localcar = '-') and not(incomment) then begin
                incomment := true;
                oldchar := apugetchar;
                incomment := false;
                if oldchar = '-' then begin
                        inpuline.len := 0; {* Force the reading of a new line *}
                        apugetchar := ' '; {* Return space *}
                end else begin
                        apugetchar := localcar;
                        charuready := true;
                end;
        end;
end; {END_ELSE char_ready}
end; {AP_GETCHAR}


function getchar : char;              {* written - Nici, 26 Aug 87 *}

{*  This rather complex interface ensures that blanks are skipped  *}
{*  whenever they are not syntactically relevant, in order to make *}
{*  the checksum of syntactically identical RPCL files invariant.  *}

    var ch : char;

    begin
        if chuthere <> ' ' then
        begin
            ch := chuthere;
            chuthere := ' ';
        end
        else begin
            ch := apugetchar;
            if ch = ' ' then
            begin
                if not eof(inpufile) then
                        repeat ch := apugetchar until ch <> ' ';
                if not ((ch in [':', '(', ')', ',', ';']) or isublank) then
                begin
                    chuthere := ch;
                    ch := ' ';
                end;
            end;
        end;
        isublank := ch in [' ', ':', '(', ')', ',', ';'];
        checksum := (3*checksum + ord(ch) - ord(' ')) mod 8999;
        if runoptions[dlexhot].value then
            if ch = ' ' then writeln(ch)
                        else write(ch);
        getchar := ch;
    end;
{
_____________________________________________________________________________
}
begin {GETOKEN}
    if tokupresent then begin
        token := lastoken;
        tokupresent := false;
    end else begin
        with token do begin

        repeat  {* Find a valid token *}

            if lastcar = ' ' then lastcar := getchar;       {* Skip blank *}
            lastindex := inpuline.start;       {* To print error messages *}

{* Check if identifier or keyword *}
            if lastcar in ['a'..'z', '_'] then begin
        {* Suppose to be an identifier or keyword *}
                kind := ident;
                name.len := 0;
                repeat
                    if name.len < maxidlen then begin   { was <= 81116 TBL }
                        name.len := name.len + 1;
                        name.str[name.len] := lastcar;
                    end;
                    lastcar := getchar;
                until not(lastcar in['a'..'z','_','0'..'9']);
        {* Fill with blanks *}
                for i := name.len+1 to maxidlen do
                    name.str[i] := ' ';

        {* Try to see if keyword *}
        {* The following is the famous binary search!!! *}
                i := 1; j := maxkeyword;
                repeat
                    k:= (i+j) div 2;
                    if name.str <= keyword[k].name then j:=k-1;
                    if name.str >= keyword[k].name then i:=k+1;
                until i>j;

                if i-1 > j then begin
                    kind := keyword[k].symbol;
                end else begin
             {* Check if illegal identifier, reserved for compiler use *}
                    with name do if
                    ((str[1]='r')and (str[2]='p')
                        and (str[3]='c') and (str[4]='_'))
                    or (name.str = 'b                        ')
                    or (name.str = 'header                   ')
                    or (name.str = 'ret                      ') then
                        linerror(identureserved);
                    kind := ident;
                end;
            end {identifier or keyword}

            else
{* Check if number *}
            if lastcar in ['0'..'9','-'] then begin
                kind := number;
                value := 0;
                if lastcar = '-' then begin
                    sign := -1;
                    lastcar := getchar;
                end else
                    sign := 1;
                repeat
                    if value < (maxint div 10) - (ord(lastcar) - ord('0')) then
                        value := value * 10 + ord(lastcar) - ord('0')
                    else begin
                        error(toomanyudig);
                        value := maxint;
                    end;
                    lastcar := getchar;
                until not(lastcar in ['0'..'9']);
                value := value * sign;
            end {if number} else begin

{* Then should be one of these symbols! *}
                if not(lastcar in[':', ';', '=', ',', '(', ')', '.']) then begin
                    kind:=nultok;
                    error(charuignored);        {* Ignore this character *}
                    end else begin
                        case lastcar of
        ':' : kind := colon;
        ';' : kind := semicolon;
        ',' : kind := comma;
        '(' : kind := opnround;
        ')' : kind := clsround;
        '.' : kind := dot;
                        end; {end case}
                    end; {if}
                    lastcar := getchar; {* Read next char *}
                end {if};

            until kind <> nultok;

        end; {END_WITH}
    end; {end_token already present}

        if runoptions[dlex].value then
            printutoken;
end; {getoken}
{
_____________________________________________________________________________
}

procedure backtoken;
{***
* This procedure save the value of 'tok' and uses it when next 'getoken' is
* called.
****}
begin
    lastoken := token;
    tokupresent := true;
end;


{* Parser *}
{ Top-down RPC compiler                                                PARSE.PAS

                 PARSER, types and blocks declaration
                 ====================================

History:
        28 May 86       Written, Antonio Pastore, Tec. Student '86, DD/OC, CERN
        30 Jun 86       Last change by Antonio
        18 Aug 86       Bug fix in block_declare (TBL) (block.list not initzd).
        31 Oct 86       Direction defaults to IN (TBL)
        20 Oct 88       Nested complex types (TBL)
        23 Nov 88       PRAGMAs timeout, external_marshalling introduced (TBL)
        30 Mar 89       PRAGMA CAST introducted (TBL)
        10 Oct 89       Bug fix: Looped on PRAGMA CAST(<undefined procedure>);



                    General useful routines
                    -----------------------


    Check for a given token, otherwise skip                             expected
    ---------------------------------------
}

function expected(what: typeutoken; skipuuntil: setuofutoken): boolean;
begin
  getoken;
  if token.kind=what then expected := true
  else begin
    expected := false;
    write  ('RPCC: Error: Found ');
    writeutoken(token.kind);
    write  (' where ');
    writeutoken(what);
    writeln(' was expected on line ',lineread:1,':');
    printulastuline;
    writeln;
    findutok(skipuuntil);
  end {if};
end;

{_______________________________________________________________________________

                Expression handling                                     append_1
                ===================

   An expression is a variable length string generated by the compiler.
   The following routines add bits on to an existing expression to
   make it more complicated:

        Append one character to an expression:
        -------------------------------------
}
procedure appendu1(var expr: expression; ch: char);
begin
   with expr do begin
        len := len+1;
        str[len] := ch;
   end;
end;

{       Format an expression according to a template                      format
        --------------------------------------------

    The template is a 10-character character array which contains a
    dollar sign wherever the original value of the expression is to be
    put.
}
procedure format(   var final:      expression;
                        template:   templateutype);
{
On entry,
    template must not be blank.
On exit,
    expression copied from template with original expression inserted
        in place of "$" signs.
}
var initial:    expression;
    e:          integer;    { effective end of template string}
    i, j:       integer;    { loop index }
begin
    initial := final;
    final.len := 0;
    e := templateulength;
    while template[e] = ' ' do e:=e-1;      { strip spaces }
    for i := 1 to e do begin
        if template[i] = '$' then
            for j := 1 to initial.len do appendu1(final, initial.str[j])
        else
            appendu1(final, template[i]);
    end {for};
end {format};

{       Put one character on the front of an expression                prepend_1
        -----------------------------------------------
}
procedure prependu1(var expr: expression; ch: char);
var     a:  integer;
begin
   with expr do begin
        for a := len downto 1 do str[a+1]:=str[a];
        str[1] := ch;
        len := len+1;
   end;
end;


{       Append an identifier to an expression                        append_name
        -------------------------------------
}
procedure appenduname(var expr: expression; name: iduname);
var     a: integer;
begin
  with name do
    for a := 1 to len do appendu1(expr, str[a]);
end;

{       Append an index to an array expression                      append_index
        --------------------------------------

    The `level' parameter determines the name of the index to be used.

    1 -> rpc_a, 2 -> rpc_b etc.
}
procedure appenduindex(var expr: expression; level: integer);
var ch: char;
begin
            format(expr, '$[rpc_    ');
            appendu1(expr, chr(ord('a')+level-1));
            appendu1(expr, ']');

end;

{       Append a decimal number to an expression                  append_decimal
        ----------------------------------------
}
procedure appendudecimal(var expr: expression; x: integer);
var     t, y, weight:  integer;
begin
    if x<0 then begin
        appendu1(expr,'-');
        t := -x
    end else t := x;

    weight := 1;                { these are array bounds }
    while weight*10 <= t do weight := weight*10;
                { weight is largest power of 10 not greater than t }
    while weight>0 do begin
        appendu1(expr, chr(ord('0')+t div weight));
        t := t mod weight;
        weight := weight div 10;
    end {while};
end;

{       Find Type in List                                              find_type
        -----------------

The TYPE table used to be in alphabetical order, to halve search time. However,
now it is kept in the order or original declaration, as we allow types to be
defined in terms of other types. When the type table is repoduced in C or
Pascal, we don't want any forward references.

This routine not only searches for existing types, but also is used to find
the previous element ("father"), when inserting into a list.

On entry
    head        is the head pointer of the list to be searched, in which
                the elements are in order.
    name        is the name to be searched for.

On exit,
    father      is set to the father of this element if any, otherwise
                will point to the last element in the list.
    son         is a pointer to that element if found, NIL otherwise.
    returns true if element found else false.

 Parameter 'name' is 'var' to speed up the processing.

}
function findutype(         head:       ptrunamedutype; { head of list}
                        var name:       iduname;
                        var father: ptrunamedutype;
                        var son   : ptrunamedutype): boolean;

var
        found, doexit : boolean;
begin
found := false;
son := head;                            {* Head of the list *}
father := son;
while (son <> nil) and not(found) do begin
        if son^.ntyuname.str = name.str then
                found := true
        else begin
                father := son;
                son := son^.ntyunext;
        end;
end;
findutype := found;
end;


{       Insert Type Identifier into list                       insert_named_type
        --------------------------------

On entry,
    ptr     points to a description of the named type.
    head    is the pointer to a list of named types.
On exit,
    The element pointed to has been put into the list.
}
procedure insertunamedutype(    ptr:    ptrunamedutype;
                            var head:   ptrunamedutype);

var     son,father:    ptrunamedutype;
begin
        if findutype(head, ptr^.ntyuname, father, son)
        then begin
           linerror(twiceudeclared);    {* Identifier already declared *}
        end else begin
                if head = nil then begin {* List empty *}
                        head := ptr;
                        ptr^.ntyunext := nil;
                end else
                        if head = son then begin {* Insert ahead *}
                                head := ptr;
                                ptr^.ntyunext := son;
                        end else begin
                                father^.ntyunext := ptr;
                                ptr^.ntyunext := son;
                        end;
        end;

end {insert_named_type};

{       Find Procedure or Function name                               find_block
        -------------------------------

 This functions looks for 'name' in the table of blocks, then returns a pointer
 to that element if found, NIL otherwise. 'Father' will be set to the father of
 this element if any, otherwise will point to the last element in the list.
 Anyway should be good enough to insert elements in queue. Input is 'var' to
 speed up the processing.

}
function findublock(    var name: iduname;
                        var father: ptrublockutable;
                        var son: ptrublockutable): boolean;
var
        found : boolean;
begin
    found := false;
    son := blockptr; {* Head of the list *}
    father := son;
    while (son <> nil) and not(found) do begin
        if son^.name.str = name.str then
                found := true
        else begin
                father := son;
                son := son^.next;
        end;
    end {while};
    findublock := found;
end;


{       Find parameter                                                find_param
        --------------
On exit,
    son             pointer to param if found, else NIL
    return value    TRUE if found else FALSE
}
function finduparam(    var name:   iduname;
                            head:   ptruidlist;
                        var son:    ptruidlist): boolean;
var found: boolean;
begin
    son := head;
    found := false;
    while (son<>nil) and not found do begin
        if son^.name.str = name.str
            then found := true
            else son := son^.next;
    end {while};
    finduparam := found;
end;

{               TYPE DECLARATION ANALIZER
                =========================

 The following routines analize the TYPE declaration and create
 the proper structure to handle them later.


        Ensure Type Named                                      ensure_type_named
        -----------------

 If the given type is not named, then this procedure will
 invent an arbitrary name for it, of the form

        rpc_stype_nnn

  where nnn is an incrementing decimal number.
}
procedure ensureutypeunamed(pt: ptrudefinedutype);

var i:          integer;        { String index }
    expr:       expression;     { for building up the variable name }
    pnt:        ptrunamedutype; { Pointer to the new named type if any }

begin
    if pt <> nil then with pt^ do
     if typuname = nil then begin
        new(pnt);
        with pnt^ do begin

            format(expr, 'rpc_stype_');
            appendudecimal(expr, numberuofuinventedutypes);
            numberuofuinventedutypes := numberuofuinventedutypes + 1;

            for i := 1 to expr.len do ntyuname.str[i] := expr.str[i];
            ntyuname.len := expr.len;
            ntyutype := pt;                     { Link name to type }
            typuname := pnt;                    { Link type to name }
            insertunamedutype(pnt, typeptr);    { Put into the type list }

        end {with};
     end {if with if};
end;

{       Parse Type                                                    parse_type
        ----------

 In this function we analize a TYPE definition and return a pointer to
 a type descriptor tree.

 Returns:
    Returns a pointer to a new created element otherwise nil if error
 Error recovery:
    If error found we exit with the offending token in 'backtoken'.
}

function parseutype: ptrudefinedutype;
var
      pt:                       ptrudefinedutype;
      pnt:                      ptrunamedutype;
      locuerr:                  boolean;
      father, son:              ptrunamedutype;


{       Parse:   ( <positive integer> )                                 get_size
        -----

    Error recovery:
        If error found we suppose token forgotten so we 'backtoken'.
}
    procedure getusize(pt: ptrudefinedutype);
    begin
        with pt^ do begin
            getoken;
            if token.kind <> opnround then
                    linerror(opnroundumiss)
            else
                    getoken;
            typulow := 1;                               { implied lower bound }
            if token.kind <> number then begin
                    linerror(numberureq); {* Assume zero *}
                    typuhigh := 0;
            end else begin
                    typuhigh := token.value;
                    getoken;
            end;
            if token.kind <> clsround then begin
                    errfind(clsroundumiss, [oftok, semicolon]);
                    backtoken;
            end;

            if typuhigh < 0 then
                    linerror(positiveureq);
        end {with};
    end; {GET_SIZE}

{       Parse Sub-array                                          parse_sub_array
        ---------------

                subarray    ::  <number> .. <number> ] of <type>
                            |   <number> .. <number> , <subarray>

 Sub-array is not really a good word for this - it's the tail of an
 array definition, which can recursively call itself. These sort of things
 come out of pure BNF definitions, and avoid loops, even if they contain
 unnested brackets!

 On entry,
    First number token not read.
 On exit,
    token read/not read as after parse_type.
    value points to type descriptor filled in (except possibly for name)

Note that a subarray is the only thing which can have an unnamed subtype.
}
    function parseusubarray: ptrudefinedutype;
    var pt:     ptrudefinedutype;
    begin
        new(pt);
        with pt^ do begin
            typuname := nil;
            typuexternal := false;
            typusubtype := nil;         { Until filled in with good value }
            typubasicutype := arraytok; { The type is "array" }
            getoken;

            if token.kind <> number then begin
                    linerror(numberureq); {* Assume zero *}
                    typulow := 0;
            end else begin
                    typulow := token.value;
                    getoken;
            end;
            {* Skip double point *}
            if token.kind = dot then begin
                    getoken;
                    if token.kind <> dot then
                            linerror(dotumiss)
                    else
                            getoken;
            end else
                    linerror(dotumiss);

            if token.kind <> number then begin
                    linerror(numberureq); {* Assume low + 1 *}
                    typuhigh := typulow + 1;
            end else begin
                    typuhigh := token.value;
                    getoken;
            end;
            if typuhigh <=      typulow then
                    linerror(invalidurange);


            if token.kind = comma then begin
                typusubtype := parseusubarray;          { , <subarray> }
            end else if token.kind = clsround then begin
                getoken;
                if token.kind = oftok then begin
                    getoken;
                    typusubtype := parseutype;          { ) of <type> }
                    {** ensure_type_named(typ_subtype); *NO* *}
                end else begin
                    errfind(clsroundumiss, [semicolon]);        { ) garbage }
                    backtoken;
                end {if};
            end else begin
                    errfind(clsroundumiss, [oftok, semicolon]);
                    backtoken;
            end {if};

            if typusubtype = nil then begin
                dispose(pt);    { Not essential -could waste it }
                parseusubarray := nil;
            end else begin
                typuminusize := (typuhigh-typulow+1) * typusubtype^.typuminusize;
                typumaxusize := (typuhigh-typulow+1) * typusubtype^.typumaxusize;
                typunesting := typusubtype^.typunesting + 1;
                parseusubarray := pt;
            end {if};
        end {with};
    end; {parse_subarray}

{       Parse Type definition (main block)                            parse_type
        ---------------------                                         main block

                <type>  ::      <simple type>
                        |       SEQUENCE ( <n> ) OF <type>
                        |       ARRAY ( <n> .. <n> ) OF <type>
                        |       STRING ( <n> )
                        |       SUBSTRING ( <n> )
                        |       RECORD [ <id> : <type> ; ]*    END RECORD
                        |       <id>
 On entry,
    TOKEN ALREADY READ
 On exit,
    Next token (semicolon) not read
    Return value points to filled-in type descriptor, or NIL if error
}
begin {parse_type}
    if token.kind = ident then begin        { <id> }
        if findutype(typeptr, token.name, father, son)
        then parseutype := son^.ntyutype
        else linerror(idunotudeclared);

    end else if token.kind in[chartok..longtok] then begin { Simple types }
        parseutype := simpleudescriptor[token.kind];

    end else if token.kind = arraytok then begin            { ARRAY ( <subarray> }
        getoken;
        if token.kind <> opnround then begin
            linerror(opnroundumiss);
            backtoken;
            parseutype := nil;
        end else
            parseutype := parseusubarray;

    end else if token.kind = accesstok then begin        { ACCESS <type> }
        new(pt);
        with pt^ do begin
            typubasicutype := token.kind;
            typuexternal := false;
            getoken;
            typusubtype := parseutype;
            ensureutypeunamed(typusubtype);
            if typusubtype <> nil then begin
                typunesting := typusubtype^.typunesting;  { No extra index }
                typuminusize := 4;  { <tbd> }
                typumaxusize := typusubtype^.typumaxusize+4;  { <tbd> }
            end {if};
        end;
        parseutype := pt;

    end else if token.kind = recordtok then begin       { R E C O R D }

        new(pt);
        parseutype := pt;
        with pt^ do begin
          typubasicutype := token.kind;
          typuexternal := false;
          typusubtype := nil;       { subtype is meaningless }
          typufields := nil;        { List of fields currently empty }
          typunesting := 0;                 { Unless we find nesting in a subtype }
          typuminusize := 0;        { Initialise total }
          typumaxusize := 0;        { Initialise total }

          getoken;
          while token.kind = ident do begin

            new(pnt);           {* It's ok, create an element *}
            pnt^.ntyuname := token.name;
            getoken;

            if token.kind <> colon      then linerror(colonumiss)   { : (or ignore)}
                                    else getoken;

            pnt^.ntyutype := parseutype;        { Analyse a type declaration}

            if pnt^.ntyutype <> nil then begin  { Unless serious error, }
                if pnt^.ntyutype^.typunesting> typunesting      { Find max. nesting }
                    then typunesting := pnt^.ntyutype^.typunesting;
                typuminusize := typuminusize + pnt^.ntyutype^.typuminusize;
                typumaxusize := typumaxusize + pnt^.ntyutype^.typumaxusize;
                insertunamedutype(pnt, typufields);
                ensureutypeunamed(pnt^.ntyutype); { Needed for FORTRAN? }
            end {if ok};

            getoken; { Read next token: ";" }

            if token.kind <> semicolon then begin
                    {* Error recovery: Skip tokens until a good token is found *}
                    errfind(semicolumiss, [semicolon, endtok]);
                    if token.kind = semicolon then {* Skip it *}
                            getoken;
            end else {* Semicolon found *}
                    getoken;

          end {while token.kind=ident};
        end {with};

        if token.kind <> endtok
            then errfind(endtokumiss, [endtok]);
        getoken;                                    { skip the END };

        if token.kind <> recordtok then begin
            linerror(recordumiss);
            if token.kind = semicolon then backtoken; { Assume he forgot RECORD }
        end;

    end else if token.kind = sequence then begin    { SEQUENCE (...) OF <type>}
        new(pt);
        pt^.typubasicutype := token.kind;
        parseutype := pt;
        getusize(pt);

        getoken;
        if token.kind <> oftok then
            linerror(oftokumiss) {* But ignore it *}
        else with pt^ do begin
            getoken;
            typuexternal := false;
            typusubtype := parseutype;
            {* ensure_type_named(typ_subtype); *NO* not needed *}
            typunesting := typusubtype^.typunesting+1;
            typuminusize := 2;
            typumaxusize := (typusubtype^.typumaxusize*(typuhigh-typulow+1))+2;
        end {if};

 end else if token.kind in [stringtok,substring]    { Substring }
 then begin
        new(pt);
        parseutype := pt;
        with pt^ do begin
            typubasicutype := token.kind;
            getusize(pt);
            typusubtype := simpleudescriptor[chartok];
            typuexternal := false;
            if typubasicutype = stringtok then begin
                typunesting := 2;           { Can be 2 or 1 needed actually. }
                typumaxusize := typuhigh + 2;
                typuminusize := 2;
            end else begin
                typunesting := 1;
                typumaxusize := typuhigh + 4;
                typuminusize := 4;
            end;
        end {with};

 end else begin
        linerror(illubasicutype);
        locuerr := true;
        backtoken;
        parseutype := nil;

 end {if};

end; {parse_type}


{       Parse PRAGMA clause                                         parse_pragma
        -------------------
Parses:

    <pragma clause> ::  PRAGMA TIMEOUT ( <block_id>, <time value> ) ;
                    |   PRAGMA CAST ( <block_id> , ... ) ;
                    |   PRAGMA CONCURRENT ( <block_id> , ... ) ;
                    |   PRAGMA EXTERNAL_MARSHALLING ( <type_id> ) ;
                    |   PRAGMA <name> ( <id> <anything> ) ;
    <name>          ::  <id>

On entry,
    The keyword PRAGMA has been read, and is the current token.
On exit,
    The token AFTER the final semicolon has been read and is the current token.
}
procedure parseupragma;

var pragmauname:    iduname;            { Name of pragma being invoked }
    blk,                                { Pointer to block refered to }
    blockufather:   ptrublockutable;    { dummy }
    nty,                                { Pointer to named type referred to }
    typeufather:    ptrunamedutype;     { dummy }
    i:              integer;            { Loop index }
    typ:            ptrudefinedutype;   { New type if needed for external }
    param:          ptruidlist;         { Pointer to relevant parameter }

begin
    if expected(ident, [semicolon])
    then begin
        pragmauname := token.name;
        if expected(opnround, [semicolon])
        then if expected(ident, [semicolon])

{   The TIMEOUT pragma allows one to define the maximum time allowed
    for a procedure or function. It is represented by the blk_timeout parameter
    for that procedure.
}
        then if pragmauname.str='timeout                  ' then begin
            if findublock(token.name, blockufather, blk)
            then if expected(comma, [semicolon])
            then if expected(number, [semicolon])
            then begin
                blk^.blkutimeout := token.value;
                if expected(clsround, [semicolon])
                then if expected(semicolon, [semicolon])
                then {fine - no more to do. }
            end;

{   The CONCURRENT pragma allows one to specify that the execution of a
    procedure must continue in parallel with the client.
    It is represented by the blk_concurrent parameter for that procedure.

    The CAST pragma allows one to specify that no reply message is used
    for a procedure.
}
        end else if (pragmauname.str='concurrent               ')
                 or (pragmauname.str='cast                     ') then begin
            while token.kind=ident do begin
                if findublock(token.name, blockufather, blk)
                then begin
                    if pragmauname.str[2]='o'
                        then blk^.blkuconcurrent := true
                        else blk^.blkucast := true;
                end else begin {not found}
                    writeln(
  'RPCC: PRAGMA CAST or CONCURRENT must refer to previously defined procedure');
                    linerror(idunotudeclared);
                end {if};
                getoken;                    { Get the comma or clsround }
                if token.kind=comma
                then if expected(ident, [clsround, semicolon])
                then {loop};
            end {while};
            if token.kind = clsround
                then begin  if expected(semicolon, [semicolon])
                            then {fine - no more to do. }
            end else begin
                linerror(clsroundumiss);
                if token.kind<>semicolon
                    then if expected(semicolon, [semicolon]) then {dummy};
            end;

{   If a type is external, it's name will be used in the name of the
    marshalling routines. Therefore, we must ensure that the name of
    the type is the name specified, and not another type (such as
    a simple type) to which it was declared equivalent.
}
        end else if pragmauname.str='external_marshalling     ' then begin
            if findutype(typeptr, token.name, typeufather, nty)
            then begin
                if nty^.ntyutype^.typuname <> nty  { If type name is different }
                then begin
                    new(typ);               { Make a new type }
                    typ^ := nty^.ntyutype^; { like the last one }
                    typ^.typuname := nty;   { except named after us }
                    nty^.ntyutype := typ;   { and use that instead. }
                end {if};
                nty^.ntyutype^.typuexternal := true;
                nty^.ntyutype^.typunesting := 0;
                externalumarshalling := true;
            end else begin
                writeln(
   'RPCC: PRAGMA EXTERNAL_MARSHALLING must refer to previously defined type');
                linerror(idunotudeclared);
            end;
            if expected(clsround, [semicolon])
            then if expected(semicolon, [semicolon])
            then {fine - no more to do. }

{   The CALL_STATUS pragma allows one parameter of a procedure or function
    to be declared as a status parameter. In this case, if an error occurs
    on the call, that parameter will be set to the status value, and the
    other parameters will be undefined (unchanged in fact).
    The parser sets up a pointer, blk_status_param, from the procedure
    to the parameter to indicate this.
}
        end else if pragmauname.str='call_status              ' then begin
            if not findublock(token.name, blockufather, blk)
            then begin
                writeln(
   'RPCC: PRAGMA CALL_STATUS: First arg (proc/func name) not (yet) declared');
                linerror(idunotudeclared);
            end else if expected(comma, [semicolon])
            then if expected(ident, [semicolon])
            then if not finduparam(token.name, blk^.list, param) then begin
                writeln(' RPCC: PRAGMA CALL_STATUS:');
                writeln(' Second argument must be formal param. of proc/func');
                linerror(idunotudeclared);
            end else begin
                if blk^.blkustatusuparam <> nil then begin
                    writeln(
                        'RPCC: Proc/func already has a CALL_STATUS parameter');
                    linerror(twiceudeclared);
                end else blk^.blkustatusuparam := param;
            end {if};
            if expected(clsround, [semicolon])
            then if expected(semicolon, [semicolon])
            then {fine - no more to do. }

{   Unknown PRAGMAs could be ADA ones or goodness knows what. We don't
    even set warning status, we ignore them.
}
        end else begin
            write  ('RPCC: Ignoring unknown pragma "');
            with pragmauname do for i:=1 to len do write(str[i]);
            writeln('"');
            findutok([semicolon])       { Ignore unknown pragma }
        end {if good PRAGMA <name> ( <id>  etc };
    end {if got pragma name};
    getoken;                            { skip semicolon }
end {parse_pragma};

{*********************************************************************
                        TYPE_DECLARE
**********************************************************************

        Parses the complete TYPE declaration section

        <typedecl>      ::      <id> IS <type> ;   <typedecl>
                        |       <void>

On entry,
    The token (TYPE) has been read
On exit,
    The next token (PROCEDURE, etc) has been read.
}

procedure typeudeclare;
var
        elem:           namedutype;
        pnt:            ptrunamedutype;
begin {TYPE_DECLARE}

 while not(token.kind in[proctok, functok]) do begin
  if token.kind=pragmatok then parseupragma
  else if token.kind <> typetok then begin
        {* Error recovery: Skip all token until a good token is found *}
        errfind(typeumiss, [proctok, functok, typetok]);
  end else begin

{* We are sure that we found a TYPE declaration *}
   getoken;
   if token.kind <> ident then begin
        {* Error recovery: Skip all token until a good token is found *}
        errfind(identureq, [proctok, functok, typetok]);
   end else begin

        new(pnt);
        pnt^.ntyuname := token.name;                        { <id> }
        getoken;

        if token.kind <> istok  then linerror(istokumiss)   { IS }
                                else getoken;

        pnt^.ntyutype := parseutype;                        { <type> }

        if pnt^.ntyutype<>nil then begin

            insertunamedutype(pnt, typeptr);        { Put into the type list }

            if pnt^.ntyutype^.typuname=nil          { Link type back to name }
                then pnt^.ntyutype^.typuname := pnt;
        end;


        getoken; { Read next token }                        {    ; }

        if token.kind <> semicolon then begin
                {* Error recovery: Skip tokens until a good token is found *}
                errfind(semicolumiss, [semicolon, proctok, functok]);
                if token.kind = semicolon then {* Skip it *}
                        getoken;
        end else {* Semicolon found *}
                getoken;
   end; {else_if, token <> ident }

  end; {else_if, token <> typetok }

 end; {end_while token = typetok}

end; {TYPE_DECLARE}

{*****************************************************************************
*
*               BLOCK DECLARATION ANALIZER
*
******************************************************************************}

{                       Parse a list of parameters
                        ==========================

                <paramlist> ::  <idlist> : <direction> <type>

                <idlist>    ::  <id>
                            |   <id> , <idlist>

                <direction> ::  <void>  |  IN | OUT | IN OUT
}
procedure parseuparameterulist(var block: blockutable);
{
 This function builds a list of identifiers with their types associated.

On entry,
    'block' describes a new procedure or function.
    no fields need be filled in in 'block'.

On exit,
    block.list          Points to a list of paramaters;
    block.nesting       gives the number of indeces needed for (un)packing
    block.blk_min_in    gives the minimum size of the input parameters
    block.blk_max_in    gives the maximum size of the input parameters
    block.blk_min_out   gives the minimum size of the output parameters
    block.blk_max_out   gives the maximum size of the output parameters

* Small explanation:
* 1) get a list of identifier separate by commas. (id1, id2, id3, ....).
     This list is pointed by 'identlist'.
* 2) get the type definition: according to PASCAL rules, token should be:
        1) identifier, then we look in the TYPE list;
        2) RPCL simple type;

        We do NOT allow to declare RPCL structured type as procedure parameter
        because PASCAL compilers do not allow the declaration of a structured
        type as parameter. Example: suppose user declares:
        PROCEDURE PLUTO(PIPPO IN : SEQUENCE[100] OF SHORT);
        We can not expand in PASCAL statement:
        PROCEDURE PLUTO(L_PIPPO : INTEGER; A_PIPPO: ARRAY[1..100] OF RPC_SHORT);
        because the PASCAL compiler would complain.
        Instead we manage this unpleasant situation in such a way:
        a) User must declare a RPCL structured type in TYPE section declaration.
        b) RPCC creates a TYPE declaration for that identifier and type ONLY
           for the structured type embedded in such RPCL type
        c) RPCC expand procedure parameter using such a definition.
        Example: (RPCL User program)

          TYPE
                PAPERO = SEQUENCE(2000) OF SHORT;
          PROCEDURE PLUTO(PIPPO IN : PAPERO);

        will be expanded as: (RPCC output in PASCAL)

          TYPE
                PAPERO = ARRAY [1..2000] OF RPC_SHORT;
          PROCEDURE PLUTO(L_PIPPO : INTEGER; A_PIPPO : PAPERO);

        (Note: The length parameter now FOLLOWS the array - TBL)

* 3) If no error found fills the list of identifiers with their attributes:
     direction and type AND

* 4) add to the global list pointed by 'headlist'.

* 5) 'Loop' until ')'.

* WARNING:
* The list of the identifiers build on point 1 is split. So in the 'code'
* generation we will produce the parameter's list in the same order as
* declared but not in 'comma' format. (Without commas, I mean). So:
*               procedure bb( a,b,c : short);
* will be output as:
*               procedure bb( a: short; b: short; c: short);
*
****}
var
        identlist, headlist, tailist, last : ptruidlist;
        scan2type, junk : ptrunamedutype;
        ptr2type : ptrudefinedutype;
        direction : attrutype;
        gotuerror : boolean;

{____________________________________________________________________________

                        FND_IDLIST
}
function fnduidlist(    var name: iduname;
                        var last: ptruidlist;
                        head : ptruidlist): ptruidlist;
{***
* This function check whether 'name' is already in the list pointed by 'head'
* Returns in 'last' last element of the list or the previous element if found.
* Returns pointer to 'name' if succesful, nil otherwise
****}
var
        scan : ptruidlist;
        found : boolean;
begin
  scan := head; {* head pointer to the list to check *}
  last := head;
  found := false;
  while (scan <> nil) and not(found) do
        if scan^.name.str = name.str then
                found := true
        else begin
                last := scan;
                scan := scan^.next;
        end;

  fnduidlist := scan; {* Returns element or nil *}
end; {FND_IDLIST}

{_____________________________________________________________________________

                        GET_IDLIST
}
function getuidlist : ptruidlist;
{***
* This function builds a list of identifier and returns a pointer
* to the header. Returns NIL in case of error (No tokens are skipped!!)
****}
var
        localist, last, scan : ptruidlist;
        junk1, junk2 : ptrunamedutype;

begin {GET_IDLIST}
    localist := nil;
    repeat
        getoken;
        {* Get a parameter definition *}
        if token.kind <> ident then
                errfind(identureq, [comma, ident, colon]);
        if token.kind = ident then begin
                {***************************
                * Check if this identifier was already declared.
                * There are 3 lists in which we check:
                1) list of global types.
                2) list of all variables declared in this block,
                         pointed by headlist.
                3) list of variables read in this functions,
                         pointed by localist.
                ******************************}
                if findutype(typeptr, token.name, junk1, junk2) or
                        (fnduidlist(token.name, scan, headlist) <> nil) or
                        (fnduidlist(token.name, last, localist) <> nil) then
                        linerror(twiceudeclared) {* But ignore it!!! *}
                else begin
                        {* Add 'name' to the list, this is done
                        * adding in QUEUE, do not change this! *}
                        new(scan);
                        scan^.name := token.name;
                        scan^.next := nil;
                        if last = nil then {* Empty list *}
                                localist := scan
                        else
                                last^.next := scan;
                end;
                getoken;
        end; {* kind = ident *}
    until token.kind <> comma;

    getuidlist := localist;
end; {GET_IDLIST}
{_____________________________________________________________________________

       main block of parse_parameter_list;
}
begin {parse_parameter_list}
    gotuerror := false;
    headlist := nil; {* Pointer to the head of the list to bulid *}
    repeat

        {* Get the list of identifiers (id1, id2, id3, ...) *}
        identlist := getuidlist;
        {* Now 'identlist points to the header of this list *}
        {* Get_idlist already reads next token *}

        if token.kind <> colon then
                linerror(colonumiss) {* But ignore *}
        else
                getoken;

{ Get direction attribute }

        if not(token.kind in[intok, outok]) then begin
                direction := intok; {* Assume inout, perhaps forgotten *}
        end else begin
                direction := token.kind;
                getoken;
                if (direction = intok) and (token.kind = outok) then begin
                                direction := inoutok;
                                getoken;
                end;
        end;

{ Get type definition       : <type>

    Originally, the type had to be a named type, but now we can generate
    an arbitrary name for an unnamed type. The name is needed for Pascal.
}

        ptr2type := parseutype;         { Parse it }
        ensureutypeunamed(ptr2type);    { Invent a name if necessary }

{   Fill in this information for each parameter name:
}
        if (identlist <> nil) and (ptr2type <> nil) then with block do begin

                { Find maximum nesting: }
                if ptr2type^.typunesting > blkunesting
                    then blkunesting := ptr2type^.typunesting;

                {* Add direction and type attributes *}
                if headlist = nil then {* Empty list *}
                        headlist := identlist
                else
                        tailist^.next := identlist;
                tailist := identlist;
                repeat
                        tailist^.attr := direction;
                        tailist^.idutype := ptr2type;
                        last := tailist;
                        tailist := tailist^.next;
                        { Accumulate total parameter size: }
                        if direction in [intok, inoutok] then begin
                            blkumaxuin := blkumaxuin+ptr2type^.typumaxusize;
                            blkuminuin := blkuminuin+ptr2type^.typuminusize;
                        end;
                        if direction in [outok, inoutok] then begin
                            blkumaxuout := blkumaxuout+ptr2type^.typumaxusize;
                            blkuminuout := blkuminuout+ptr2type^.typuminusize;
                        end;

                until tailist = nil;
                tailist := last;
        end; {* Else ignore this wrong definition *}
        getoken;
        if not(token.kind in [semicolon, clsround]) then begin
                {* Semicolon or clsround missed.
                * Error recovery: Skip this parameter's definition and
                * find a semicolon or a closed round bracket or
                * a proctok/functok declaration
                *}
                errfind(semicuroundumiss, [semicolon, clsround, functok,
                         proctok, ident]);
                case token.kind of
                        semicolon, clsround: ;
                        ident: backtoken;{* Continue analisys *}
                        colon, functok, proctok: begin
                                        gotuerror := true;
                                        backtoken;
                                        end;
                end;
        end;
    until (token.kind = clsround) or (gotuerror);

    block.list := headlist;
end; {parse_parameter_list}

{*********************************************************************
                BLOCK_DECLARE
**********************************************************************}

procedure blockudeclare;
var
        block:          blockutable;
        junk1, scan:    ptrunamedutype;
        blk, junk2,
        lastblock:      ptrublockutable;
        localerr:       boolean;
        ascan:          ptruidlist;

begin {BLOCK_DECLARE}
  repeat
    localerr := false;

    with block do begin
        blkunesting := 0;   { Initialise indeces required }
        blkuminuin := 0;    { Initialize parameter size counts }
        blkuminuout := 0;
        blkumaxuin := 0;
        blkumaxuout := 0;
    end;

    if not(token.kind in[proctok, functok, pragmatok, endtok]) then
        errfind(procuexp, [proctok, functok, endtok, pragmatok]);

    if token.kind=pragmatok then parseupragma
    else if token.kind <> endtok then begin
        {* Save block type *}
        block.butype := token.kind;
        block.inuonly := (token.kind = proctok);        { could be in_only }

        getoken;                                                { <ident> }
        if token.kind <> ident then begin
                {* Error recovery: ignore, but do not build any block later *}
                linerror(identureq);
                localerr := true;
        end else begin
                block.name := token.name;
                if findutype(typeptr, token.name, junk1, scan) or
                        findublock(token.name, lastblock, junk2) then begin
                        {* Error recovery: ignore, but do not build any block later *}
                        linerror(twiceudeclared);
                        localerr := true;
                end;
        end;

        getoken;                                                {  (    }
        if token.kind = opnround then begin
                parseuparameterulist(block);                    { ... ) }
                getoken;
        end else
                if token.kind in[inoutok, intok, outok] then begin
                        linerror(opnroundumiss);
                        backtoken;
                        parseuparameterulist(block);            { ... ) }
                        getoken;
                end else
                        block.list := nil;              { TBL 18.8.86 }



        if block.butype = functok then begin
                if token.kind <> return then
                        linerror(returnumiss) {* But ignore *}
                else
                        getoken;
                if token.kind = ident then begin
                        {* Scan the types' list *}
                        if findutype(typeptr, token.name, junk1, scan) then
                                block.return := scan^.ntyutype^.typubasicutype
                        else begin
                                linerror(idunotudeclared);
                                localerr := true;
                        end;
                end else
                        block.return := token.kind;

                if block.return in[chartok..longtok] then begin
                        getoken;
                        block.blkumaxuout := block.blkumaxuout +
                            simpleudescriptor[block.return]^.typumaxusize;
                        block.blkuminuout := block.blkuminuout +
                            simpleudescriptor[block.return]^.typuminusize;
                end else begin
                        localerr := true;
                        linerror(simtypeureq)
                end;

        end {functok} else {proctok} begin

        { Check for OUT or INOUT parameters to a procedure }

                ascan := block.list;
                while ascan <> nil do begin
                        if ascan^.attr <> intok then block.inuonly := false;
                        ascan := ascan^.next;
                end; {while}

        end; {procok}

        if token.kind <> semicolon then begin
                errfind(semicolumiss, [semicolon, proctok, functok, endtok]);
                if token.kind = semicolon then
                        getoken;
        end else
                getoken;

        if not localerr then begin
                {* Create a new element and save the name *}
                new(blk);
                blk^ := block;
                blk^.next := nil;
                blk^.blkustatusuparam := nil;
                blk^.blkucast := false;
                blk^.blkuconcurrent := false;
                if runoptions[timeout].value
                 then blk^.blkutimeout := timeoutuval       { default timout }
                 else blk^.blkutimeout := rpcudefaultutimeout;

                if lastblock = nil then
                        {* List empty, update blockptr *}
                        blockptr := blk
                else
                        lastblock^.next := blk;
        end;
    end; {if kind <> endtok}

  until token.kind = endtok;

end; {BLOCDECLARE}


{************************************************************************

        Parser Initialisation

On exit,
    The simple type descriptors are prepared for the more complex
    types and parameters to be linked to.
}
procedure initialiseuparser;
var t:  typeutoken;
begin
  for t := chartok to longtok do begin
   new(simpleudescriptor[t]);
   with simpleudescriptor[t]^ do begin
     typubasicutype := t;
     typunesting := 0;      { No indeces are necessary for this type }
     typusubtype := nil;
     new(typuname);
     with typuname^ do begin
        ntyuname.len := maxidlen;
        ntyutype := simpleudescriptor[t];
        ntyunext := nil;
     end;

     case t of
        chartok,
        bytetok:    typumaxusize := 1;
        shortok,
        integertok: typumaxusize := 2;
        real32tok,
        longtok:    typumaxusize := 4;
        real48tok,
        real64tok:  typumaxusize := 8;
        real128tok: typumaxusize := 16;
     end; {end_case}
     typuminusize := typumaxusize;
   end {with};
  end {for};

  simpleudescriptor[chartok]^.typuname^.ntyuname.str
        := 'rpc_char                 ';
  simpleudescriptor[bytetok]^.typuname^.ntyuname.str
        := 'rpc_byte                 ';
  simpleudescriptor[shortok]^.typuname^.ntyuname.str
        := 'rpc_short                ';
  simpleudescriptor[integertok]^.typuname^.ntyuname.str
        := 'rpc_integer              ';
  simpleudescriptor[real32tok]^.typuname^.ntyuname.str
        := 'rpc_real32               ';
  simpleudescriptor[longtok]^.typuname^.ntyuname.str
        := 'rpc_long                 ';
  simpleudescriptor[real48tok]^.typuname^.ntyuname.str
        := 'rpc_real48               ';
  simpleudescriptor[real64tok]^.typuname^.ntyuname.str
        := 'rpc_real64               ';
  simpleudescriptor[real128tok]^.typuname^.ntyuname.str
        := 'rpc_real128              ';

  for t := chartok to longtok do
    with simpleudescriptor[t]^, typuname^, ntyuname do
        while str[len]=' ' do len := len-1;     { strip trailing spaces }

  numberuofuinventedutypes := 0;
  externalumarshalling := false;    { Not invoked yet }

end {initialise_parser};

{****************************************************************
*
*               PARSER, MAIN LOOP
*
*****************************************************************}

procedure parser;

begin {PARSER}
    initialiseuparser;
{* Look for 'PACKAGE' *}
    getoken;
    if token.kind <> package then
        linerror(packageumiss) {* Ignored if missed *}
    else
        getoken;
{* Look for 'unit - name' *}
    if token.kind <> ident then
        linerror(identureq) {Ignore}
    else begin
        unitname := token.name;
        getoken;
    end {if};

{* Look for 'is' *}
    if token.kind <> istok then
        linerror(istokumiss) {Ignore}
    else
        getoken;

{* Look for 'type' *}
    typeudeclare;
{* Returns with the next token already read *}

    if token.kind = endtok then
        linerror(blocksureq)
    else begin
        blockudeclare;
        if token.kind <> endtok then
                linerror(endtokumiss)
        else
                getoken;
        if token.kind <> ident then
                linerror(identureq)
        else
                if token.name.str <> unitname.str then
                        linerror(baduname);
        getoken;
        if token.kind <> semicolon then
                linerror(semicolumiss);

    end {if};
end; {PARSER}


{ Code generation (PILS, FORTRAN generation in separate files)
}
{                                                                       CODEFOR

        Top-down RPC compiler:          Code generator for FORTRAN
        ======================          ==========================


History:
       Nov 88   Composite types and pointers; arrays of named types.
                FORTRAN version made from Pascal/C versions! (TBL)
                (Talk about backward steps!?!)

     8 May 1988 detach_xxx and close_xxx and S_xxxx common block introduced.
    22 Jun 1989 REQUEST is now declared INTEGER*2 as it is UPK_SHORT()-ed
    24 Oct 1989 Records of strings now declared correctly in client.
     9 Feb 1990 External marshalling routines end in _FOR too. (TBL)
    16 Mar 1990 RPC_CALL_STATUS is now declared as INTEGER*4.
    17 Apr 1990 Error message numbers are now declared INTEGER

Requirements of the run-time system:

    1.  RPC_BEGIN_CALL_FOR must
            o   Set up the 'procedure number' and 'version' fields;
            o   Leave the m_index pointer to call_header_length;
            o   Set up m_socket etc. as for rpc_begin().
    2.  RPC_END_CALL_FOR must dispose of the message
    3.  RPC_CALL must leave m_index set to return_header_length
    4.  On entry into the server stub, m_index must be = call_header_length-4
    5.  The routine RPC_INIT_RETURN_FOR must set it to return_header_length
    6.  PCK/UPK_STRING_FOR must pack and unpack strings, and post-align.
    7.  PCK/UPK_SUBSTRING_FOR must pack and unpack substrings, and post-align.
    8.  RPC_SET_ERROR must set the m_status field of the message as given.
    9.  RPC_ATTACH_STUB_FOR is a fortran callable rpc_attach_stub
   10.  RPC_DETACH_STUB_FOR is a fortran callable rpc_detach_stub
   11.  RPC_OPEN_FOR        is a fortran callable rpc_open
   12.  RPC_CLOSE_FOR       is a fortran callable rpc_close
   13.  PCK/UPK_xxxx_FOR are the packing/unpacking routines Fortran callable.
   14.  RPC_NO_RETURN must set the message status to "Normal" to inhibit reply.
   15.  RPC_REPORT_ERROR_FOR is a fortran callable rpc_report_error

VAX/FORTRAN-isms in code produced by this module:

    o   The long names and embedded underscore characters in identifiers;
    o   The STRUCTURE/RECORD types (if used);
    o   The length specification on INTEGER for rpc_short & rpc_long (if used);
    o   The BYTE data type for rpc_byte (if used).

Restrictions:

    1.  The data types are restricted to VAX/FORTRAN data types: No pointers
        but record types are allowed.

Note:
    1.  The PARAMETER statements generated to define the two RPC error
        codes produced by a server stub should match RPC$CONST always.
}
{___________________________________________________________________________

        Convert character to Upper case
}
function upperucase(ch: char): char;
begin
    if (ch >= 'a') and (ch<='z')
        then upperucase := chr(ord(ch)-ord('a')+ord('A'))
        else upperucase := ch;
end;

{___________________________________________________________________________

        Write out one identifier (in uppercase if FORTRAN)
}
procedure writeuname(var where: text; var name: iduname);
var
        a:  integer;
begin
    with name do
        for a := 1 to len do
            if omode=vaxfor
                then write(where, upperucase(str[a]))
                else write(where, str[a]);
end;
{___________________________________________________________________________

        Write out component expression  (in uppercase if FORTRAN)
        ------------------------------
}
procedure writeuexp(var where: text; var expr: expression);
var a : integer;
begin
    with expr do
        for a := 1 to len do
            if omode=vaxfor
                then write(where, upperucase(str[a]))
                else write(where, str[a]);
end;
{___________________________________________________________________________

        Write a quoted string padded to 40 characters
}
procedure writeunameupadded(var opufile:text; name: iduname; ch: char);
begin
  write(opufile, ch);
  writeuname(opufile, name);
  write(opufile, ch:rpcunameulength + 1 - name.len);
end {write_name_padded};

{=============================================================================

                        FORTRAN-sepcific procedures
}
{____________________________________________________________________________

        Code generator error
        --------------------

    Code generator errors are internal errors or target-dependent error.
}
procedure codeforuerror(why: errorustring);
begin
    writeln(' RPCC: **** Error: ', why);
    writeln('       (Error detected at FORTRAN code generator stage.)');
    halt;
end;
{___________________________________________________________________________

        Indenting algorithm
}
function indfor(level: integer): integer;
begin
    indfor := 3+level*3
end;


{___________________________________________________________________________

        Start a Continuation Line
}
procedure continuation;
begin
    writeln(opufile);
    write(opufile, '     +');
end;
{___________________________________________________________________________

        Append a FORTRAN index to an array expression

    Level = 1       =>      RPC_A   used,
    Level = 2       =>      RPC_B   used etc...
}
procedure appenduindexufor(var expr: expression; level: integer);
var ch: char;
begin
            format(expr, '$(RPC_    ');
            appendu1(expr, chr(ord('A')+level-1));
            appendu1(expr, ')');

end;

{___________________________________________________________________________

      Generate procedure name to (un)pack a simple type or descriptor
      ---------------------------------------------------------------

    Used for the function return result.

|       generates:      upk/pck_ttt(rpc_p_buf,
}
procedure packusimpleufor(var where: text; what : typeutoken; topack: boolean);
begin
    write(where, '      CALL ');
    if topack then write(where,'PCK_') else write(where, 'UPK_');
    case what of
        chartok:        write(where, 'CHAR');
        bytetok:        write(where, 'BYTE');
        shortok:        write(where, 'SHORT');
        integertok:     write(where, 'INTEGER');
        real32tok:      write(where, 'REAL32');
        real48tok:      write(where, 'REAL48');
        real64tok:      write(where, 'REAL64');
        real128tok:     write(where, 'REAL128');
        longtok:        write(where, 'LONG');
    end;
    write(where, '_FOR(RPC_P_BUF,');
end;

{___________________________________________________________________________

        Generate code to:       Align on a 2**power byte boundary

    Currently, only aligns on word boundaires, because that is all
    we need.
}
procedure genualignufor(    power:  integer;
                            level:  integer);
begin
    writeln(opufile, ' ':indfor(level),
                'CALL RPC_ALIGN(RPC_P_BUF,', power:1, ')' );
    { Extra runtime support needed here }
end {gen_align_for};
{___________________________________________________________________________

        Generate code to:       (un)Pack a parameter of given type
        ----------------        ----------------------------------

    This recursive procedure uses the 'level' parameter to control the
    amount of indentation at each stage, and which rpc_<level> parameters
    may be accessed by any given stage.

    If the type is declared as 'EXTERNAL_MARSHALLING, then an external
    procedure PCK_<typename>_FOR or UPK_<typename>_FOR is called to do the
    marshalling.

On entry,
    Output cursor at left margin.
On exit,
    Output cursor at left margin.
}
procedure genupackutypeufor(var where:  text;   { output file }
                            expr:    expression;{ Expression for variable }
                            typ:    ptrudefinedutype;   { type descriptor }
                            level:  integer;    { indentation & rpc_<level> }
                            topack: boolean);   { pack or unpack? }
var ch:     char;
    scan:   ptrunamedutype;
    element:ptrudefinedutype;   { Array element type }
    depth:  integer;        { Depth of nesting of array element }
    exp2:   expression;
    luexpr,                 { Expression for the length of a string, etc }
    suexpr: expression;     { Expression for the start of a substring }
    i:      integer;        { Loop index }
    localulabel: integer;   { Record of last label we used here }

{_____________________________________________________________________________
                    Main block of gen_pack_type_for
}
begin {gen_pack_type_for}

   {    Generate a few useful expressions: }

    ch := chr(ord('A')+level-1); {for RPC_A etc }
    luexpr := expr;             {   x }
    prependu1(luexpr, '_');     {  _x }
    suexpr := luexpr;           {  _x }
    prependu1(luexpr, 'L');     { L_x }
    prependu1(suexpr, 'S');     { S_x }

    with typ^ do
    if typuexternal then begin
            write(where, ' ':indfor(level), 'CALL ');
            if topack then write(where,'PCK_') else write(where, 'UPK_');
            writeuname(opufile, typuname^.ntyuname);
            write(where, '_FOR(RPC_P_BUF,');
            writeuexp(where, expr);
            writeln(where, ')');

    end else case typubasicutype of
{ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                                                        S I M P L E

Here we assume that the names of the pck/upk routines are consistent
with the names rpc_xxx of the basic types. The exception is the code
for unpacking a character, which is going to be a bit wierd, because
of the descriptor.

}
        chartok: begin
            write(where, ' ':indfor(level), 'CALL ');
            if topack then write(where,'PCK') else write(where, 'UPK');
            write(where, '_CHAR_FOR(RPC_P_BUF,');
            writeuexp(where, expr);
            writeln(where, ')');
        end {simple type };

        bytetok,
        shortok, integertok,
        real32tok, real48tok,
        real64tok, real128tok,
        longtok: begin
            write(where, ' ':indfor(level), 'CALL ');
            if topack then write(where,'PCK') else write(where, 'UPK');
            with typuname^.ntyuname do
             for i := 4 to len do
              write(where, upperucase(str[i]));  { eg "_INTEGER" }
            write(where, '_FOR(RPC_P_BUF,');
            writeuexp(where, expr);
            writeln(where, ')');
        end {simple type };

{ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

                                                        A R R A Y
Example:    DO 101 RPC_A=1,12
                DO 102 RPC_B=1,10
                        ...MYVAR(RPC_A,RPC_B)...
        102     CONTINUE
        101 CONTINUE

}
        arraytok: begin
            exp2 := expr;
            depth := level;
            ch := chr(ord('A')+depth-1);
            element := typusubtype;
            format(exp2, '$(RPC_    ');
            appendu1(exp2, ch);
            writeln(where, ' ':indfor(depth),
                    'DO ', nextulabel:1,
                    ', RPC_', ch, '=1,', typuhigh-typulow+1:1);
            nextulabel := nextulabel+1;         { Get a new labels }
            while element^.typubasicutype=arraytok do with element^ do begin
                depth := depth +1;
                ch := chr(ord('A')+depth-1);
                writeln(where, ' ':indfor(depth),
                    'DO ', nextulabel:1,
                    ', RPC_', ch, '=1,',typuhigh-typulow+1:1);
                nextulabel := nextulabel+1;             { Get a new labels }
                format(exp2, '$,RPC_    ');
                appendu1(exp2, ch);
                element := element^.typusubtype;
            end {while};
            appendu1(exp2, ')');

            localulabel := nextulabel;  { Save label }
            genupackutypeufor(where, exp2, element, depth+1, topack);

            while depth>= level do begin
                localulabel := localulabel-1;  { recalculate original label }
                writeln(where, localulabel:4,
                    ' ':indfor(depth)-4, 'CONTINUE');   { Not END DO }
                depth := depth-1;       { Come back out of nesting }
            end {while};
        end {arraytok};
{ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}
        recordtok: begin                                { R E C O R D }
            scan := typufields;
            while scan <> nil do with scan^ do begin
                exp2 := expr;
                appendu1(exp2,'.');
                appenduname(exp2, ntyuname);
                genupackutypeufor(where, exp2, ntyutype, level, topack);
                scan := ntyunext;
            end {while};
        end {recordtok};
{ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                                                        A C C E S S

}
        accesstok: begin
            codeforuerror(
                'No ACCESS (POINTER) type exists in FORTRAN!     ');
        end {accesstok};

{ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}
        sequence: begin                                 { S E Q U E N C E }
            if level>1 then codeforuerror(
                'SEQUENCE type not allowed within composite type.');

            genupackutypeufor(where, luexpr,
                    simpleudescriptor[integertok],level, topack);

            exp2 := expr;
            format(exp2, 'A_$       ');     { Make an expression for the array}
            appenduindexufor(exp2, level);  { Make expression for an element }

            write(where, ' ':indfor(level));
                write(where,    'DO ', nextulabel:1, ', RPC_', ch, '=1, L_');
                writeuexp(where, expr);
                writeln(where);
            localulabel := nextulabel;
            nextulabel := nextulabel+1;
            genupackutypeufor(where, exp2, typusubtype, level+1, topack);
            writeln(where, localulabel:4, ' ':indfor(level)-4, 'CONTINUE');
        end {sequence};

{ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

In the server, strings are unpacked and the lengths fudged in the descriptor
by the routine UPK_VSTRING_FOR(buffer, string, length).
}
        stringtok: begin                                { S T R I N G }

            write(where, ' ':indfor(level));
            if topack
                then                write(where, 'CALL PCK_STRING_FOR')
                else if client then write(where, 'CALL UPK_STRING_FOR')
                else                write(where, 'CALL UPK_VSTRING_FOR');
            write(where, '(RPC_P_BUF,');
            writeuexp(where, expr);
            if not topack and not client then write(where, ',', typuhigh:1);
            writeln(where, ')');

        end {stringtok};

{ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                                                          S U B S T R I N G

}
        substring: begin

            write(where, ' ':indfor(level));
            if topack
                then    write(where, 'CALL PCK_SUBSTRING_FOR')
                else    write(where, 'CALL UPK_SUBSTRING_FOR');
            write(where, '(RPC_P_BUF,A_');
            writeuexp(where, expr);
            write(where, ',');
            continuation;               { Keep it in 72 characters };
            write(where,    'S_');
            writeuexp(where, expr);
            write(where,    ',L_');
            writeuexp(where, expr);
            writeln(where,    ')');

        end {substring};

    end {case};
end {gen_pack_type_for};

{___________________________________________________________________________

        Generate code to:       (un)Pack all parameters necessary
        -----------------       ---------------------------------

The dirn parameter is INtok or OUTok, to specify which parameters to unpack.

A string descriptor needs to be initialised, including the creation of the
string, at the server side, and the string has to be disposed of afterwards.

}
procedure genupackufor( var     where:  text;           { Output file }
                        head:   ptruidlist;     { Parameter list }
                        dirn:   attrutype;      { Parameter filter }
                        topack: boolean);       { Pack, rather than unpack? }

var expr:    expression;
begin {Gen_Pack_For}
  repeat
    with head^, idutype^ do begin


        { generate an expression for the variable: }

        expr.len := 0;      { Expression for the variable }
        appenduname(expr, head^.name);    { expr is the param. name }


{       Pack or unpack
        --------------
}
        if (attr = inoutok)
            or (attr = dirn)
        then begin
            deref := client
                    and not (runoptions[byvalue].value   {* not value param *}
                    and (attr = intok));            { and in-only param }

            genupackutypeufor(where, expr, idutype, {level} 1, topack);

        end {if};

    end {with};
    head := head^.next;
  until head = nil;
end; {Gen_Pack_For}


{               (end of packing/unpacking)
*****************************************************************************

            Generate:       A type declaration in FORTRAN

    This recursive procedure generates a definition of a type, given
    a name and a type descriptor. It is impossible to generate a type
    itself, as the name is buried within it in FORTRAN.

    Examples
            CHARACTER*80    MYSTR(10,2)
                for
                    mystr:  ARRAY[1..10, 1..2] OF STRING(80);

            RECORD /MYSTRUCTURE/ MYVARIABLE(4)
                for
                    myvariable: ARRAY[1..4] OF MYSTRUCTURE;

    Note: In a client routine, a string parameter (though not a string element
    in a record parameter) has an undefined length: "CHARACTER * ( * )".
    [The spaces here are to avoid <*> <)> closing this Pascal comment!]
    In the server, a length must be specified explicitly.

On entry,
    Cursor is assumed to be at left margin.
On exit,
    Cursor is again at the left margin.
}

procedure genutypeudeclufor(expr:   expression;         { for the varaiable }
                            pt:     ptrudefinedutype;   { type structure }
                            level:  integer);           { indentation }
var     scan:   ptrudefinedutype;
        exp2:   expression;
begin
    with pt^ do case typubasicutype of

        chartok,                    { First token, simple type, courier type }
        bytetok,
        shortok, integertok,
        real32tok, real48tok,
        real64tok, real128tok,
        longtok: begin
            write(opufile, ' ':indfor(level));
            case typubasicutype of
                chartok:        write(opufile, 'CHARACTER*1');
                bytetok:        write(opufile, 'BYTE');         { @@ VAX ONLY }
                integertok:     write(opufile, 'INTEGER');
                real32tok:      write(opufile, 'REAL');
                real48tok:      write(opufile, 'DOUBLE');
                real64tok:      write(opufile, 'DOUBLE');
                real128tok:     write(opufile, 'DOUBLE');
                shortok:        write(opufile, 'INTEGER*2');    { @@ VAX ONLY }
                longtok:        write(opufile, 'INTEGER*4');    { @@ VAX ONLY }
            end {case};
            write(opufile, ' ');
            writeuexp(opufile, expr);   { Example: rpc_char x }
            writeln(opufile);
        end {simple type};

        substring,
        stringtok:  begin               { Example: rpc_char x[81] }
                write(opufile, ' ':indfor(level));
                if client and (level=1) then                    { 91024 }
                    write(opufile, 'CHARACTER*(*) ')
                else
                    write(opufile, 'CHARACTER*', (typuhigh):1, ' ');
                writeuexp(opufile, expr);
                writeln(opufile);
        end {stringtok};

        sequence,
        arraytok:   begin               { example  INTEGER MYARR(10,20) }
                exp2 := expr;
                appendu1(exp2,'(');
                appendudecimal(exp2,typuhigh-typulow+1);
                scan := typusubtype;
                while scan^.typubasicutype = arraytok do begin
                    appendu1(exp2,',');
                    appendudecimal(exp2,scan^.typuhigh-scan^.typulow+1);
                    scan := scan^.typusubtype;
                end {while};
                appendu1(exp2,')');
                genutypeudeclufor(exp2, scan, level); {non-array type}
        end {arraytok};

        accesstok: begin
            codeforuerror(
                'Package has pointers: Can''t make FORTRAN stubs. ');
        end;

        recordtok: begin                { VAX/FORTRAN ONLY! }
            write(opufile, ' ':indfor(level), 'RECORD ');
            if typuname = nil then codeforuerror(
                'Every structure must be named for VAX/FORTRAN.  ')
            else begin
                write(opufile,'/');
                writeuname(opufile,typuname^.ntyuname);
                write(opufile,'/ ');
            end {if};
            writeuexp(opufile, expr);
            writeln(opufile);
        end;

    end {with, case};
end {gen_type_decl_for};

{_____________________________________________________________________________

                Generate Common Block for this client

}
procedure genuclientucommon;
begin
        write(opufile, '      COMMON /C_');
        writeuname(opufile, unitname);

        write(opufile, '/ H_');
        writeuname(opufile, unitname);
        writeln(opufile);

        write(opufile, '      INTEGER H_');
        writeuname(opufile, unitname);
        writeln(opufile);

end {gen_client_common};

{_____________________________________________________________________________

                Generate Common Block for this server

}
procedure genuserverucommon;
begin
        write(opufile, '      COMMON /S_');
        writeuname(opufile, unitname);

        write(opufile, '/ P_');
        writeuname(opufile, unitname);
        writeln(opufile);

        write(opufile, '      INTEGER P_');
        writeuname(opufile, unitname);
        writeln(opufile);
end {gen_server_common};

{____________________________________________________________________________

            Generate Parameter list
}
procedure genudummies(head: ptruidlist);

  { Generates the formal parameter list enclosed in brackets  (a,b,c,d)  }
var
        col:        integer;
        scan:       ptruidlist;

    procedure commaucont(name: iduname);
    begin
        write(opufile, ',');            { Separate two parameters }
        col := col + 1;                 { Count characters }
        if col + name.len > 65 then     { Allow 5 spare }
        begin
            continuation;
            col := 6;
        end;
    end;


begin {gen_dummies}

     if head <> nil then begin

        scan := head;
        write(opufile, '(');
        col := 40; { upper limit to characters so far on line }
        while scan <> nil do begin
            if scan^.idutype^.typubasicutype in [sequence, substring]
                    then write(opufile, 'A_');
            writeuname(opufile, scan^.name);
            col := col + scan^.name.len + 2;

            if scan^.idutype^.typubasicutype = substring then
            begin
                commaucont(scan^.name);
                write(opufile, 'S_');
                writeuname(opufile, scan^.name);
                col := col + scan^.name.len + 2;
            end;

            if scan^.idutype^.typubasicutype in [sequence, substring] then
            begin
                commaucont(scan^.name);
                write(opufile, 'L_');
                writeuname(opufile, scan^.name);
                col := col + scan^.name.len + 2;
            end;

            scan := scan^.next;
            if (scan<>nil) then commaucont(scan^.name);
        end {while};
        write(opufile, ')');
     end{if list not NIL};
end; {gen_dummies}
{_____________________________________________________________________________

        Generate local variables (server) / parameter (client)
        ------------------------------------------------------

This subroutine will generate

    -  The parameters to the client subroutine,
    -  or the local variables in the server.

}

procedure genuparamsufor(head: ptruidlist);
var
        auexpr,
        luexpr,
        suexpr,
        expr:       expression;
        a:          integer;
        scan:       ptruidlist;
        ch:         char;
        simpleuref:  boolean;   { Should simple variables be dereferenced? }
        compositeuref: boolean; { what about composite variables? }
begin {gen_params_for}


    scan := head;
    while scan <> nil do begin

        {* Write the type's name if any otherwise the full declaration *}
        {* Sequences and substrings MUST be expanded a little *}

        with scan^.idutype^ do begin

            expr.len := 0;
            appenduname(expr, scan^.name);
            luexpr := expr;             {   x }
            prependu1(luexpr, '_');     {  _x }
            suexpr := luexpr;           {  _x }
            auexpr := luexpr;           {  _x }
            prependu1(luexpr, 'L');     { L_x }
            prependu1(suexpr, 'S');     { S_x }
            prependu1(auexpr, 'A');     { A_x }

            case typubasicutype of

            chartok,
            bytetok,
            shortok, integertok,
            real32tok, real48tok,
            real64tok, real128tok,
            longtok,
            stringtok,
            recordtok,
            arraytok:
                genutypeudeclufor(expr, scan^.idutype, 1);

            sequence:
                begin                                      { a_xxx: tttt; }
                    genutypeudeclufor(auexpr, scan^.idutype, 1);
                                                     { l_xxx: rpc_integer }
                    genutypeudeclufor(luexpr,
                        simpleudescriptor[integertok], 1);
                end;

            substring:
                begin                                      { a_xxx: tttt; }
                    genutypeudeclufor(auexpr, scan^.idutype, 1);

                    genutypeudeclufor(suexpr,
                        simpleudescriptor[integertok], 1);

                    genutypeudeclufor(luexpr,
                        simpleudescriptor[integertok], 1);
                end;
            end {case};

        end; {with}

        scan := scan^.next;
   end {while};
end; {GEN_PARAMS_for}

{_________________________________________________________________________

            Generate:   A structure declaration in FORTRAN
            ==============================================

Example:    STRUCTURE /structurename/
                CHARACTER*80    fieldname
                INTEGER         fieldname2
            END STRUCTURE
}
procedure genustructufor(   pt:     ptrudefinedutype;   { type structure }
                            level:  integer);           { indentation }

var     scan:   ptrunamedutype;
        exp2:   expression;
begin

        with pt^ do if typubasicutype = recordtok then begin
                                                { VAX/FORTRAN ONLY! }
            write(opufile, ' ':indfor(level), 'STRUCTURE ');
            if typuname = nil then codeforuerror(
                'Every structure must be named for VAX/FORTRAN.  ')
            else begin
                write(opufile,'/');
                writeuname(opufile, typuname^.ntyuname);
                write(opufile,'/ ');
            end {if};
            writeln(opufile);

            scan := typufields;
            while scan <> nil do begin
                exp2.len := 0;
                appenduname(exp2, scan^.ntyuname);
                genutypeudeclufor(exp2, scan^.ntyutype, level+1);
                scan := scan^.ntyunext;
            end {while};

            writeln(opufile, ' ':indfor(level), 'END STRUCTURE');

    end {with..if};
end {gen_struct_for};
{___________________________________________________________________________

        Generate type definitions for whole package
        ===========================================

Example:
            INTEGER         P1
            INTEGER*4       P2(3,3)
            CHARACATER*80   P3(10)
}

procedure generateutypesufor;
var
        scan:   ptrunamedutype;
        expr:   expression;

begin {generate types}
    scan := typeptr;
    while scan <> nil do with scan^, ntyutype^ do begin
        if typubasicutype = recordtok then begin

{   Now make the main type definition:
}
            genustructufor(ntyutype, 1);
            writeln(opufile);                       { Blank line }
        end {if};

        scan := ntyunext;
    end; {while}
end;

{******************************************************************************

                Client code generator
                =====================

Generates one procedure or function for the client.
}

procedure clientugenufor;
var
        scan :          ptrublockutable;
        procunumber :   integer;

  procedure genuclientublock(ptr: ptrublockutable);

{***
* On each block, the client allocates 'RPC_integer' variables to pack or
* unpack parameters.
****}
  var
        a:              integer;
        expr:           expression;     { Expression for return value }

 begin {GEN_client_BLOCK}
    with ptr^ do begin
{   Generate Header:
}
        write(opufile, ' ':indfor(1));
        if butype = functok then begin

            case return of
                chartok:        write(opufile, 'CHARACTER*1');
                bytetok,
                shortok,
                longtok,
                integertok:     write(opufile, 'INTEGER');
                real32tok:      write(opufile, 'REAL');
                real48tok,
                real64tok,
                real128tok:     write(opufile, 'DOUBLE');
            end {case};
            write(opufile, ' FUNCTION ');
            writeuname(opufile, name);
            case return of
                chartok:        ;
                bytetok:        write(opufile, '*1');           { @@ VAX ONLY }
                integertok,
                real32tok,
                real48tok,
                real64tok,
                real128tok:     { ok as it is};
                shortok:        write(opufile, '*2');       { @@ VAX ONLY }
                longtok:        write(opufile, '*4');       { @@ VAX ONLY }
            end {case};
        end else begin
            write(opufile, 'SUBROUTINE ');
            writeuname(opufile, name);
        end {if};
        genudummies(list);      { Generate formal parameter list }
        writeln(opufile);
        genuclientucommon;

        generateutypesufor;              { generate local types   }

        genuparamsufor(list);

{   Local variables:
}
        write(opufile,  '      INTEGER RPC_P_BUF');
        for a := 1 to blkunesting do
            write(opufile, ',RPC_', chr(ord('A') - 1 + a));
        writeln(opufile);

        if butype = functok then begin      { Variable for return value }
            format(expr, 'RPC_RET   ');
            genutypeudeclufor(expr, simpleudescriptor[return], 1);
        end {if};

{   Declaration of RPC_CALL_STATUS function type
}
        if blkustatusuparam <> nil then begin
            writeln(opufile, '      INTEGER*4 RPC_CALL_STATUS');
            writeln(opufile);
        end {if};

{   Code of routine starts here:
}
        write  (opufile, '      CALL RPC_BEGIN_CALL_FOR(RPC_P_BUF, H_');
        writeuname(opufile, unitname);
        writeln(opufile,            ',',
                    blkumaxuin:1,   ',',
                    blkumaxuout:1,  ',',
                    versionunum:1,  ',',
                    procunumber:1,  ')'     );

  {* generate packing statements for IN & INOUT params *}
        if list <> nil then genupackufor(opufile, list, intok, true);

  {* generate rpc_call and bookeeping *}
        if blkustatusuparam <> nil then begin
            write(opufile,'      ');
            writeuname(opufile, blkustatusuparam^.name);
            write(opufile,'=RPC_CALL_STATUS(H_');
        end else if blkucast then begin
            write(opufile, '      CALL RPC_CAST(H_');
        end else begin
            write(opufile, '      CALL RPC_CALL(H_');
        end {if};
        writeuname(opufile, unitname);
        write(opufile, ',RPC_P_BUF');
        if not blkucast then write(opufile, ',', blkutimeout:1);
        writeln(opufile, ')');

        if blkustatusuparam <> nil then begin
            write(opufile,'      IF (MOD(');
            writeuname(opufile, blkustatusuparam^.name);
            writeln(opufile,',2).NE.0) THEN');
        end;

  {* if function: unpack return value *}
        if butype = functok then begin
            packusimpleufor(opufile, return, false);
            writeln(opufile, 'RPC_RET)');
        end {if};

  {* generate UNpacking statements for OUT and INOUT params *}
        if list <> nil then genupackufor(opufile, list, outok, false);

        if blkustatusuparam <> nil
        then writeln(opufile,'      END IF');

  {* clear up everything and exit *}

        writeln(opufile, '      CALL RPC_END_CALL_FOR(RPC_P_BUF)');

        if butype = functok then begin
            write  (opufile, '      ');
            writeuname(opufile, name);
            writeln(opufile,    '=RPC_RET');    { Define function value }
        end {if};
        writeln(opufile, '      RETURN');
        writeln(opufile, '      END');
        writeln(opufile);
    end {with}
 end; {gen_client_block}

 procedure genuopen;
 begin
    if not runoptions[noautoinit].value then writeln(opufile,
                'C     Call this procedure at initialisation time ***');
    write(opufile, '      SUBROUTINE OPEN_');
    writeuname(opufile, unitname);
    writeln(opufile);

    genuclientucommon;

    writeln(opufile,'      INTEGER STATUS');

    write  (opufile,'      CALL RPC_OPEN_FOR(STATUS,H_');
    writeuname(opufile, unitname);
    write  (opufile, ',');
    continuation;
    writeunameupadded(opufile, unitname, '''');
    writeln(opufile, ')');

    writeln(opufile, '      CALL RPC_REPORT_ERROR_FOR(STATUS)');
    writeln(opufile, '      END');
    writeln(opufile);
 end; {GEN_OPEN}

    procedure genuclose;
        begin
        write(opufile, '      SUBROUTINE CLOSE_');
        writeuname(opufile, unitname);
        writeln(opufile);

        genuclientucommon;

        writeln(opufile,'      INTEGER STATUS');

        write  (opufile,'      CALL RPC_CLOSE_FOR(STATUS,H_');
        writeuname(opufile, unitname);
        writeln(opufile, ')');

        writeln(opufile, '      CALL RPC_REPORT_ERROR_FOR(STATUS)');
        writeln(opufile, '      END');
        writeln(opufile);
    end; {GEN_close}

begin {client_gen_for}

    nextulabel := 100;      { An arbitrary starting label }

    write  (opufile,'C     CLIENT STUB routines for package ');
    writeuname(opufile, unitname);
    writeln(opufile);
    writeln(opufile,'C     ====================');
    writeln(opufile,'C');
    writeln(opufile,'C     Generated automatically by the RPC Compiler');
    writeln(opufile,'C     ');

{*      Generate procedures *}

    procunumber := 1;
    scan := blockptr;
    while scan <> nil do begin
        genuclientublock(scan);
        procunumber := procunumber + 1;
        scan := scan^.next;
    end;

    genuopen;
    genuclose;

end; {CLIENT_GEN_for}



{*****************************************************************************

        S E R V E R     C O D E         G E N E R A T O R
        *************************************************

}
procedure serverugenufor;

var     i,                      { Label counter for computed goto }
        procunum :  integer;
        scan:       ptrublockutable;


{_____________________________________________________________________________

        Generate server stub for one procedure
}
 procedure genururoutine;
 var    a:  integer;        { Loop variable }
        expr:   expression; { Expression for the name of the function }
 begin

    write(opufile, '      SUBROUTINE R_');
    writeuname(opufile, scan^.name);
    writeln(opufile, '(RPC_P_BUF)' );

    write(opufile,  '      INTEGER RPC_P_BUF');

    if scan^.butype = functok then begin    { Declare function type }
        writeln(opufile);
        expr.len := 0;
        appenduname(expr, scan^.name);
        genutypeudeclufor(expr, simpleudescriptor[scan^.return], 1);
    end;

    for a := 1 to scan^.blkunesting do
            write(opufile, ',RPC_', chr(ord('A') - 1 + a));
    writeln(opufile);

    generateutypesufor;              { Declare structure types   }

    genuparamsufor(scan^.list);      { Declare types of parameters }

{   Generate UNpacking statements for IN and INOUT params:
}
    if scan^.list <> nil then genupackufor(opufile, scan^.list, intok, false);

  {* generate call and bookeeping *}
    writeln(opufile,    '      CALL RPC_INIT_RETURN_FOR(RPC_P_BUF)');

    if (scan^.butype = proctok)
    and scan^.inuonly
    then
        if (runoptions[concurrent].value or scan^.blkuconcurrent)
           then writeln(opufile, '      CALL RPC_EARLY_RETURN(RPC_P_BUF)')
        else if scan^.blkucast
           then writeln(opufile, '      CALL RPC_NO_RETURN(RPC_P_BUF)');

    write(opufile, '      ');

  {* if function: generate return variable, else just call subroutine.*}
    if scan^.butype = functok
        then packusimpleufor(opufile, scan^.return, true)
        else write(opufile, 'CALL ');
    writeuname(opufile, scan^.name);
    genudummies(scan^.list);
    if scan^.butype = functok then begin
        if (scan^.list=nil) then write(opufile, '()');    { Mandatory }
        write(opufile, ')');
    end {if};
    writeln(opufile);

  {* generate packing statements for OUT and INOUT params*}
    if scan^.list <> nil then genupackufor(opufile, scan^.list, outok, true);

    writeln(opufile, '      END');
    writeln(opufile);                       { Blank line after each one }
 end {gen_r_routine};
{_____________________________________________________________________________}

{       Generate Main Server Procedure
        ------------------------------
}

begin {server_gen_for}

  nextulabel := 100;        { An arbitrary starting label }

  write  (opufile,'C     SERVER STUB routines for package ');
  writeuname(opufile, unitname);
  writeln(opufile);
  writeln(opufile,'C     ====================');
  writeln(opufile,'C');
  writeln(opufile,'C     Generated automatically by the RPC Compiler');
  writeln(opufile,'C     ');

{       Generate individual routines for each procedure:
        -----------------------------------------------
}
  scan := blockptr;
  procunum := 0;
  while scan <> nil do begin
          procunum := procunum + 1; { Count the procedures }
          genururoutine;            { Generate the individual stub subroutines }
          scan := scan^.next;
  end {while};

{       Generate the main server stub subroutine:
        ----------------------------------------
}
  writeln(opufile, 'C');
  writeln(opufile, 'C                         Main stub entry point');
  writeln(opufile, 'C');
  write  (opufile, '      SUBROUTINE R_');
  writeuname(opufile, unitname);
  writeln(opufile, '(RPC_P_BUF)');
  writeln(opufile, '      INTEGER RPC_P_BUF');
  writeln(opufile, '      INTEGER*2 RPC_REQUEST');
  writeln(opufile, '      INTEGER*4 STATUS');
  writeln(opufile, '      INTEGER RPC_S_UNSUPPORTED_VERSION');
  writeln(opufile, '      PARAMETER(RPC_S_UNSUPPORTED_VERSION=139624458)');
  writeln(opufile, '      INTEGER RPC_S_BAD_PROCEDURE_NUMBER');
  writeln(opufile, '      PARAMETER(RPC_S_BAD_PROCEDURE_NUMBER=139624466)');
  writeln(opufile);
  writeln(opufile, '      CALL UPK_SHORT_FOR(RPC_P_BUF, RPC_REQUEST)');
  writeln(opufile, '      IF ((RPC_REQUEST.NE.0).AND.(RPC_REQUEST.NE.',
                            versionunum:1, ')) THEN');
  writeln(opufile,
          '      CALL RPC_SET_ERROR(RPC_P_BUF,RPC_S_UNSUPPORTED_VERSION)');
  writeln(opufile, '      ELSE');
  writeln(opufile, '         CALL UPK_SHORT_FOR(RPC_P_BUF, RPC_REQUEST)');
  writeln(opufile, '         IF ((RPC_REQUEST.LE.0).OR.(RPC_REQUEST.GT.',
                                    procunum:1, ')) THEN');
  writeln(opufile,
        '            CALL RPC_SET_ERROR(RPC_P_BUF,RPC_S_BAD_PROCEDURE_NUMBER)');

  writeln(opufile, '         ELSE');
  write(opufile,   '            GOTO(10');
  for i := 2 to procunum do begin
    if (i mod 10)=0 then begin
        writeln(opufile);
        write(opufile,'     + ');
    end {if};
    write(opufile, ',', i*10:1);
  end {for};
  writeln(opufile, '),RPC_REQUEST');

  scan := blockptr;                     { Make jump table: }
  for i := 1 to procunum do begin
    writeln(opufile);
    write  (opufile,i*10:5, '       CALL R_');
    writeuname(opufile, scan^.name);
    writeln(opufile, '(RPC_P_BUF)');
    writeln(opufile, '            GOTO 888');
    scan := scan^.next;
  end;

  writeln(opufile, '  888       CONTINUE');
  writeln(opufile);
  writeln(opufile, '         END IF');      { If good request }
  writeln(opufile, '      END IF');         { If good version }
  writeln(opufile, '      END');            { subroutine }
  writeln(opufile);

{       Generate Code to Attach stub to RPCRTS
        --------------------------------------
}

  if not runoptions[noautoinit].value then writeln(opufile,
                'C     Call this procedure at initialisation time ***');
  write  (opufile, '      SUBROUTINE ATTACH_');
  writeuname(opufile, unitname);
  writeln(opufile);

  genuserverucommon;

  write(opufile, '      EXTERNAL R_');
  writeuname(opufile, unitname);
  writeln(opufile);

  writeln(opufile, '      INTEGER STATUS');
  writeln(opufile);

  write(opufile, '      CALL RPC_ATTACH_STUB_FOR(STATUS,R_');
  writeuname(opufile, unitname);
  write  (opufile, ',');
  continuation;
  writeunameupadded(opufile, unitname, '''');
  write  (opufile, ',P_');
  writeuname(opufile, unitname);
  writeln(opufile, ')');

  writeln(opufile, '      CALL RPC_REPORT_ERROR_FOR(STATUS)');
  writeln(opufile, '      END');
  writeln(opufile);

{       Generate Code to Detach stub from RPCRTS
        ----------------------------------------
}

  write  (opufile, '      SUBROUTINE DETACH_');
  writeuname(opufile, unitname);
  writeln(opufile);

  genuserverucommon;

  write(opufile,   '      CALL RPC_DETACH_STUB_FOR(P_');
  writeuname(opufile, unitname);
  writeln(opufile, ')');

  writeln(opufile, '      END');

end; {SERVER_GEN_FOR}
{                                                                      CODEPILS
        Top-down RPC compiler:          Code generator for PILS
        ======================          =======================


History:
           Dec 88       PILS version made from FORTRAN versions (LTR)

Requirements of the run-time system:

    1.  rpc_BEGIN_CALL_PILS must
            o   Set up the 'procedure number' and 'version' fields;
            o   Leave the m_index pointer to call_header_length;
    2.  rpc_END_CALL_PILS must dispose of the message
    3.  rpc_CALL must leave m_index set to return_header_length
    4.  On entry into the server stub, m_index must be = call_header_length-4
    5.  The routine rpc_INIT_RETURN_PILS must set it to return_header_length
    6.  pck/upk_string_PILS must pack & unpack strings, and post-align.
    7.  pck/upk_substring_PILS must pack & unpack substrings, and post-align.
    8.  rpc_SET_ERROR must set the m_status field of the message as given.
    9.  rpc_ATTACH_STUB_PILS is a fortran callable rpc_attach_stub
   10.  rpc_OPEN_PILS is a fortran callable rpc_open
   11.  pck/upk_xxxx_PILS are the packing/unpacking routines Fortran callable.

Restrictions:

        No pointers
        No Records
{___________________________________________________________________________

        Write out one identifier
}
procedure writeunameupils(var where: text; var name: iduname);
var
        a:  integer;
begin
    with name do
        for a := 1 to len do
                write(where, str[a]);
end;
{___________________________________________________________________________

        Write out component expression
        ------------------------------
}
procedure writeuexpupils(var where: text; var expr: expression);
var a : integer;
begin
    with expr do
        for a := 1 to len do
                write(where, str[a]);
end;
{___________________________________________________________________________

        Write a quoted string padded to 40 characters
}
procedure writeunameupaddedupils(var opufile:text; name: iduname; ch: char);
begin
  write(opufile, ch);
  writeuname(opufile, name);
  write(opufile, ch:rpcunameulength + 1 - name.len);
end {write_name_padded};

{=============================================================================

                        PILS-sepcific procedures
}
{____________________________________________________________________________

        Code generator error
        --------------------

    Code generator errors are internal errors or target-dependent error.
}
procedure codeuerrorupils(why: errorustring);
begin
    writeln(' RPCC: **** Error: ', why);
    writeln('       (Error detected at PILS code generator stage.)');
    halt;
end;
{___________________________________________________________________________

        Indenting algorithm
}
function indupils(level: integer): integer;
begin
    indupils := (level)*3;
end;


{___________________________________________________________________________

        Append a PILS index to an array expression

    Level = 1       =>      rpc_a   used,
    Level = 2       =>      rpc_b   used etc...
}
procedure appenduindexupils(var expr: expression; level: integer);
var ch: char;
begin
            format(expr, '$(rpc_    ');
            appendu1(expr, chr(ord('a')+level-1));
            appendu1(expr, ')');

end;

{___________________________________________________________________________

      Generate procedure name to (un)pack a simple type or descriptor
      ---------------------------------------------------------------

    Used for the function return result.

|       generates:      upk/pck_ttt(rpc_p_buf,
}
procedure packusimpleupils(var where: text; what : typeutoken; topack: boolean);
begin
    if topack then write(where,'pck_') else write(where, 'upk_');
    case what of
        chartok:        write(where, 'char');
        bytetok:        write(where, 'byte');
        shortok:        write(where, 'short');
        integertok:     write(where, 'integer');
        real32tok:      write(where, 'real32');
        real48tok:      write(where, 'real48');
        real64tok:      write(where, 'real64');
        real128tok:     write(where, 'real128');
        longtok:        write(where, 'long');
    end;
    write(where, '(rpc_p_buf,');
end;

{___________________________________________________________________________

        Generate code to:       Align on a 2**power byte boundary

    Currently, only aligns on word boundaires, because that is all
    we need.
}
procedure genualignupils(    power:  integer;
                            level:  integer);
begin
    writeln(opufile, ' ':indupils(level),
                'rpc_align(rpc_p_buf,', power:1, ')' );
    { Extra runtime support needed here }
end {gen_align_pils};
{___________________________________________________________________________

        Generate code to:       (un)Pack a parameter of given type
        ----------------        ----------------------------------

    This recursive procedure uses the 'level' parameter to control the
    amount of indentation at each stage, and which rpc_<level> parameters
    may be accessed by any given stage.

    If the type is declared as 'EXTERNAL_MARSHALLING, then an external
    procedure pck_<typename> or Upck_<typename> is called to do the
    marshalling.

On entry,
    Output cursor at left margin.
On exit,
    Output cursor at left margin.
}
procedure genupackutypeupils(var where:  text;   { output file }
                            expr:    expression;{ Expression for variable }
                            typ:    ptrudefinedutype;   { type descriptor }
                            level:  integer;    { indentation & rpc_<level> }
                            topack: boolean);   { pack or unpack? }
var ch:     char;
    scan:   ptrunamedutype;
    element:ptrudefinedutype;   { Array element type }
    depth:  integer;        { Depth of nesting of array element }
    exp2:   expression;
    luexpr,                 { Expression for the length of a string, etc }
    suexpr: expression;     { Expression for the start of a substring }
    i:      integer;        { Loop index }

{_____________________________________________________________________________
                    Main block of gen_pack_type_pils
}
begin {gen_pack_type_pils}

   {    Generate a few useful expressions: }

    ch := chr(ord('a')+level-1); {for rpc_a etc }
    luexpr := expr;             {   x }
    prependu1(luexpr, '_');     {  _x }
    suexpr := luexpr;           {  _x }
    prependu1(luexpr, 'L');     { L_x }
    prependu1(suexpr, 'S');     { S_x }

    with typ^ do
    if typuexternal then begin
            write(where, ' ':indupils(level));
            if topack then write(where,'pck_') else write(where, 'upk_');
            writeuname(opufile, typuname^.ntyuname);
            write(where, '(rpc_p_buf,');
            writeuexp(where, expr);
            writeln(where, ')');

    end else case typubasicutype of
{ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                                                        S I M P L E

Here we assume that the names of the pck/upk routines are consistent
with the names rpc_xxx of the basic types. The exception is the code
for unpacking a character, which is going to be a bit weierd, because
of the descriptor.

}
        chartok: begin
            write(where, ' ':indupils(level));
            if topack then write(where,'pck') else write(where, 'upk');
            write(where, '_char_PILS(rpc_p_buf,');
            writeuexp(where, expr);
            writeln(where, ')');
        end {simple type };

        bytetok: begin
            write(where, ' ':indupils(level));
            if topack then write(where,'pck') else write(where, 'upk');
            write(where, '_byte_PILS(rpc_p_buf,');
            writeuexp(where, expr);
            writeln(where, ')');
        end {simple type };

        shortok, integertok,
        real32tok, real48tok,
        real64tok, real128tok,
        longtok: begin
            write(where, ' ':indupils(level));
            if topack then write(where,'pck') else write(where, 'upk');
            with typuname^.ntyuname do
             for i := 4 to len do
              write(where, str[i]);
            write(where, '(rpc_p_buf,');
            writeuexp(where, expr);
            writeln(where, ')');
        end {simple type };

{ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

                                                        A R R A Y
Example:    FOR rpc_a=1,12
                FOR rpc_b=1,10
                        ...myvar(rpc_a,rpc_b)...
                ENDFOR
            ENDFOR

}
        arraytok: begin
            exp2 := expr;
            depth := level;
            ch := chr(ord('a')+depth-1);
            element := typusubtype;
            format(exp2, '$(rpc_    ');
            appendu1(exp2, ch);
            write(where, ' ':indupils(depth));
            writeln(where, 'FOR rpc_', ch, '=1 TO ', typuhigh-typulow+1:1);
            while element^.typubasicutype=arraytok do with element^ do begin
                depth := depth +1;
                ch := chr(ord('a')+depth-1);
                write(where, ' ':indupils(depth));
                writeln(where, 'FOR rpc_', ch, '=1 TO ',typuhigh-typulow+1:1);
                format(exp2, '$,rpc_    ');
                appendu1(exp2, ch);
                element := element^.typusubtype;
            end {while};
            appendu1(exp2, ')');

            genupackutypeupils(where, exp2, element, depth+1, topack);

            while depth>= level do begin
                writeln(where, ' ':indupils(depth), 'ENDFOR');
                depth := depth-1;       { Come back out of nesting }
            end {while};
        end {arraytok};
{ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}
        recordtok: begin                                { R E C O R D }
            codeuerrorupils(
                'No RECORD (STRUCTURE) type exists in PILS!      ');
        end {recordtok};
{ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                                                        A C C E S S

}
        accesstok: begin
            codeuerrorupils(
                'No ACCESS (POINTER) type exists in PILS!        ');
        end {accesstok};

{ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}
        sequence: begin                                 { S E Q U E N C E }
            if level>1 then codeuerrorupils(
                'SEQUENCE type not allowed within composite type.');

            genupackutypeupils(where, luexpr,
                    simpleudescriptor[integertok],level, topack);

            exp2 := expr;
            format(exp2, 'A_$       ');     { Make an expression for the array}
            appenduindexupils(exp2, level);          { Make expression for an element }

            write(where, ' ':indupils(level));
                write(where,    'FOR rpc_', ch, '=1 TO L_');
                writeuexp(where, expr);
                writeln(where);

            genupackutypeupils(where, exp2, typusubtype, level+1, topack);
            writeln(where, ' ':indupils(level), 'ENDFOR');
        end {sequence};

{ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}
        stringtok: begin                                { S T R I N G }

            write(where, ' ':indupils(level));
            if topack
                then    write(where, 'pck_string')
                else    write(where, 'upk_string');
            write(where, '(rpc_p_buf,');
            writeuexp(where, expr);
            writeln(where, ')');

        end {stringtok};

{ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                                                          S U B S T R I N G

}
        substring: begin

            write(where, ' ':indupils(level));
            if topack
                then    write(where, 'pck_substring')
                else    write(where, 'upk_substring');
            write(where, '(rpc_p_buf,A_');
            writeuexp(where, expr);
            write(where, ',');
            write(where,    'S_');
            writeuexp(where, expr);
            write(where,    ',L_');
            writeuexp(where, expr);
            writeln(where,    ')');

        end {substring};

    end {case};
end {gen_pack_type_pils};

{___________________________________________________________________________

        Generate code to:       (un)Pack all parameters necessary
        -----------------       ---------------------------------

The dirn parameter is INtok or OUTok, to specify which parameters to unpack.

A string descriptor needs to be initialised, including the creation of the
string, at the server side, and the string has to be disposed of afterwards.

}
procedure genupackupils( var     where:  text;           { Output file }
                        head:   ptruidlist;     { Parameter list }
                        dirn:   attrutype;      { Parameter filter }
                        topack: boolean);       { Pack, rather than unpack? }

var expr:    expression;
begin {gen_pack_pils}
  repeat
    with head^, idutype^ do begin


        { generate an expression for the variable: }

        expr.len := 0;      { Expression for the variable }
        appenduname(expr, head^.name);    { expr is the param. name }


{       Pack or unpack
        --------------
}
        if (attr = inoutok)
            or (attr = dirn)
        then begin
            deref := client
                    and not (runoptions[byvalue].value   {* not value param *}
                    and (attr = intok));            { and in-only param }

            genupackutypeupils(where, expr, idutype, {level} 1, topack);

        end {if};

    end {with};
    head := head^.next;
  until head = nil;
end; {gen_pack_pils}


{               (end of packing/unpacking)
*****************************************************************************

            Generate:       A type declaration in PILS

    This recursive procedure generates a definition of a type, given
    a name and a type descriptor. It is impossible to generate a type
    itself, as the name is buried within it in PILS.

    Examples
            CHAR    MYSTR(10,2)
                for
                    mystr:  ARRAY[1..10, 1..2] OF STRING

On entry,
    Cursor is assumed to be at left margin.
On exit,
    Cursor is again at the left margin.
}

procedure genutypeudeclupils(expr:   expression;        { for the variable }
                            pt:     ptrudefinedutype;   { type structure }
                            level:  integer);           { indentation }
var     scan:   ptrudefinedutype;
        exp2:   expression;
begin
    with pt^ do case typubasicutype of

        chartok,                    { First token, simple type, courier type }
        bytetok,
        shortok, integertok,
        real32tok, real48tok,
        real64tok, real128tok,
        longtok: begin
            { write(op_file, ' ':ind_pils(level)); }
            case typubasicutype of
                chartok:        write(opufile, 'int16');
                bytetok:        write(opufile, 'int16');
                integertok:     write(opufile, 'INT');
                real32tok:      write(opufile, 'REAL');
                real48tok:      write(opufile, 'REAL');
                real64tok:      write(opufile, 'REAL');
                real128tok:     write(opufile, 'REAL');
                shortok:        write(opufile, 'INT16');
                longtok:        write(opufile, 'INT32');
            end {case};
            write(opufile, ' ');
            writeuexp(opufile, expr);   { Example: rpc_char x }
        end {simple type};

        substring,
        stringtok:  begin               { Example: rpc_char x[81] }
                { write(op_file, ' ':ind_pils(level)); }
                write(opufile, 'CHAR ');
                writeuexp(opufile, expr);
        end {stringtok};

        sequence,
        arraytok:   begin               { example  INT MYARR(10,20) }
                exp2 := expr;
                appendu1(exp2,'(');
                appendudecimal(exp2,typuhigh-typulow+1);
                scan := typusubtype;
                while scan^.typubasicutype = arraytok do begin
                    appendu1(exp2,',');
                    appendudecimal(exp2,scan^.typuhigh-typulow+1);
                    scan := scan^.typusubtype;
                end {while};
                appendu1(exp2,')');
                genutypeudeclupils(exp2, scan, level); {non-array type}
        end {arraytok};

        accesstok: begin
            codeuerrorupils(
                'Package has pointers: Can''t make PILS stubs.    ');
        end;

        recordtok: begin
            codeuerrorupils(
                'Package has records:  Can''t make PILS stubs.    ');
        end;

    end {with, case};
end {gen_type_decl_pils};

{_____________________________________________________________________________

        Generate local variables (server) / parameter (client)
        ------------------------------------------------------

This subroutine will generate

    -  The parameters to the client subroutine,
    -  or the local variables in the server.

}

procedure genuparamsupils(head: ptruidlist);
var
        auexpr,
        luexpr,
        suexpr,
        expr:       expression;
        a:          integer;
        scan:       ptruidlist;
        ch:         char;
        simpleuref:  boolean;   { Should simple variables be dereferenced? }
        compositeuref: boolean; { what about composite variables? }

begin {gen_params_pils}

    scan := head;

    if client then
        write(opufile, '(');

    while scan <> nil do begin

    {* Write the parameter attribute *}

    if client then
        case scan^.attr of
           intok: write(opufile, 'IN ');
           outok: write(opufile, 'OUT ');
           inoutok: write(opufile, 'INOUT ');
        end {CASE};

        {* Write the type's name if any otherwise the full declaration *}
        {* Sequences and substrings MUST be expanded a little *}

        with scan^.idutype^ do begin

            expr.len := 0;
            appenduname(expr, scan^.name);
            luexpr := expr;             {   x }
            prependu1(luexpr, '_');     {  _x }
            suexpr := luexpr;           {  _x }
            auexpr := luexpr;           {  _x }
            prependu1(luexpr, 'L');     { L_x }
            prependu1(suexpr, 'S');     { S_x }
            prependu1(auexpr, 'A');     { A_x }

            case typubasicutype of

            chartok,
            bytetok,
            shortok, integertok,
            real32tok, real48tok,
            real64tok, real128tok,
            longtok,
            stringtok,
            recordtok,
            arraytok:
                genutypeudeclupils(expr, scan^.idutype, 1);

            sequence:
                begin                                      { a_xxx: tttt; }
                    genutypeudeclupils(auexpr, scan^.idutype, 1);
                                                     { l_xxx: rpc_integer }
                    if client then write(opufile, ',')          { PILS }
                         else write(opufile, ';');              { PILS }

                    genutypeudeclupils(luexpr,
                        simpleudescriptor[integertok], 1);
                end;

            substring:
                begin                                      { a_xxx: tttt; }
                    genutypeudeclupils(auexpr, scan^.idutype, 1);

                    if client then write(opufile, ',')          { PILS }
                         else write(opufile, ';');              { PILS }

                    genutypeudeclupils(suexpr,
                        simpleudescriptor[integertok], 1);

                    if client then write(opufile, ',')          { PILS }
                         else write(opufile, ';');              { PILS }

                    genutypeudeclupils(luexpr,
                        simpleudescriptor[integertok], 1);
                end;
            end {case};

        end; {with}
        scan := scan^.next;

       if scan <> nil then
            if client then write(opufile, ',')          { PILS }
                 else write(opufile, ';');              { PILS }

   end {while};

   if client then write(opufile, ')');                  { PILS }

   writeln(opufile);

end; {GEN_PARAMS_PILS}

{____________________________________________________________________________

            Generate Formal Parameter List
}
procedure genuformals(head: ptruidlist);

  { Generates the formal parameter list enclosed in brackets  (a,b,c,d)  }
var
        scan:       ptruidlist;

begin {gen_formals}

     if head <> nil then begin

        scan := head;
        write(opufile, '(');
        while scan <> nil do begin
            if scan^.idutype^.typubasicutype in [sequence, substring]
                    then write(opufile, 'A_');
            writeuname(opufile, scan^.name);
            if scan^.idutype^.typubasicutype = substring then
            begin
                write(opufile, ', S_');
                writeuname(opufile, scan^.name);
            end;
            if scan^.idutype^.typubasicutype in [sequence, substring] then
            begin
                write(opufile, ', L_');
                writeuname(opufile, scan^.name);
            end;

            scan := scan^.next;
            if scan <> nil then
                write(opufile, ',');
        end {while};
        write(opufile, ')');
     end {if list not NIL};
end; {gen_formals}

{_________________________________________________________________________

            Generate:   A structure declaration in PILS
            ==============================================

Example:    STRUCTURE /structurename/
                CHARACTER*80    fieldname
                INTEGER         fieldname2
            END STRUCTURE
}
procedure genustructupils(   pt:     ptrudefinedutype;   { type structure }
                            level:  integer);           { indentation }

var     scan:   ptrunamedutype;
        exp2:   expression;
begin

      { writeln(op_file, '!-- Sorry, No PILS Structures !!!'); }

end {gen_struct_pils};
{___________________________________________________________________________

        Generate type definitions for whole package
        ===========================================

Example:
            INT         P1
            INT         P2(3,3)
            CHAR        P3(10)
}

procedure generateutypesupils;

begin {generate types}

      { writeln(op_file, '!-- Sorry, No PILS User Defined Types !!!'); }

end;

{******************************************************************************

                Client code generator
                =====================

Generates one procedure or function for the client.
}

procedure clientugeneratorupils;
var
        scan :          ptrublockutable;
        procunumber :   integer;

  procedure genuclientublock(ptr: ptrublockutable);

{***
* On each block, the client allocates 'rpc_integer' variables to pack or
* unpack parameters.
****}
  var
        a:              integer;
        expr:           expression;     { Expression for return value }

 begin {GEN_client_BLOCK}
    with ptr^ do begin
{   Generate Header:
}
        if butype = functok then begin
            write(opufile, 'DEF ');
            case return of
                chartok:        write(opufile, 'int16');
                bytetok:        write(opufile, 'int16');
                shortok,
                integertok:     write(opufile, 'INT16');
                longtok:        write(opufile, 'INT32');
                real32tok,
                real48tok,
                real64tok,
                real128tok:     write(opufile, 'REAL');
            end {case};
            write(opufile, ' ');
            writeuname(opufile, name);
        end else begin
            write(opufile, 'SUB ');
            writeuname(opufile, name);
        end {if};

        genuparamsupils(list);      { Generate formal parameter list }

        writeln(opufile);

{   Local variables:
}
        write(opufile,  '   INT rpc_p_buf');
        for a := 1 to blkunesting do
            write(opufile, ',rpc_', chr(ord('a') - 1 + a));
        writeln(opufile);

        if butype = functok then begin      { Variable for return value }
            writeln(opufile, '   INT rpc_ret');
            {
            format(expr, 'rpc_ret   ');
            gen_type_decl_pils(expr, simple_descriptor[return], 1);
            }
        end {if};
        writeln(opufile);

{   Code of routine starts here:
}
        write  (opufile, '   rpc_begin_call(rpc_p_buf,h_');
        writeuname(opufile, unitname);
        writeln(opufile,            ',',
                    blkumaxuin:1,   ',',
                    blkumaxuout:1,  ',',
                    versionunum:1,  ',',
                    procunumber:1,  ')'     );

  {* generate packing statements for IN & INOUT params *}
        if list <> nil then genupackupils(opufile, list, intok, true);

  {* generate rpc_call and bookeeping *}
        if blkucast then write(opufile, '   rpc_cast(h_')
                    else write(opufile, '   rpc_call(h_');
        writeuname(opufile, unitname);
        write(opufile, ',rpc_p_buf');
        if not blkucast then write(opufile, ',', blkutimeout:1);
        writeln(opufile, ')');

  {* if function: unpack return value *}
        if butype = functok then begin
            write(opufile, '   ');
            packusimpleupils(opufile, return, false);
            writeln(opufile, 'rpc_ret)');
        end {if};

  {* generate UNpacking statements for OUT and INOUT params *}
        if list <> nil then genupackupils(opufile, list, outok, false);

  {* clear up everything and exit *}

        writeln(opufile, '   rpc_end_call(rpc_p_buf)');

        if butype = functok then begin
            write  (opufile, '   ');
            writeuname(opufile, name);
            writeln(opufile,    '=rpc_ret');    { Define function value }
        end {if};
        write(opufile, 'END');
        if butype = functok then write(opufile, 'DEF')
            else write(opufile, 'SUB');
        writeln(opufile);

        writeln(opufile);
    end {with}
 end; {gen_client_block}

procedure genuclientuheader;

{
Generates:      The CLIENT module header
                The constant and type definitions
}
begin {GEN_CLIENT_HEADER}

    writeln(opufile,'!---');
    write  (opufile,'!---  CLIENT STUB routines for package ');
    writeuname(opufile, unitname);
    writeln(opufile);
    writeln(opufile,'!---');
    writeln(opufile,'!---  Generated automatically by the RPC Compiler');
    writeln(opufile,'!---');

    writeln(opufile);
    writeln(opufile, 'OPTION IMPLICIT OFF');
    writeln(opufile, 'OPTION BASE 1');

    write(opufile, 'MODULE CLI');
    writeunameupils(opufile, unitname);
    writeln(opufile);
    writeln(opufile);

    write(opufile, 'GLOBAL INT h_');
    writeunameupils(opufile, unitname);
    writeln(opufile);
    writeln(opufile);

    { generate_externals; }

end; {GEN_CLIENT_HEADER}

 procedure genuopen;
 begin
    write(opufile, 'SUB open_');
    writeunameupils(opufile, unitname);
    writeln(opufile, '()');
    writeln(opufile, '   INT status');
    writeln(opufile, '   CHAR service');
    writeln(opufile);

    write(opufile, '   service = ');
    writeunameupaddedupils(opufile, unitname, '"');
    writeln(opufile);

    write(opufile, '   rpc_open(status,h_');
    writeunameupils(opufile, unitname);
    writeln(opufile, ',service)');

    writeln(opufile, '   rpc_report_error(status)');
    writeln(opufile, 'ENDSUB');
    writeln(opufile);
    if not runoptions[noautoinit].value then
         begin
         writeln(opufile,
         '!---  This procedure will be called at initialisation time ***');
         write(opufile, 'open_');
         writeunameupils(opufile, unitname);
         writeln(opufile, '()');
         writeln(opufile);
         end;

 end; {GEN_OPEN}

begin {client_generator_pils}

    genuclientuheader;

{*      Generate procedures *}

    procunumber := 1;
    scan := blockptr;
    while scan <> nil do begin
        genuclientublock(scan);
        procunumber := procunumber + 1;
        scan := scan^.next;
    end;

    genuopen;

    writeln(opufile, 'ENDMODULE');

end; {CLIENT_GEN_pils}



{*****************************************************************************

        S E R V E R     C O D E         G E N E R A T O R
        *************************************************

}
procedure serverugeneratorupils;

var     i,                      { Label counter for computed goto }
        procunum :  integer;
        scan:       ptrublockutable;


{_____________________________________________________________________________

        Generate server stub for one procedure
}
 procedure genururoutine;
 var a,b : integer;
 begin

    write(opufile, 'SUB r_');
    writeuname(opufile, scan^.name);
    writeln(opufile, '(INOUT INT rpc_p_buf)' );

    generateutypesupils;              { Declare structure types   }

    genuparamsupils(scan^.list);      { Declare types of parameters }
    writeln(opufile);

    b := scan^.blkunesting;
    if b > 0 then write(opufile, '   INT rpc_a');
    for a := 2 to b do
            write(opufile, ',rpc_', chr(ord('a') - 1 + a));
    writeln(opufile);
    writeln(opufile);

{   Generate UNpacking statements for IN and INOUT params:
}
    if scan^.list <> nil then genupackupils(opufile, scan^.list, intok, false);

  {* generate call and bookeeping *}
    writeln(opufile,'   rpc_init_return(rpc_p_buf)');

    if  (scan^.butype = proctok)
        and scan^.inuonly
    then if ((runoptions[concurrent].value) or scan^.blkuconcurrent)
          then writeln(opufile, '   rpc_early_return(rpc_p_buf)')
         else if scan^.blkucast
          then writeln(opufile, '   rpc_no_return(rpc_p_buf)');

  {* if function: generate return variable, else just call subroutine.*}
    write(opufile, '   ');
    if scan^.butype = functok
        then packusimpleupils(opufile, scan^.return, true);
    writeuname(opufile, scan^.name);
    genuformals(scan^.list);
    if scan^.butype = functok then write(opufile, ')');
    writeln(opufile);

  {* generate packing statements for OUT and INOUT params*}
    if scan^.list <> nil then genupackupils(opufile, scan^.list, outok, true);

    writeln(opufile, 'ENDSUB');
    writeln(opufile);                       { Blank line after each one }
 end {gen_r_routine};
{_____________________________________________________________________________}

{       Generate Code to Attach stub to RPCRTS
        --------------------------------------
}

procedure genuattach;
begin

  if not runoptions[noautoinit].value then writeln(opufile,
                '!---  Call this procedure at initialisation time ***');
  write(opufile, 'SUB attach_');
  writeunameupils(opufile, unitname);
  writeln(opufile, '()');

  writeln(opufile, '   INT status');
  writeln(opufile, '   CHAR service');
  writeln(opufile);
  write (opufile,  '   service = ');
  writeunameupaddedupils(opufile, unitname, '"');
  writeln(opufile);

  write(opufile, '   rpc_attach_stub(status,address(r_');
  writeunameupils(opufile, unitname);
  writeln(opufile, '),service,program_number)');

  writeln(opufile, '   rpc_report_error(status);');

  writeln(opufile, 'ENDSUB');
  writeln(opufile);

end; {GEN_ATTACH}

{_____________________________________________________________________________

        Generate Code to Detach stub from RPCRTS
        ----------------------------------------
}

procedure genudetach;
begin

  if not runoptions[noautoinit].value then writeln(opufile,
                '!---  Call this procedure at exit time ***');
  write(opufile, 'SUB detach_');
  writeunameupils(opufile, unitname);
  writeln(opufile, '()');
  writeln(opufile);

  writeln(opufile, '   rpc_detach_stub(program_number)');

  writeln(opufile, 'ENDSUB');
  writeln(opufile);

end; {GEN_DETACH}

{       Generate Main Server Procedure
        ------------------------------
}

begin {server_generator_pils}

  writeln(opufile,'!---');
  write  (opufile,'!---  SERVER STUB routines for package ');
  writeuname(opufile, unitname);
  writeln(opufile);
  writeln(opufile,'!---');
  writeln(opufile,'!---  Generated automatically by the RPC Compiler');
  writeln(opufile,'!---');

  writeln(opufile);
  writeln(opufile, 'OPTION IMPLICIT OFF');
  writeln(opufile, 'OPTION BASE 1');

  write(opufile, 'MODULE SER');
  writeunameupils(opufile, unitname);
  writeln(opufile);
  writeln(opufile,'HIDDEN INT program_number');
  writeln(opufile);

{       Generate individual routines for each procedure:
        -----------------------------------------------
}
  scan := blockptr;
  procunum := 0;
  while scan <> nil do begin
          procunum := procunum + 1; { Count the procedures }
          genururoutine;            { Generate the individual stub subroutines }
          scan := scan^.next;
  end {while};

{       Generate the main server stub subroutine:
        ----------------------------------------
}
  writeln(opufile, '!---');
  writeln(opufile, '!---        Main stub entry point');
  writeln(opufile, '!---');
  write  (opufile, 'SUB r_');
  writeuname(opufile, unitname);
  writeln(opufile, '(INOUT INT rpc_p_buf)');
  writeln(opufile, '   INT16 rpc_request');
  writeln(opufile);
  writeln(opufile, '   upk_short(rpc_p_buf,rpc_request)');
  if versionunum <> 0 then begin
        writeln(opufile, '   IF (rpc_request # 0) AND (rpc_request # ',
                           versionunum:1, ') THEN');
        writeln(opufile,
            '      rpc_set_error(rpc_p_buf,rpc_s_unsupported_version)');
        writeln(opufile, '   ELSE');
  end {if version number used };
  writeln(opufile, '      upk_short(rpc_p_buf,rpc_request)');

  writeln(opufile, '      SELECT rpc_request');

  scan := blockptr;
  procunum := 1;
  while scan <> nil do
  begin
      writeln(opufile, '      CASE ',procunum:2);
      write(opufile,   '         r_');
      writeunameupils(opufile, scan^.name);
      writeln(opufile, '(rpc_p_buf)');
      procunum := procunum + 1;
      scan := scan^.next;
  end;

  writeln(opufile, '      CASE ELSE ');
  write(  opufile, '         ');
  writeln(opufile, 'rpc_set_error(rpc_p_buf,rpc_s_bad_procedure_number)');
  writeln(opufile, '      ENDSELECT');
  if versionunum <> 0 then writeln(opufile, '   ENDIF');
  writeln(opufile, 'ENDSUB');
  writeln(opufile);

{       Generate Code to Attach stub to RPCRTS
        --------------------------------------
}
  genuattach;
  genudetach;

  writeln(opufile, 'ENDMODULE');

end; {SERVER_GEN_PILS}
{                                                                CODEGEN
        Top-down RPC compiler:          Code generator
        ======================          ==============


History:
        3 Jun 1986      Written, Antonio Pastore, Tec.Student 1986, DD/OC, CERN
        15 Aug 86       TBL  Bug fix in gen_loc_type
        18 Aug 86       length follows array param in sequence, like string.
        19 Aug 86       VAX strings take descriptor only. New & dispose in.
         2 Sep 86       CALL_MESSAGE bug fix -- should be CALL_HEADER_LENGTH
        15 Oct 86       CALL_HEADER_LENGTH -2 used, as c_h_l includes proc#.
        23 Oct 86       m_index used insted of _count: reentrant code.
                        Local variables preceded by underline.
                        Server procedures are all local to main jump procedure
        31 Oct 86       rpc_integer type introduced and used for substrings
         8 Nov 86       Bug fix in substring handling: packed l-s bytes 61108
        17 Dec 86       M6809 addition
         6 Jan 87       Bug fix
        30 Jan 87       Fixes for M6809; externals inline in client now.
         9 Mar 87       Timeout parameter added to rpc_call
        20 Mar 87       Rename: pack_xxx -> pck_xxx, unpack_xxx -> upk_xxx
         4 Jun 87       handle for 6809 in external, not entry
        14 Aug 87       Code reworked to include C stub generation - Nici.
        17 Aug 87       Option byvalue added - Nici.
        27 Aug 87       Options timeout and version added - Nici.
        31 Aug 87       C bug fix: added brackets for (un)packing arrays
                        and sequences (macro expansion!) - Nici.
         1 Sep 87       Added options types and <s,c>pcturbo - Nici.
         6 Oct 87       With clause does not extend round rpc_call (71006)
        17 Dec 87       C stub is passed **buffer, not *buffer. TBL (71217)
        23 Jun 88       C client with no args omitted the '()': fixed. TBL
                        C stubs #include with <brackets> now.
        26 Aug 88       Options <s,c>macturbo added.    (Roberto Bagnara, DD/OC)
                        PCTurbo client stubs now automatically initializes the
                        exit procedure mechanism even if NOAUTOINIT is required,
                        thus avoiding a system crash at program termination.
                        PCTurbo server stubs are now generated automatically,
                        Attach_XXX procedure included.
        30 Aug 88       Unused variables (ch : char) removed in gen_exit,   (RB)
                        gen_open and gen_attach. Modified also gen_attach to
                        declare prog_no of type program_pointer (MacTurbo and
                        PCTurbo) instead of program_index (all the other output
                        modes).
         1 Sep 88       Close_XXX procedure generated correctly also for    (RB)
                        MacTurbo client stubs. In PCTurbo and MacTurbo
                        stubs as well as in the .EXT file, the TYPE keyword
                        is not generated if there are no type declarations.
           Nov 88       Composite types and pointers; arrays of named types.
        21 Mar 89       VAX/VMS fortran compat. uses RPC_RTS_FOR for strings.
         8 May 89       close_xxx, attach_xxx, p_xxx in response to requests
         7 Jun 89       Substring start & length run 1..n in C now, not 0..n-1.
        13 Jun 89       In C, sequence size was one too big. (TBL, FC)
        10 Jul 89       String[] types restored for Turbopascal and M6809
        13 Oct 89       MAXLENGTH becomes RPC_BUFFER_SIZE (clash with PILS)
        30 Nov 89       Bug fix: string length in vaxpas is short, not int.
         6 Dec 89       Bug fix: handle was declared int in client.
         5 Feb 90       Change rpcheader.h to rpcrts.h for VM/CMS length limit.
        28 Mar 90       Buffer fragmentation code put in. rpc_begin() used.
}
{____________________________________________________________________________

        Code generator error
        --------------------

    Code generator errors are internal errors or target-dependent error.
}
procedure codegenuerror(why: errorustring);
begin
    writeln(' RPCC: **** Error: ', why);
    writeln('       (Error detected at code generator stage.)');
    halt;
end;
{____________________________________________________________________________
}
{       Write language elements to file:
        -------------------------------
}
procedure writok(var where: text; what: typeutoken);
begin
    case what of
        chartok :       write(where, 'rpc_char');
        bytetok :       write(where, 'rpc_byte');
        shortok :       write(where, 'rpc_short');
        integertok:     write(where, 'rpc_integer');
        real32tok:      write(where, 'rpc_real32');
        real48tok:      write(where, 'rpc_real48');
        real64tok:      write(where, 'rpc_real64');
        real128tok:     write(where, 'rpc_real128');
        longtok :       write(where, 'rpc_long');
        arraytok,
        sequence,
        stringtok,
        substring:      begin
                            if omode in cmode then write(where, '[')
                                else write(where, 'ARRAY [1..');
                        end;
        proctok :       if not (omode in cmode) then write(where, 'PROCEDURE ');
        functok :       if not (omode in cmode) then write(where, 'FUNCTION ');
    end;
end;
{___________________________________________________________________________

        Indenting algorithm
}
function leftin(level: integer): integer;
begin
    leftin := level*3
end;

{___________________________________________________________________________
}
procedure writeudeclaration(var where: text; prefix: char;
                            ptr: ptruidlist; ref: boolean);
begin
    with ptr^ do
    begin
        if not (omode in cmode) then begin
            if ref then write(where, 'VAR ');
            if prefix <> ' ' then write(where, prefix, '_');
            writeuname(where, name);
            write(where, ': ');
        end;

        with idutype^ do
            if typuname <> nil then writeuname(where, typuname^.ntyuname)
            else codegenuerror(
                'Internal error: Unnamed type when name needed!  ');

        if omode in cmode then begin
            write(where, ' ');
            if ref then write(where, '*');
            if prefix <> ' ' then write(where, prefix, '_');
            writeuname(where, name);
        end
    end;
end;
{___________________________________________________________________________

        Generate:   Declatation of an integer
}
procedure writeuinteger(var where:  text;       { output file }
                            prefix: char;       { Space, or l or s for example }
                        var name:   iduname;    { Variable name }
                            ref : boolean);     { Pass by reference? }
begin
    write(where, '    ');
    if omode in cmode then write(where, 'rpc_integer ');
    if ref then
        if omode in cmode then write(where, '*')
            else write(where, 'VAR ');
    if prefix <> ' ' then write(where, prefix, '_');
    writeuname(where, name);
    if not (omode in cmode) then write(where, ' : rpc_integer');
end;

{___________________________________________________________________________

        Find out whether type needs dynamic allocation
        ----------------------------------------------

  This function returns whether a type needs memory allocation performing.
  In general, pointer types are dynamic, and composite types are dynamic
  if they contain any dynamic types.

  The rules applied to pointers depend on the direction of the parameter
  and are summarised in the following table.

  IN parameters are straightforward, in that they are allocated by the
  client, and temporarily reconstructed on the server node by the stub.

  For INOUT parameters, the client must ensure that no other pointers are kept
 to items in the original tree, as it will be deallocated by the stub. The
 server must ensure that any items removed from the tree are returned to the
 heap, and any added to the tree are taken from the heap.

  For OUT parameters, the client's pointer is overwritten and its original
  value lost, without any deallocation of any items to which it may have
  pointed. The new tree is allocated by the stub from the heap, and the
  client is responsible for eventually deallocating it.


                CLIENT                      SERVER
    ______      _________________________   ______________________________
    IN          allocated by client
                MARSHALLED
                                            ALLOCATED DURING UNMARSHALLING
                                            untouched by server
                                            DEALLOCATED *

    IN OUT      allocated by client
                MARSHALLED
                DEALLOCATED
                                            ALLOCATED DURING UNMARSHALLING
                                            modified by server
                                            MARSHALLED
                                            DEALLOCATED
                ALLOCATED DURING UNMARSHALLING

    OUT         assumed unallocated
                                            set to NIL
                                            allocated by server
                                            MARSHALLED
                                            DEALLOCATED
                ALLOCATED DURING UNMARSHALLING

    Unmarshalling always implies allocation and vice-versa.
    Marshalling implies deallocation EXCEPT for in parameters,
    and deallocation has to be done (sometimes) on it's own.


 String descriptors contain pointers which are different in that they are
 allocated (normally statically) by the client in all cases:

String          CLIENT                      SERVER
 descriptors:   _________________________   ________________________________

                (MARSHALLED)
                                            ALLOCATED *
                                            (UNMARSHALLED)
                                            (MARSHALLED)
                                            DEALLOCATED *
                (UNMARSHALLED)

(De)Allocation has to be done independently of [un]marshalling,
in cases marked "*". In this case, a procedure is needed to check a composite
type to see whether it is going to need allocation or deallocation. This is
done by "contains"

____________________________________

    Check to see whether given types occur anywhere in a composite type:
}
function contains(pt: ptrudefinedutype; typeset: setuofutoken): boolean;
var scan:   ptrunamedutype;
begin
    with pt^ do begin
    contains := typubasicutype in typeset;
    case typubasicutype of

        chartok,
        bytetok,
        shortok, integertok,
        real32tok, real48tok,
        real64tok, real128tok,
        longtok,
        substring,
        sequence,
        stringtok:          { ok };

        accesstok,
        arraytok: if contains(typusubtype, typeset) then contains := true;

        recordtok: begin
            scan := typufields;
            while scan <> nil do with scan^ do begin
                if contains(ntyutype, typeset) then contains := true;
                scan := ntyunext;
            end {while};
        end {recordtok};

     end {case};
    end {with}
end; {contains}
{_____________________________________________________________________________

        Generate external definitions of external marshalling routines
        --------------------------------------------------------------

    Should only be called if it is appropriate for the output mode
    (Not pcturbo, macturbo)

Example (vaxvms):
|
|               procedure upk_mytype(var rpc_p_buf: rpc_message_pointer;
|                   var param: mytype); external;
|               procedure pck_mytype(var rpc_p_buf: rpc_message_pointer;
|                   param: mytype); external;
}
procedure decluextumarshal;
var pnt:    ptrunamedutype;
    topack: boolean;
begin
    pnt := typeptr;
    while pnt <> nil do begin
        if pnt^.ntyutype^.typuexternal then
        for topack := false to true do begin
            if omode in cmode then begin
                write(opufile, 'extern ', upkpck[topack], '_');
                writeuname(opufile, pnt^.ntyuname);
                writeln(opufile, '();');
            end else begin
                write(opufile, 'procedure ', upkpck[topack], '_');
                writeuname(opufile,  pnt^.ntyuname);
                writeln(opufile, '(var rpc_p_buf: rpc_message_pointer;');
                write(opufile, '    ');
                if not topack then write(opufile, 'var ');
                write(opufile, 'param: ');
                writeuname(opufile,  pnt^.ntyuname);
                if (omode in [m6809])
                    then writeln(opufile, '); EXTERNAL; ')
                    else writeln(opufile, '); EXTERN; ');
            end {if pascal}
        end {if for};
        pnt := pnt^.ntyunext;
    end {while};
end {decl_ext_marshal};


{_____________________________________________________________________________

                Perform dynamic allocation/deallocation
                ---------------------------------------

    This procedure will allocate and deallocate VMS strings, and it will
    DEallocate pointers. It won't allocate pointers (this is done during
    unmarshalling);

    Note that in the case of an array, we look ahead to see whether the subtype
    is going to need allocation, before producing the code to do the loop.

Pascal:         new(p);                             dispose(p);

C:              [not handled]                       free(p);
            p = ( t * ) malloc(sizeof(t));
}
procedure genuallocate( var where:  text;
                        expr:   expression;         { ... for the variable }
                        pt:     ptrudefinedutype;
                        level:  integer;            { nesting so far }
                        doualloc: boolean;          { T: all, F: deallocate }
                        typeset:    setuofutoken);   { types to allocate }

var scan:   ptrunamedutype;
    exp2:   expression;
    ch:     char;

begin
    ch := chr(ord('a')+level-1);
    with pt^ do
     case typubasicutype of

        chartok,                    { First token, simple type, courier type }
        bytetok,
        shortok, integertok,
        real32tok, real48tok,
        real64tok, real128tok,
        longtok,                    { Last simple type }
        substring:          { Do nothing };

{   Pointers are deallocated after having recursively deallocated anything
    they point to.
}
        accesstok: if typubasicutype in typeset then begin
            if doualloc then codegenuerror(
                    'Internal:  Call GEN_ALLOCATE to allocate pointer');
            exp2 := expr;
            if omode in cmode then begin
                write  (where, ' ':leftin(level),   'if ('              );
                writeuexp(where,                    expr                );
                write(where,                        ') {'               );
                format(exp2, '(*$)      ');
                genuallocate(where, exp2, typusubtype,
                        level+1, doualloc, typeset);
                write(where, ' ':leftin(level+1),   'free('             );
                writeuexp(where,                    expr                );
                writeln(where,                      ');'                );
                writeln(where, ' ':leftin(level),   '} /* end if */'    );
            end else begin
                write  (where, ' ':leftin(level),   'if ');
                writeuexp(where,                    expr                );
                writeln(where,                      ' <> NIL then begin');
                format(exp2, '$^        ');
                genuallocate(where, exp2, typusubtype,
                        level+1, doualloc, typeset);
                write(where, ' ':leftin(level+1),   'dispose('          );
                writeuexp(where,                    expr                );
                writeln(where, ' ':leftin(level+1), ');'                );
                writeln(where, ' ':leftin(level),   'end {if};'         );
            end {pascal};
        end;

        stringtok: if (omode=vaxvms) and (stringtok in typeset) then begin
            if doualloc then begin
                  write  (where, '   with ');
                  writeuexp(where, expr);
                  writeln(where, ' do begin new(StrAdr); strlen :=',
                          typuhigh:1, '; { Init. string descriptor }');
                  writeln(where, '     DType := 0; Cont := 0; end;'); {ok?@@@}
            end else begin
                write(where, ' ':leftin(level), 'dispose(');
                writeuexp(where, expr);
                writeln(where,  '.StrAdr);');
            end;

        end {stringtok};

        arraytok: if contains(typusubtype, typeset) then begin
            exp2 := expr;
            appenduindex(exp2, level);

            write(where, ' ':leftin(level));
            if not (omode in cmode)
            then begin
                writeln(where, 'FOR rpc_', ch, ' := ',
                        typulow:1, ' TO ', typuhigh:1, ' DO BEGIN')
            end else begin
                write(where, 'for (rpc_', ch, ' = 0; rpc_', ch, '<',
                         typuhigh - typulow + 1:1, '; rpc_', ch, '++) {');
            end {if};

            genuallocate(where, exp2, typusubtype, level+1, doualloc, typeset);

            if omode in cmode
            then writeln(where, ' ':leftin(level), '      }')
            else writeln(where, ' ':leftin(level), 'END {FOR};');
        end {arraytok};

        recordtok: begin
            scan := typufields;
            while scan <> nil do with scan^ do begin
                exp2 := expr;
                appendu1(exp2,'.');
                appenduname(exp2, ntyuname);
                genuallocate(where, exp2, ntyutype, level, doualloc, typeset);
                scan := ntyunext;
            end {while};
        end {recordtok};

     end {with, case};
end; {gen_allocate}

{___________________________________________________________________________

      Generate procedure name to (un)pack a simple type or descriptor
      ---------------------------------------------------------------

    Used for the function return result.

|       generates:      upk/pck_ttt(rpc_p_buf,
}
procedure dopackusimple(var where: text; what : typeutoken; topack: boolean);
begin
    write(where, upkpck[topack], '_');
    case what of
        chartok:        write(where, 'char(');
        bytetok:        write(where, 'byte(');
        shortok:        write(where, 'short(');
        integertok:     write(where, 'integer(');
        real32tok:      write(where, 'real32(');
        real48tok:      write(where, 'real48(');
        real64tok:      write(where, 'real64(');
        real128tok:     write(where, 'real128(');
        longtok:        write(where, 'long(');
    end;
    write(where, 'rpc_p_buf, ');
end;

{___________________________________________________________________________

        Generate code to:       Align on a 2**power byte boundary

    Currently, only aligns on word boundaires, because that is all
    we need.
}
procedure genualign(    var where:  text;
                            power:  integer);
begin
            if power <> 1 then codegenuerror(
                'Internal error: Gen_align on non-word boundary! ');

            if odd(sizeusoufar) then sizeusoufar := sizeusoufar+1;

            if not (omode in cmode) then begin
                writeln(where,
                        '    IF ODD(m_index) THEN m_index:=m_index+1;');
            end else begin
                writeln(where,
                    '    if (rpc_p_buf->m_index%2) rpc_p_buf->m_index++;');
            end {if};

end {gen_align};

{___________________________________________________________________________

        Generate code to:       (un)Pack a parameter of given type
        ----------------        ----------------------------------

    This recursive procedure uses the 'level' parameter to control the
    amount of indentation at each stage, and which rpc_<level> parameters
    may be accessed by any given stage.

On entry,
    Output cursor at left margin.
On exit,
    Output cursor at left margin.
}
procedure genupackutype(var where:  text;       { output file }
                            expr:    expression;{ Expression for variable }
                            typ:    ptrudefinedutype;   { type descriptor }
                            level:  integer;    { indentation & rpc_<level> }
                            deref:  boolean;    { Dereference simple types? }
                            topack: boolean;    { pack or unpack? }
                            checkusize: boolean);{ check for buffer overflow? }
var ch:     char;
    scan:   ptrunamedutype;
    exp2:   expression;
    luexpr,                 { Expression for the length of a string, etc }
    suexpr: expression;     { Expression for the start of a substring }
    i:      integer;        { Loop index }
    originalusize: integer;  { Max size packed before this call }
    checkusubtypes: boolean; { Pass check_size down to subtypes? }

{___________________________________________________________________________

        Generate code to:       (Un)pack String
        -----------------       ---------------

This non-recursive procedure handles the differences between string
representations in all the various target environments. Complications are:

1.  Under TurboPascal & Omegasoft pascal, the length is in the zeroth element
    of the string, and so has type 'char'.

2.  Under VMS/Pascal, if one wants to be VMS FORTRAN compatible, one
    has to pad strings out with blanks to fit into the descriptors when they
    arrive. FORTRAN does not have the concept of a variable length string,
    only of a conformant one.

    However, on a server we coerce the length of the server string to
    the length of the string sent, but this is done by the support routine.

3.  When there is no specific string handling software available, the
    compiler generates references to an extra length parameter.
    Obviously, this cannot be used in a nested subtype.

    Example of the sort of code generated:

                upk/pck_integer(l_xxx);
                FOR rpc_a := 1 TO l_xxx DO
    either      upk_char(rpc_p_buf, a_xxx[rpc_a]);
    or              begin
                        a_xxx[rpc_a]:=rpc_ch[m_index];
                        m_index := m_index+1;
                    end;

4.  C is different in that the index starts at zero, not 1, and the string
    is zero_terminated. There is (can never be) any protection against overflow.
}
 procedure genupackustring(var  where: text;
                expr:   expression;         { string variable name }
                pt:     ptrudefinedutype;   { data type }
                level:  integer;            { indentation level }
                topack: boolean);           { pack or unpack? }

 var
        indexua,
        indexub:        char;           { last character of name of index }
        auexpr,                         { One element of the string }
        iuexpr,                         { The integer value of the length }
        luexpr:         expression;     { Expression for the length variable }

 begin {gen_pack_string}

    indexua := chr( ord('a')+level-1);  { rpc_a etc }
    indexub := chr( ord('a')+level);    { rpc_b etc }

    with pt^ do
    if omode in cmode then begin

        auexpr := expr;
        appenduindex(auexpr, level);    { eg    x[rpc_a]            }

        format(luexpr, 'rpc_      ');   { the length }
        appendu1(luexpr, indexub);          { eg    rpc_b }

{   Find length of string: }

        if topack then begin
                writeln(where, ' ':leftin(level), 'for (rpc_',indexub,
                    ' = 0; rpc_',indexub,' <= ',
                     typuhigh:1, '; rpc_',indexub,'++)');
                write(where, ' ':leftin(level), 'if (');
                writeuexp(where, expr);
                writeln(where, '[rpc_',indexub,'] == ''\0'') break;');
        end;

{   (un)pack the length of the string :
}
        genupackutype(where, luexpr, simpleudescriptor[integertok],
                level, false, { Don't dereference locals }
                topack, checkusubtypes);

{   Generate the for loop:
}
        writeln(where, ' ':leftin(level),           { a := 0 to b }
                'for (rpc_',indexua,' = 0; rpc_', indexua,
                ' < rpc_',indexub,'; rpc_',indexua,'++) {');

{   Pack each character:
}
        genupackutype(where, auexpr, simpleudescriptor[chartok],
                level, false {deref}, topack, checkusubtypes);

{   Terminate the for loop:
}
        writeln(where, ' ':leftin(level), '}');

{   Add the terminator in memory:
}
        if not topack then
        begin
                write(where,    ' ':leftin(level));
                writeuexp(where, expr);
                writeln(where,  '[rpc_',indexub,'] = ''\0'';');
        end;

        sizeusoufar := originalusize  + typumaxusize;

    end else begin {not C -- Pascal: - - - - - - - - - - - - - - - - - - - - -}

{   VAX/VMS descriptors are handled by a subroutine:
}
      if omode = vaxvms then begin

            write(where, ' ':leftin(level), upkpck[topack]);
            if (not client and not topack)
                then write (where, '_Vstring_for(rpc_p_buf, ') {server unpck }
                else write (where, '_string_for(rpc_p_buf, ');
            writeuexp(where, expr);
            write(where, '::rpc_string_descriptor');           { Standard Descriptor }
            if (not client and not topack)
                then write(where, ', ',typuhigh:1);    { Max size }
            writeln(where, ');');

{   Generate expressions for the length of the string:
}
      end else begin
        luexpr := expr;     { The name of the length variable }
        iuexpr := expr;     { The integer value of the length }
        auexpr := expr;     { The array containing the string }
        if omode in [m6809, pcturbo, macturbo] then begin
            format(luexpr, '$[0]      ');
            format(iuexpr, 'ord($[0]) ');
        end else if omode = vaxpas then begin
            format(luexpr, '$.length  ');
            iuexpr := luexpr;           { The expression already is an integer }
        end else begin
            format(luexpr, 'l_$       ');
            format(auexpr, 'a_$       ');
            iuexpr := luexpr;           { The expression already is an integer }
            if level <> 1 then codegenuerror(
                    'String in composite type illegal for this target')
        end {if};
        appenduindex(auexpr, level);    { a_expr now refers to one element }

{   (un)pack the length of the string :

        It's a little unusual for the M6809 etc as the length is of type
        character.
}
        write(where, '    ');
        if omode in [m6809, pcturbo, macturbo] then
                if topack then begin
                    genupackutype(where, iuexpr,
                         simpleudescriptor[integertok], level+2, false,
                         topack, checkusubtypes);
                end else begin
                    write(where, ' ':leftin(level));
                    writeuexp(where, luexpr);
                    writeln(where, ' := rpc_ch[m_index + 1];');
                    write(where, ' ':leftin(level));
                    write(where, 'm_index := (m_index + 2);');
                end

        else begin
            if (omode=vaxpas) then              { string length is short }
                genupackutype(where, luexpr,
                        simpleudescriptor[shortok],
                        level+2, false {deref}, topack, checkusubtypes)
            else
                genupackutype(where, luexpr,
                        simpleudescriptor[integertok],
                        level+2, false {deref}, topack, checkusubtypes);
        end;

{   Generate the FOR loop:
}
        write(where, ' ':leftin(level), 'FOR rpc_',indexua,' := 1 TO ');
        writeuexp(where, iuexpr);
        writeln(where, ' DO BEGIN');

{   Generate the code to (un)pack the character
}
        genupackutype(where, auexpr, simpleudescriptor[chartok], level+2,
            false {deref}, topack, checkusubtypes);

{   Terminate the FOR loop:
}
        writeln(where, ' ':leftin(level), 'END {for ');
        writeuexp(where, iuexpr);
        writeln(where, '};');

      end {if not VAXVMS pascal };
      sizeusoufar := originalusize  + typumaxusize ;
    end {if not C };
 end {gen_pack_string};


{       Check Buffer Overflow if necessary
        ----------------------------------

    An attempt is made to optimise the tradeoff of the execution of checking
    code, against the efficiency of filling the buffers. Hence, small
    composite types are not allowed to cross buffer boundaries.
    There are three conditions for inserting the overflow check:

    1.  The parameter check_size indicates that this type is not a component
        of a larger type which has already been checked;

    2.  It is possible that, after marshalling this type, that the packet
        size (NON_FRAGMENTATION_LIMIT) might be exceeded.

    3.  The maximum size of this type must be less than or equal the
        FRAGMENTATION_THRESHOLD: If it is greater, then we check each of
        the subtypes individually. The threshold must be set so that all
        atomic types are checked.

    If the last test fails, then we have to turn on checking in the subtypes.
}
 procedure checkubufferuoverflow(allowance: integer);
 begin
     if checkusize
     and ((sizeusoufar + allowance) > nonufragmentationulimit)
     then if (allowance <= fragmentationuthreshold)
     then begin
        fragmentationuused := true;
        if omode in cmode then begin
              write(where, ' ':leftin(level)+4,
                'if (rpc_p_buf->m_index > RPC_BUFFER_SIZE - ',allowance:1,
                ') (void)rpc_');
              if topack then writeln(where, 'put(&rpc_p_buf);')
                        else writeln(where, 'get(&rpc_p_buf);');
        end else begin
              write(where, ' ':leftin(level)+4,
                'if (rpc_p_buf^.m_index > RPC_BUFFER_SIZE - ',allowance:1,
                ') then rpc_');
              if topack then writeln(where, 'put(rpc_p_buf);')
                        else writeln(where, 'get(rpc_p_buf);');
        end {pascal};
     end{ if};
 end;


{_____________________________________________________________________________
                    Main block of gen_pack_type
}
begin {gen_pack_type}

   originalusize := sizeusoufar;        { We will update it }

   {    Generate a few useful expressions: }

    ch := chr(ord('a')+level-1); {for rpc_a etc }
    luexpr := expr;             {   x }
    prependu1(luexpr, '_');     {  _x }
    suexpr := luexpr;           {  _x }
    prependu1(luexpr, 'l');     { l_x }
    prependu1(suexpr, 's');     { s_x }


{   If the type is marshalled by an external routine, just generate a call
    to that routine:
}
    with typ^ do
    if typuexternal then begin
        write(where, ' ':leftin(level), upkpck[topack], '_');
        writeuname(opufile, typuname^.ntyuname);
        write(where, '(rpc_p_buf,');
        if (omode in cmode) and not topack then write(where, '&');
        writeuexp(where, expr);
        writeln(where, ');');

    end else begin

     checkusubtypes := checkusize
         and (sizeusoufar + typumaxusize > nonufragmentationulimit)
         and (typumaxusize > fragmentationuthreshold);

     checkubufferuoverflow(typumaxusize);

     case typubasicutype of
{ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                                                        S I M P L E

Here we assume that the names of the pck/upk routines are consistent
with the names rpc_xxx of the basic types.

Note that in Pascal, a character has to be unpacked with an assignment
statement, because the expression can be for a component of a packed array,
which cannot be passed by address. If the expression does not end with ']',
we can go ahead and use upk_char, as it is not an array compenent at all.
In general, we prefer to call upk_char as it may be faster, and is certainly
shorter.

}
        chartok: begin
            if (omode in cmode) or topack or (expr.str[expr.len]<>']')
            then begin
                write(where, ' ':leftin(level), upkpck[topack], '_char(rpc_p_buf, ');
                if (level=1) and deref and (omode in cmode)
                    then write(where, '*');
                writeuexp(where, expr);
                writeln(where, ');');
            end else begin {pascal unpack}
                writeln(where, ' ':leftin(level), 'BEGIN');
                write(where, ' ':leftin(level)+4);
                writeuexp(where, expr);
                writeln(where, ':=rpc_ch[m_index];');
                writeln(where, ' ':leftin(level)+4, 'm_index := m_index+1;');
                writeln(where, ' ':leftin(level), 'END;');
            end;
            sizeusoufar := sizeusoufar+1;
        end;

        bytetok,
        shortok, integertok,
        real32tok, real48tok,
        real64tok, real128tok,
        longtok: begin
            write(where, ' ':leftin(level), upkpck[topack]);
            with typuname^.ntyuname do
                for i := 4 to len do write(where, str[i]);  { eg "_integer" }
            write(where, '(rpc_p_buf, ');
            if (level=1) and deref and (omode in cmode)
                then write(where, '*');
            writeuexp(where, expr);
            writeln(where, ');');
            case typubasicutype of
                bytetok:    sizeusoufar := sizeusoufar + 1;
                shortok,
                integertok: sizeusoufar := sizeusoufar + 2;
                real32tok,
                longtok:    sizeusoufar := sizeusoufar + 4;
                real48tok:  sizeusoufar := sizeusoufar + 6;
                real64tok:  sizeusoufar := sizeusoufar + 8;
                real128tok: sizeusoufar := sizeusoufar + 16;

            end;
        end {simple type };

{ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}
        arraytok: begin                                 { A R R A Y }
            exp2 := expr;
            appenduindex(exp2, level);

            write(where, ' ':leftin(level));
            if not (omode in cmode)
            then begin
                writeln(where, 'FOR rpc_', ch, ' := ',
                        typulow:1, ' TO ', typuhigh:1, ' DO BEGIN')
            end else begin
                writeln(where, 'for (rpc_', ch, ' = 0; rpc_', ch, '<',
                         typuhigh - typulow + 1:1, '; rpc_', ch, '++) {');
            end {if};

{   When making the recursive call to the subtype packing, size_so_far must
    already be increased to the maximum size, so that the checking code will
    be put in if necessary. As size_so_far is corrupted by that call, it must
    be reset again afterward.
}
            sizeusoufar := originalusize+typumaxusize;
            genupackutype(where, exp2, typusubtype, level+1,
                                    false {deref}, topack, checkusubtypes);
            sizeusoufar := originalusize+typumaxusize;

            if omode in cmode
            then writeln(where, ' ':leftin(level), '}')
            else writeln(where, ' ':leftin(level), 'END {FOR};');

        end {arraytok};
{ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

                                                        R E C O R D

    In C, records must be passed by pointers, just like scalar variables, if
    they are OUT or INOUT, or the option /BYVALUE is not selected.
}
        recordtok: begin
            scan := typufields;
            while scan <> nil do with scan^ do begin
                exp2 := expr;
                if (level=1) and deref and (omode in cmode)
                    then format(exp2, '$->       ')
                    else format(exp2, '$.        ');
                appenduname(exp2, ntyuname);
                genupackutype(where, exp2, ntyutype, level, false,topack,
                                checkusubtypes);
                scan := ntyunext;
            end {while};
        end {recordtok};
{ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                                                        A C C E S S

    A pointer is marshalled as a single byte (0 or 1) to say whether the
    original pointer was NIL (0) or valid (1). If and only if it was valid,
    the byte is followed by the marshalled data which the pointer pointed to.

    Allocation is always done on unmarshalling (if the pointer is not NIL).
    Deallocation is not done here, but by the gen_allocate routine.

    Examples:   PASCAL                          C           ([]should be curly)
|
|               if x = NIL then pck_byte(0)     if (x==NULL) pck_byte(0)
|               else begin                      else [
|                   pck_byte(1);                    pck_byte(1);
|                   ...                             ...
|               end;                            ] /* end if */
}
        accesstok: begin

            if checkusubtypes then
                checkubufferuoverflow(1);       { Check we aren't on the end }
            sizeusoufar := sizeusoufar + 1;     { For flag }
            if omode in cmode then begin
                exp2 := expr;
                format(exp2, '*($)      ');
                if not topack then begin

{                   If the pointer was NULL, skip the lot:
}
                    write(where,        ' ':leftin(level));
                    writeln(where,    'if (next_byte(rpc_p_buf)) {' );

{                   Allocate space:
}
                    write(where,        ' ':leftin(level+1));
                    writeuexp(where, expr);
                    if typusubtype^.typuname=nil then codegenuerror(
                      'Pointer to unnamed type - cannot allocate space!');
                    write(where, ' = (');
                    writeuname(where, typusubtype^.typuname^.ntyuname);
                    write(where, '*) malloc(sizeof(' );
                    writeuname(where, typusubtype^.typuname^.ntyuname);
                    writeln(where, '));');

                end {if unpack} else begin {pack}

                    write  (where, ' ':leftin(level),  'if (');
                    writeuexp(where,                   expr);
                    writeln(where,                     '==NULL) pck_byte(0);');
                    writeln(where, ' ':leftin(level),  'else {');
                    writeln(where, ' ':leftin(level+1),'pck_byte(1);');

                end {packing};
                genupackutype(where, exp2, typusubtype,
                                        level, false, topack, checkusubtypes);

                writeln(where, ' ':leftin(level),   '} /* end if */');

            end else begin {Pascal}

                exp2 := expr;
                appendu1(exp2, '^');            { x^ }

                if not topack then begin

{                   If the pointer was NIL, skip the lot:
}
                    writeln(where,      'm_index:=m_index+1;');
                    writeln(where, ' ':leftin(level),
                                        'if b[m_index-1]<>0 then begin');
{                    Allocate space:
}
                    write(where,' ':leftin(level+1),    'new('      );
                    writeuexp(where,                    expr        );
                    writeln(where,                      ');'        );

                end else begin {if topack}
                    write  (where, ' ':leftin(level),   'if ');
                    writeuexp(where,                    expr);
                    writeln(where, ' = NIL then pck_byte(0)');
                    writeln(where, ' ':leftin(level),   'else begin');
                    writeln(where, ' ':leftin(level+1), 'pck_byte(1);');

                end {if topack};
                genupackutype(where, exp2, typusubtype, level, false, topack,
                    checkusubtypes);
                writeln(where, ' ':leftin(level),   'end {if};');

            end {if Pascal}
        end {accesstok};

{ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}
        sequence: begin                                 { S E Q U E N C E }
            if level>1 then codegenuerror(
                'SEQUENCE type not allowed within composite type.');

            genupackutype(where, luexpr,
                    simpleudescriptor[integertok],level, deref, topack,
                        checkusubtypes);

            exp2 := expr;
            format(exp2, 'a_$       ');     { Make an expression for the array}
            appenduindex(exp2, level);      { Make expression for an element }

            write(where, ' ':leftin(level));
            if not (omode in cmode)
            then begin
                write(where,    'FOR rpc_', ch, ' := 1 TO l_');
                writeuexp(where, expr);
                writeln(where,  ' DO BEGIN')
            end else begin
                write(where, 'for (rpc_', ch, ' = 0; rpc_', ch, '< ');
                if deref and (omode in cmode) then write(where, '*');
                writeuexp(where, luexpr);
                writeln(where,'; rpc_', ch, '++) {');
            end {if};

            sizeusoufar := originalusize + typumaxusize;{ See comment for array }
            genupackutype(where, exp2, typusubtype, level+1, false {deref},
                                                topack, checkusubtypes);
            sizeusoufar := originalusize + typumaxusize;{ See comment for array }

            if omode in cmode
            then writeln(where, ' ':leftin(level), '      }')
            else writeln(where, ' ':leftin(level), 'END {FOR};');
        end {sequence};

{ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}
        stringtok: begin                                { S T R I N G }

            genupackustring(where, expr, typ, level, topack);
            genualign(where, 1);    { Align on word boundary }

        end {stringtok};

{ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
|                                                         S U B S T R I N G
| Example:      upk/pck_integer(s_xxx);
|               upk/pck_integer(l_xxx);
|               FOR rpc_a := s_xxx TO s_xxx + l_xxx - 1 DO
|                   upk/pck_integer(rpc_p_buf, a_xxx[rpc_a]);
|
| Note: in C, the start address in the string used to run in the range 0..n-1,
| while is Pascal it runs in the range 1..n.  The cannonical form for
| transmission involves it running from 1..n, so it was adjusted in the
| C code.  This was changed (7-Jun-89) so that it is always passed 1..n.
}
        substring: begin

            exp2 := suexpr;                     { Start Position In String }
            genupackutype(where, exp2,
                        simpleudescriptor[integertok],level, deref, topack,
                        checkusubtypes);

            genupackutype(where, luexpr, simpleudescriptor[integertok],
                        level, deref, topack, checkusubtypes);

{ in C:      for (rpc_a = *s_x-1; rpc_x < *l_x + *s_x-1; rpc_a++) {         }
{ in Pascal: FOR rpc_a := s_x to s_x + l_x - 1 DO BEGIN             }

            if omode in cmode then begin
                write(where, '    for (rpc_', ch, ' = ');
                if deref then write(where, '*');
                writeuexp(where, suexpr);
                write(where,    '-1; rpc_', ch, ' < ');
                if deref then write(where, '*');
                writeuexp(where, luexpr);
                write(where, ' + ');
                if deref then write(where, '*');
                writeuexp(where, suexpr);
                writeln(where, '-1; rpc_', ch, '++) {')

            end else begin

                write(where,    '    FOR rpc_', ch, ' := ');
                writeuexp(where, suexpr);
                write(where,    ' TO ');
                writeuexp(where, luexpr);
                write(where, '+');
                writeuexp(where, suexpr);
                writeln(where, ' - 1 DO BEGIN')
            end;

            exp2:= expr;                {   x }
            prependu1(exp2, '_');       {  _x }
            prependu1(exp2, 'a');       { a_x }
            appenduindex(exp2, level);  { a_x[rpc_a] }

            sizeusoufar := originalusize + typumaxusize;
            genupackutype(where, exp2, simpleudescriptor[chartok],
                         level+1, false {deref}, topack, checkusubtypes);
            sizeusoufar := originalusize + typumaxusize;

            if omode in cmode then begin        { Terminate the loop }
                writeln(where, '    }')
            end else begin
                writeln(where,  '    END;');
            end;

            genualign(where,1 );                { Align on word boundary }

        end {substring};

     end {case};
    end {if not external type }

end {gen_pack_type};

{___________________________________________________________________________

        Generate code to:       (un)Pack all parameters necessary
        -----------------       ---------------------------------

The dirn parameter is INtok or OUTok, to specify which parameters to unpack.

A string descriptor needs to be initialised, including the creation of the
string, at the server side, and the string has to be disposed of afterwards.

}
procedure genupack( var where:  text;           { Output file }
                        head:   ptruidlist;     { Parameter list }
                        dirn:   attrutype;      { Parameter filter }
                        topack: boolean);       { Pack, rather than unpack? }

var expr:    expression;
begin {GEN_PACK}
  repeat
    with head^, idutype^ do begin


        { generate an expression for the variable: }

        expr.len := 0;      { Expression for the variable }
        appenduname(expr, head^.name);    { expr is the param. name }


{       Initialise any VMS descriptors on server side
        ------------------------------------------
}
        if (not client) and (not topack)
            and (omode=vaxvms)
            then if contains(idutype, [stringtok])
            then genuallocate(where, expr, idutype, 1, true {allocate},
                            [stringtok]);


{       Pack or unpack
        --------------
}
        if (attr = inoutok)
            or (attr = dirn)
        then begin
            deref := client
                    and not (runoptions[byvalue].value   {* not value param *}
                             and (attr = intok));    { and in-only param }

            genupackutype(where, expr, idutype, {level} 1, deref,
                    topack, true {check for overflow});

        end {if};

{       Deallocate Memory
        -----------------
}
        if (not client) and topack              { ie end of server }

            then if omode = vaxvms then begin
                if contains(idutype, [stringtok, accesstok])
                then genuallocate(where, expr, idutype, 1,
                                false {deallocate}, [stringtok, accesstok]);

            end else begin {not vms}
                if contains(idutype, [accesstok])
                then genuallocate(where, expr, idutype, 1,
                                false {deallocate}, [accesstok]);
            end;


        { Remove client's tree for an IN OUT parameter }

        if client and (attr=inoutok) and topack
        then if contains(idutype, [accesstok])
        then genuallocate(where, expr, idutype, 1, false {deallocate},
                                    [accesstok]);

    end {with};
    head := head^.next;
  until head = nil;
end; {GEN_PACK}


{               (end of packing/unpacking)
*****************************************************************************
}
{       Generate local variables (server) / parameter (client)
        ------------------------------------------------------

This subroutine will generate

    -  The parameters to the client subroutine,
    -  or a procedure definition for a turbopascal INTERFACE
    -  or the local variables in the server.

It is a bit of a mess at present, with the same routine trying to do
many different things. It is currently called in eight different circumstances:

                                                            Params  C D P

from Server                             Pascal              y       F F T

                                        C                   y       F F T

from gen_external               )
     from gen_module            )       Pascal              y       F T T
      from gen_server_block     )
 or from gen_external           )       C                   y       F T T
     from extern_gen            )

Client INTERFACE section                Turbopascal only    y       T F T

Client routines definition              Pascal, except      y       T F T
                                        Turbopascal         n       T F F

                                        C                   y       T F T

where C=client(global flag), D=doing_externals, P=Parameters_wanted.

Unfortunately, any rearrangement of the code is likely to be just as messy.
______________________________________________________________________________
}
procedure genuheader(var where: text;
                        ptr: ptrublockutable;
                        doinguexternals,        { Making ext. declarations? }
                        parametersuwanted: boolean);

{____________________________________
}
  procedure genuformalsuc(head: ptruidlist);

  { Generates the formal parameter list enclosed in brackets }
  var
        a:          integer;
        scan:       ptruidlist;

  begin {gen_formals_c}

        {* We are sure that head is NOT nil *}

        a := 0;
        scan := head;
        write(where, '(');
        while scan <> nil do begin
            if scan^.idutype^.typubasicutype in [sequence, substring]
                    then write(where, 'a_');
            writeuname(where, scan^.name);
            if scan^.idutype^.typubasicutype = substring then
            begin
                write(where, ', s_');
                writeuname(where, scan^.name);
                a := a + 1;
            end;
            if scan^.idutype^.typubasicutype in [sequence, substring] then
            begin
                write(where, ', l_');
                writeuname(where, scan^.name);
                a := a + 1;
            end;

            scan := scan^.next;
            if scan <> nil then
            begin
                write(where, ', ');
                a := a + 1;
                if a > 3 then
                begin
                    a := 0;
                    writeln(where);
                    write(where, '        ');
                end;
            end;
        end {while};
        writeln(where, ')');
  end; {gen_formals_c}
{____________________________________
}
  procedure genuparams(head: ptruidlist);

{ Generates the parameter list enclosed in brackets,
  or VAR followed by list

On entry,
    head    must not be NIL, must point to list of ids.
}

  var
        a:          integer;
        scan:       ptruidlist;
        ch:         char;
        simpleuref:  boolean;   { Should simple variables be dereferenced? }
        arrayuderef: boolean;   { what about composite variables? }
  begin {gen_params}

        {* We are sure that head is NOT nil *}


    scan := head;
    repeat
        write(where, '    ');

        {* Write the type's name if any otherwise the full declaration *}
        {* Sequences and substrings MUST be expanded a little *}
        {* The var's name depends on its type, so be careful! *}


{   Array variables are not dereferenced in C because they are always
    passed by address anyway. In Pascal, the VAR is not needed in the
    server list, but needed everywhere else.
}
        arrayuderef := (client or doinguexternals)
                        and not (omode in cmode);

{   Simple types and structures  must be dereferenced unless /BYVALUE was
    specified and they are IN.
    The VAR is never needed for the list of variables in the server.
}
        simpleuref :=  (client or doinguexternals)      { ie not server list }
                        and not ((runoptions[byvalue].value)
                                    and (scan^.attr = intok));

        with scan^.idutype^ do
        case typubasicutype of

            chartok,
            bytetok,
            shortok, integertok,
            real32tok, real48tok,
            real64tok, real128tok,
            longtok,
            recordtok,
            accesstok:
                writeudeclaration(where, ' ', scan, simpleuref);{ xxxx : tttt }

            arraytok:                                       { xxxx : tttt }
                writeudeclaration(where, ' ', scan, arrayuderef);

            stringtok:
                begin                                  { (a_)xxx: tttt(;) }
                    if not(omode in(cmode+
                        [vaxvms, vaxpas, m6809, pcturbo, macturbo]))
                    then begin
                        writeudeclaration(where, 'a', scan, arrayuderef);
                        writeln(where, ';');       { (l_xxx: rpc_integer) }
                        writeuinteger(where, 'l', scan^.name, simpleuref);
                    end else begin
                        writeudeclaration(where, ' ', scan, arrayuderef);
                    end;
                end;

            sequence:
                begin                                      { a_xxx: tttt; }
                    writeudeclaration(where, 'a', scan, arrayuderef);
                    writeln(where, ';');
                                                     { l_xxx: rpc_integer }
                    writeuinteger(where, 'l', scan^.name, simpleuref);
                end;

            substring:
                begin                                      { a_xxx: tttt; }
                    writeudeclaration(where, 'a', scan, arrayuderef);
                    writeln(where, ';');
                                                     { s_xxx: rpc_integer }
                    writeuinteger(where, 's', scan^.name, simpleuref);
                    writeln(where, ';');
                                                     { l_xxx: rpc_integer }
                    writeuinteger(where, 'l', scan^.name, simpleuref);
                end;

        end; {with, case}

        scan := scan^.next;
        if scan <> nil then
                writeln(where, ';')
   until scan = nil;

  end; {GEN_PARAMS}

{- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
}
begin {GEN_HEADER}

 with ptr^ do begin

        if omode in cmode then begin            { C language }
            if client then begin
                if butype = functok then
                begin
                    writok(where, return);      { eg:    int xxx(a,b,c)  }
                    write(where, ' ')
                end {function};
                writeuname(where, name);
                genuformalsuc(list);    { Generate formal parameter list }

            end else begin                      { C server or externals }
                if not doinguexternals  then write(where, 'r_');
                writeuname(where, name);
                write(where, '()');
                if doinguexternals then
                begin
                    writeln(where, ';');
                    write(where, '/*  Parameters: ');
                    if list = nil then write(where, 'none');
                end;
                writeln(where);
                if not doinguexternals then writeln(where, '{');
            end {if not client}

        end else begin                          { Pascal }
             if (client or doinguexternals) then begin
                writok(where, butype);
                writeuname(where, name);

             end else begin                             { Pascal server }
                writok(where, proctok);
                if not doinguexternals  then write(where, 'r_');
                writeuname(where, name);
                if not doinguexternals then writeln(where, ';');
            end {server};
        end {Pascal};

{* PRINT PARAMETER LIST if necessary*}


        if parametersuwanted then
            if (list <> nil) then begin
                if not (omode in cmode) then
                    if client or doinguexternals then writeln(where, '(')
                else writeln(where, 'VAR');
                genuparams(list);
                if (client or doinguexternals) and not (omode in cmode)
                    then write(where, ')');
            end;
{* else
                if client and (omode in Cmode) then write(where,'()'); *}

{* PRINT RESULT only for client PASCAL functions *}

        if  not (omode in cmode) then begin
            if (client or doinguexternals) then begin
                if parametersuwanted
                    and (butype = functok)
                then begin
                    write(where, ': ');
                    writok(where, return);
                end;
            end {if client }
        end {if Pascal};

        if      ((client or doinguexternals) and not (omode in cmode))
                or (list <> nil)
         then writeln(where, ';');

 end; {end_with}
end; {GEN_HEADER}


{___________________________________________________________________________

        Generate externals
        ------------------

Generates the list of external declarations of each procedure
or function.
}
procedure generateuexternals;
var
        scan : ptrublockutable;
begin
        scan := blockptr;
        while scan <> nil do begin
            if omode in cmode then
            begin
                write(opufile, 'extern ');
                if scan^.butype = functok then
                begin
                    writok(opufile, scan^.return);
                    write(opufile, ' ');
                end;
                genuheader(opufile, scan, true {externals}, true);
                writeln(opufile, '    */');
            end
            else begin
                genuheader(opufile, scan, true {externals}, true);
                if omode = m6809 then writeln(opufile, 'EXTERNAL;')
                    else writeln(opufile, 'EXTERN;');
            end;
            writeln(opufile);
            scan := scan^.next;
        end; {while}
end;

{___________________________________________________________________________

            Generate:       A type declaration in C

    This recursive procedure generates a definition of a type, given
    a name and a type descriptor. It is impossible to generate a type
    itself, as the name is buried within it in C. This is one of the
    more trying aspects of C. For example

    One must be careful of the precedence of operators such as [] and *.

            char * mytype[80]
    for     mytype      is          ARRAY[0..79] OF ACCESS RPC_CHAR

On entry,
    Cursor is assumed to be at left margin.
On exit,
    Trailing semicolon and newline NOT done.
}

procedure genutypeudecluc(var where:  text;             { output file }
                            expr:   expression;         { for the varaiable }
                            pt:     ptrudefinedutype;   { type structure }
                            level:  integer);           { indentation }
var     scan:   ptrunamedutype;
        exp2:   expression;
begin
    with pt^ do case typubasicutype of

        chartok,                    { First token, simple type, courier type }
        bytetok,
        shortok, integertok,
        real32tok, real48tok,
        real64tok, real128tok,
        longtok: begin
            write(where, ' ':leftin(level));
            writeuname(where,
                simpleudescriptor[typubasicutype]^.typuname^.ntyuname);
            write(where, ' ');
            writeuexp(where, expr);     { Example: rpc_char x }
        end {simple type};

        substring,
        stringtok:  begin               { Example: rpc_char x[81] }
                write(where, ' ':leftin(level));
                write(where, 'rpc_char ');
                writeuexp(where, expr);
                write(where, '[', (typuhigh+1):1, ']');
        end {stringtok};

        arraytok:   begin               { example  int x[100] }
                exp2 := expr;
                appendu1(exp2,'[');
                appendudecimal(exp2,typuhigh-typulow+1);
                appendu1(exp2,']');
                genutypeudecluc(where, exp2, typusubtype, level);
        end {arraytok};

        sequence:   begin
                exp2 := expr;
                appendu1(exp2,'[');
                appendudecimal(exp2,typuhigh-typulow+1);    { 90613 }
                appendu1(exp2,']');
                genutypeudecluc(where, exp2, typusubtype, level);
        end; {sequence}

        accesstok: begin
                exp2 := expr;
                prependu1(exp2, '(');
                prependu1(exp2, '*');
                appendu1 (exp2, ')');   { Example:  int  (*x)   }
                genutypeudecluc(where, exp2, typusubtype, level);
        end;

        recordtok: begin
            writeln(where, ' ':leftin(level), 'struct {');
            scan := typufields;
            while scan <> nil do begin
                        exp2.len := 0;
                        appenduname(exp2, scan^.ntyuname);
                        genutypeudecluc(where, exp2, scan^.ntyutype, level+1);
                        writeln(where, ';');
                        scan := scan^.ntyunext;
            end {while};

            write(where, ' ':leftin(level), '} ');
            writeuexp(where, expr);
        end;

    end {with, case};
end {gen_type_decl_c};
{___________________________________________________________________________

            Generate:       A type declaration in Pascal

    This recursive procedure generates an expression for a type, given
    a type descriptor.

    Example:
            ARRAY OF RECORD H: INTEGER; C: CHAR END

    If a subtype has a name, that name will be used instead of elaborating
    the subtype itself - For example, in Pascal pointers must be declared in
    terms of the base type name, not a type expression.

On entry,
    We assume any indentation required has been done for the 1st line.
On exit,
    The semicolon and Newline on the end are not done.
}

procedure genutypeupas( var where:  text;               { output file }
                            pt:     ptrudefinedutype;   { type structure }
                            level:  integer);           { indentation }
var     scan:   ptrunamedutype;
        exp2:   expression;
begin
    with pt^ do
    if (level>1) and (typuname <> nil) then begin
        writeuname(where, typuname^.ntyuname);
    end else case typubasicutype of
        chartok,                    { First token, simple type, courier type }
        bytetok,
        shortok, integertok,
        real32tok, real48tok,
        real64tok, real128tok,
        longtok: begin          { Example:  rpc_char }
            writeuname(where,
                simpleudescriptor[typubasicutype]^.typuname^.ntyuname);
        end {simple type};

        substring,
        stringtok:  begin               { Example: rpc_char x[81] }
                if (omode=vaxvms) and (typubasicutype=stringtok) then begin
                    writeln(where, 'RECORD strlen: rpc_short;');
                    writeln(where,  ' ':leftin(level)+8,
                                    'DType, Cont: rpc_byte;');
                    write  (where, ' ':leftin(level)+8, 'StrAdr: ^t_');
                    writeuname(where, typuname^.ntyuname);
                    writeln(where);
                    write  (where, ' ':leftin(level)+4, 'END');
                end else if (omode=vaxpas)
                                and (typubasicutype=stringtok) then begin
                    write(where, 'VARYING[',typuhigh:1, '] OF CHAR');
                end else if (omode in [m6809, pcturbo, macturbo])
                                and (typubasicutype=stringtok) then begin
                    write(where, 'STRING[',typuhigh:1, ']');
                end else begin
                    write(where, 'PACKED ARRAY [1..',typuhigh:1, '] OF CHAR');
                end {if};
        end {stringtok};

        arraytok:   begin               { example  int x[100] }
                write(where, 'ARRAY [', typulow:1, '..', typuhigh:1, '] OF ');
                genutypeupas(where, typusubtype, level+1);
        end {arraytok};

        sequence:   begin
                write(where, 'ARRAY [1..', typuhigh:1, '] OF ');
                genutypeupas(where, typusubtype, level+1);
        end; {sequence}

        accesstok: begin
                write(where, '^');
                genutypeupas(where, typusubtype, level+1);
        end;

        recordtok: begin
                writeln(where, 'RECORD');
                    scan := typufields;
                    while scan <> nil do begin
                        exp2.len := 0;
                        write(where, ' ':(leftin(level)+4));
                        writeuname(where, scan^.ntyuname);
                        write(where, ': ');
                        genutypeupas(where, scan^.ntyutype, level+1);
                        writeln(where, ';');
                        scan := scan^.ntyunext;
                    end {while};
                write(where, ' ':leftin(level), 'END {RECORD}');
        end;

    end {with, case};
end {gen_type_pas};

{___________________________________________________________________________

        Generate type definitions for whole package

Example:
                typedef rpc_char xxxxx;
                typedef rpc_integer my_array[80];
or:
                xxxxx = rpc_char;
                my_array = ARRAY[1..80] OF rpc_integer;
}

procedure generateutypes(var where : text);
var
        scan:   ptrunamedutype;
        expr:   expression;

begin {generate types}
    scan := typeptr;
    while scan <> nil do with scan^ do
    begin

{   Just specially for the VAX/VMS, define a basic array type for each
    length of string by descriptor:
}
        if omode = vaxvms then
            if (ntyutype^.typubasicutype = stringtok)     { Define char array: }
            then begin
                write(where, 't_');             { jammy! }
                writeuname(where, ntyuname);
                write(where, ' = ');
                writeln(where,
                    'PACKED ARRAY [1..',ntyutype^.typuhigh:1, '] OF CHAR;');
            end; {if}

{   Now make the main type definition:
}
        expr.len := 0;
        appenduname(expr, ntyuname);        { expr is name of variable }
        if omode in cmode then begin
            write(where, 'typedef ');
            genutypeudecluc(where, expr, ntyutype, 1);
        end else begin
            writeuname(where, ntyuname);
            write(where, ' = ');
            genutypeupas(where, ntyutype,1);
        end {if};
        writeln(where, ';');
        scan := ntyunext;
    end; {end_while}
    writeln(where);
end;


{_____________________________________________________________________________

        Generate External Definition File
        =================================

    PILS and FORTRAN do not get declaraction files generated, as they
    don't do enough type checking to merit it.
}
procedure extugenerator( extumode: outputumode; extuname: astring);

begin
if not(omode in [pils, vaxfor])
 then if fileuopen(opufile, extuname, rewriting) then begin
        client := false;
        omode := extumode;
        writeln(opufile);
        if omode in cmode
            then write(opufile, '/*  ')
            else write(opufile, '{*  ');
        write(opufile, 'Stub Version Number: ');
        if versionunum = 0
            then write(opufile, 'zero (no check)')
            else write(opufile, versionunum:1);
        if omode in cmode
            then writeln(opufile, '  */')
            else writeln(opufile, '  *}');
        writeln(opufile);
        if (runoptions[types].value) and (typeptr <> nil) then
        begin
            if not(omode in cmode) then writeln(opufile, 'TYPE');
            generateutypes(opufile);
        end;
        generateuexternals;
        if not(fileuclose(opufile)) then
                error(cantuclsuext);
end else
        error(cantuopnuext);
end; {EXT_GENERATOR}

{_____________________________________________________________________________


                Generate Module Declaration Code
                ================================

Generates the heading, CONST, TYPE and VAR sections of a stub module,
and the standard include files.
}
procedure genumodule(var where: text);

{       Invoke Include file
|       -------------------
|
| On VMS, the file is referred to by a logical name (no . in it, and a $
| to avoid conflict with real files).
}
  procedure douinclude(which: optuname);
  var i: integer;
  begin
        write(where, '*INCLUDE rpc');
        for i:=1 to 10 do if which[i]<>' ' then write(where,which[i]);
        writeln(where);
  end;

{               Main block for Gen_module
                -------------------------

Generates:      The module header
                The constant and type definitions
}
begin {GEN_MODULE}
        {* Generate header according to xxx_mode *}

    writeln(where);
    if omode in cmode then
    begin
        writeln(where, '#include <rpcrts.h>');
        writeln(where);
    end
    else begin

        if not (omode in [monolith, pcturbo, macturbo]) then
        begin
            write(where, 'MODULE ');
            if client then write(where, 'CLI') else write(where, 'SER');
            writeuname(where, unitname);
            writeln(where, ';');
        end;

        if (omode in [pcturbo, macturbo]) then begin
            write(where, 'UNIT ');
            if client then write(where, 'CLI') else write(where, 'SER');
            writeuname(where, unitname);

            { For MacTurbo, a unit number, in parenthesis, is required between
              the unit name and the semicolon terminating the unit header.
              This number (a positive 16-bit integer constant) should be
              different from any other unit number that the user program might
              use. The current allocation of unit numbers (MacTurbo version)
              is:

                        Module          Unit Number

                        RPCMacPas       1
                        RPCStub         2
                        TSTurbo         3
                        RPCRTS          4

              That's why we use the unit number 5 for stub modules. If more
              stubs have to be combined in a single program, the user will
              need to modify stub unit numbers in order to ensure their
              uniqueness.
            }

            if (omode = macturbo) then
                write(where, '(5)');
            writeln(where, ';');
            writeln(where, 'INTERFACE');
            if (omode = pcturbo) then begin
                writeln(where, 'USES rpcpcpas,');
                writeln(where, '     rpcstub,');
                writeln(where, '     tsturbo,');
                if externalumarshalling then writeln(where, '     extmarsh,');
                write(  where, '     rpcrts');
            end
            else begin

                { For MacTurbo things are more complicated. }

                writeln(where, '{$U RPCMacPas}');
                writeln(where, '{$U RPCStub}');
                writeln(where, '{$U TSTurbo}');
                if externalumarshalling then writeln(where, '{$U extmarsh}');
                writeln(where, '{$U RPCRTS}');
                if not client then begin
                    writeln(where, '{--> It is assumed, by default, that the name of the unit library file <--}');
                    writeln(where, '{--> containing the server routines is the same as the package name.   <--}');
                    writeln(where, '{--> If the assumption is wrong, just replace the following line.      <--}');
                    write(where, '{$U '); writeuname(where, unitname); writeln(where, '}');
                end;
                writeln(where);
                writeln(where, 'USES MemTypes,');
                writeln(where, '     QuickDraw,');
                writeln(where, '     OSIntf,');
                writeln(where, '     ToolIntf,');
                writeln(where, '     RPCMacPas,');
                writeln(where, '     RPCStub,');
                writeln(where, '     TSTurbo,');
                if externalumarshalling then writeln(where, '     extmarsh,');
                write(  where, '     RPCRTS');
            end;

            if not client then begin
                writeln(where, ',');
                writeln(where, '{--> It is assumed, by default, that the name of the unit containing   <--}');
                writeln(where, '{--> the server routines is the same as the package name. If the       <--}');
                writeln(where, '{--> assumption is wrong, just replace the following line.             <--}');
                write  (where, '     '); writeuname(  where, unitname);
            end;
            writeln(where, ';');
            writeln(where);

            if (not client) and (typeptr <> nil) then begin
                writeln(where, '{--> Being this a server stub, the TYPE declarations should not be     <--}');
                writeln(where, '{--> included here (they belong to the unit containing the server      <--}');
                writeln(where, '{--> routines). However you will find them below (commented out), in   <--}');
                writeln(where, '{--> case you need them for reference.                                 <--}');
                writeln(where, '{');
            end;

            if (typeptr <> nil) then
                writeln(where, 'TYPE');

        end else begin

        {* Generate constants *}
            writeln(where);
            writeln(where, 'CONST');
            douinclude('const.h   ');   { Include rpc constants file }

        {* Generate types *}
            writeln(where);
            writeln(where, 'TYPE');
            douinclude('types.h   ');   { Include rpc types file }

        end{if};
    end;

    generateutypes(where);              { generate local types   }

    { For MacTurbo and PCTurbo server stubs stop commenting. }

    if (not client) and (omode in [pcturbo, macturbo])
        and (typeptr <> nil) then
        writeln(where, '}');

{   Generate global variables  (handle for client, program index for server)
}
    if client then begin
        if omode in cmode then
        begin
            write(where, 'rpc_handle h_');              { 891206 }
            writeuname(where, unitname);
            writeln(where, ';')
        end
        else begin
            if (omode = cerncross)
                then writeln(where, 'EXPORT')
                else writeln(where, 'VAR');
            write(where, '    h_');
            writeuname(where, unitname);
            if (omode in [vaxvms, vaxpas])
                then writeln(where, ': [global] integer;')
            else if (omode = m6809)
                then writeln(where, ': integer external;') {70604}
            else if (omode in [pcturbo, macturbo])
                then writeln(where, ': client_pointer;')   {80429}
            else
                writeln(where, ': integer;');
        end;

    end else begin {server}
        if omode in cmode then begin
            write(where, 'program_index p_');
            writeuname(where, unitname);
            writeln(where, ';')
        end else begin {pascal}
            if (omode = cerncross)
                then writeln(where, 'EXPORT')
                else writeln(where, 'VAR');
            write(where, '    p_');
            writeuname(where, unitname);
            if (omode in [vaxvms, vaxpas])
                then writeln(where, ': [global] integer;')
{***        else if (omode = m6809)
                then writeln(where, ': program_index external ;')  ***}
            else if (omode in [pcturbo, macturbo])
                then writeln(where, ': program_pointer;')
            else
                writeln(where, ': program_index;');
        end {if pascal};
    end;
    writeln(where);

    if not (omode in cmode+[pcturbo, macturbo]) then
    begin
        douinclude('proc.h    ');       { External definitions of RTS }
        douinclude('stub.h    ');       { Local stub procedures }
        writeln(where);
    end;

{   Declare external marshalling routines if needed:
}
    if externalumarshalling
     then if (omode in cmode+[cerncross, vaxvms, vaxpas, unixbsd, m6809])
      then decluextumarshal;

{*    if omode in [pcturbo, macturbo] then writeln(where, 'IMPLEMENTATION'); *}

    if (not client) and (not (omode in [pcturbo, macturbo])) then
        generateuexternals;

end; {GEN_MODULE}



{******************************************************************************

                Client code generator
                =====================

Generates one procedure or function for the client.
}

procedure clientugenerator(clientumode: outputumode; cliuname: astring);
var
        scan :          ptrublockutable;
        procunumber :   integer;
{___________________________________________________
}
  procedure genuclientublockupas(ptr: ptrublockutable);

{***
* On each block, the client allocates 'RPC_integer' variables to pack or
* unpack parameters.
****}
  var
        a:              integer;
        returnuvalue:   expression;     { Expression for return value }

 begin {gen_client_block_pas}

  {* generate and print procedure header *}
    if omode in [vaxvms, vaxpas] then writeln(opufile, '[GLOBAL]');

  {  For MacTurbo we must generate the procedure heading
     formal parameter part ONLY in the INTERFACE section.
  }
    sizeusoufar := 0;       { No marshalling code generated yet }
    genuheader(opufile, ptr, false, not(omode =macturbo));

    if omode = m6809 then writeln(opufile, '    ENTRY;');

  {* print local vars and buffer allocation *}
    writeln(opufile, 'VAR rpc_p_buf: rpc_message_pointer;');
    for a := 1 to ptr^.blkunesting do
          writeln(opufile, '    rpc_',
                                     chr(ord('a') - 1 + a), ' : rpc_integer;');
    if ptr^.butype = functok then begin
          write(opufile, '    rpc_ret : ');
          writok(opufile, ptr^.return);
          writeln(opufile, ';');
    end;
    writeln(opufile, 'BEGIN');
    write(opufile, '  rpc_begin(rpc_p_buf, h_');
    writeuname(opufile, unitname);
    writeln(opufile, ', {vers=}', versionunum:1,
                     ', {proc=}', procunumber:1, ');' );
    writeln(opufile, '  WITH rpc_p_buf^ DO BEGIN');

  {* generate packing statements for IN & INOUT params *}
    if ptr^.list <> nil then genupack(opufile, ptr^.list, intok, true);

    writeln(opufile, '  END  {WITH};'); { 71006 }

  {* generate rpc_call and bookeeping *}

    write(opufile, '    ');
    if ptr^.blkustatusuparam <> nil then begin
        writeuname(opufile, ptr^.blkustatusuparam^.name);
        write(opufile, ' := ');
    end {if};

    if ptr^.blkucast then write(opufile, 'rpc_cast')
                     else write(opufile, 'rpc_call');
    if ptr^.blkustatusuparam <> nil then write(opufile, '_status');
    write(opufile, '(h_');
    writeuname(opufile, unitname);
    write(opufile, ', rpc_p_buf');
    if not ptr^.blkucast then write(opufile, ',', ptr^.blkutimeout:1);
    writeln(opufile, ');');

    if ptr^.blkustatusuparam <> nil then begin
        write(opufile, '  IF odd(');
        writeuname(opufile, ptr^.blkustatusuparam^.name);
        write(opufile, ') THEN ');
    end {if};
    writeln(opufile, '  WITH rpc_p_buf^ DO BEGIN');     { 71006 }

    sizeusoufar := 0;       { No unmarshalling code generated yet }
    if ptr^.blkumaxuout>0 then begin
      if omode in cmode
        then write(opufile, '    rpc_p_buf->m_index =')
        else write(opufile, '    m_index :=');
      writeln(opufile, ' RETURN_HEADER_LENGTH;');
    end {if};

  {* if function: unpack return value *}
    if ptr^.butype = functok then
    begin
      write(opufile, '    ');
      dopackusimple(opufile, ptr^.return, false);
      writeln(opufile, 'rpc_ret);');
    end;

  {* generate UNpacking statements for OUT and INOUT params *}
    if ptr^.list <> nil then genupack(opufile, ptr^.list, outok, false);

  {* clear up everything and exit *}
    if omode in cmode then
    begin
      writeln(opufile, '    c_dispose(rpc_p_buf);');
      if ptr^.butype = functok then writeln(opufile, '    return(rpc_ret);');
      writeln(opufile, '    }');
    end
    else begin
      if ptr^.butype = functok then
      begin
          write(opufile, '    ');
          writeuname(opufile, ptr^.name);
          writeln(opufile, ' := rpc_ret;');
      end;
      writeln(opufile, '  END  {WITH};');
      writeln(opufile, '  rpc_dispose(rpc_p_buf);');
      writeln(opufile, 'END;');
    end;
    writeln(opufile);
 end; {gen_client_block_pas}
{___________________________________________________
}
  procedure genuclientublockuc(ptr: ptrublockutable);

{***
* On each block, the client allocates 'RPC_integer' variables to pack or
* unpack parameters.
****}
  var
        a:              integer;
        returnuvalue:   expression;     { Expression for return value }

  begin {gen_client_block_c}

  {* generate and print procedure header *}

    sizeusoufar := 0;       { No marshalling code generated yet }
    genuheader(opufile, ptr, false, true);

  {* print local vars and buffer allocation *}

    writeln(opufile);
    writeln(opufile, '{   rpc_message *rpc_p_buf;');
    for a := 1 to ptr^.blkunesting do
          writeln(opufile, '    register rpc_integer rpc_',
                chr(ord('a') - 1 + a), ';');
    if ptr^.butype = functok then
    begin
          write(opufile, '    ');
          writok(opufile, ptr^.return);
          writeln(opufile, ' rpc_ret;');
    end;
    writeln(opufile);
    write(opufile, '    ');

    write(opufile, 'c_begin(rpc_p_buf, h_');
    writeuname(opufile, unitname);
    writeln(opufile, ', /*vers=*/', versionunum:1,
                     ', /*proc=*/', procunumber:1, ');' );

  {* generate packing statements for IN & INOUT params *}
    if ptr^.list <> nil then genupack(opufile, ptr^.list, intok, true);

  {* generate rpc_call and bookeeping *}
    write(opufile, '    ');
    if ptr^.blkustatusuparam <> nil then begin
        write(opufile, '*');
        writeuname(opufile, ptr^.blkustatusuparam^.name);
        write(opufile, ' = ');
    end {if};
    write(opufile, 'c_call');
    if ptr^.blkustatusuparam <> nil then write(opufile, '_status');
    write(opufile, '(h_');
    writeuname(opufile, unitname);
    writeln(opufile, ', rpc_p_buf, ', ptr^.blkutimeout:1, ');');

    if ptr^.blkustatusuparam <> nil then begin
        write(opufile, '  if (*');
        writeuname(opufile, ptr^.blkustatusuparam^.name);
        writeln(opufile, ' & 1) { /* If good call */ ');
    end {if};

    sizeusoufar := 0;       { No unmarshalling code generated yet }
    if ptr^.blkumaxuout>0 then
        writeln(opufile, '    rpc_p_buf->m_index = RETURN_HEADER_LENGTH;');

  {* if function: unpack return value *}
    if ptr^.butype = functok then
    begin
      write(opufile, '    ');
      dopackusimple(opufile, ptr^.return, false);
      writeln(opufile, 'rpc_ret);');
    end;

  {* generate UNpacking statements for OUT and INOUT params *}
    if ptr^.list <> nil then genupack(opufile, ptr^.list, outok, false);

    if ptr^.blkustatusuparam <> nil
    then writeln(opufile, '  } /* end if good call */');


  {* clear up everything and exit *}
    writeln(opufile, '    c_dispose(rpc_p_buf);');
    if ptr^.butype = functok then writeln(opufile, '    return(rpc_ret);');
    writeln(opufile, '    }');
    writeln(opufile);
 end; {gen_client_block_c}

{___________________________________________________
}
 procedure genuopen;
 begin
  if omode in [vaxvms, vaxpas] then
      if runoptions[noautoinit].value
          then writeln(opufile, '[GLOBAL]')
          else writeln(opufile, '[GLOBAL, INITIALIZE]');
  writok(opufile, proctok);
  write(opufile, 'open_');
  writeuname(opufile, unitname);
  if omode in cmode then writeln(opufile, '()')
                    else writeln(opufile, ';');
  if omode = m6809 then writeln(opufile, '   ENTRY;');

  if omode in cmode then
  begin
      writeln(opufile, '{   rpc_status       status;');
  end
  else begin
      writeln(opufile, 'VAR');
      writeln(opufile, '    status : rpc_status;');
      writeln(opufile, '    service : rpc_name;');
      writeln(opufile, 'BEGIN');
      write(opufile, '    service := ');
      writeunameupadded(opufile, unitname, '''');
      writeln(opufile, ';');
  end;

  if omode in cmode then writeln(opufile);
  write(opufile, '    ');
  if not (omode in cmode) then write(opufile, 'rp');
  write(opufile, 'c_open(status, h_');
  writeuname(opufile, unitname);
  write(opufile, ', ');
  if omode in cmode then writeunameupadded(opufile, unitname,'"')
        else write(opufile, 'service');
  writeln(opufile, ');');

  write(opufile, '    ');
  if not (omode in cmode) then write(opufile, 'rp');
  writeln(opufile, 'c_report_error(status);');
  if omode in cmode then writeln(opufile, '    }')
      else writeln(opufile, 'END;');
 end; {GEN_OPEN}
{______________________________________________

        Generate procedure to close stub:  close_xxx()

    In PCturbo, this is a valid exit procedure, which replaces the exit
    procedure pointer when it is done.
}
    procedure genuclose;
    begin
        if omode in [vaxvms, vaxpas] then writeln(opufile, '[GLOBAL]');
        writok(opufile, proctok);
        write(opufile, 'close_');
        writeuname(opufile, unitname);
        if omode in cmode then writeln(opufile, '()')
                          else writeln(opufile, ';');
        if omode = m6809 then writeln(opufile, '   ENTRY;');

        if omode in cmode then
        begin
            writeln(opufile, '{   rpc_status       status;');
            writeln(opufile);
        end else begin
          writeln(opufile, 'VAR');
          writeln(opufile, '    status : rpc_status;');
          writeln(opufile, 'BEGIN');
        end;

        write(opufile, '    ');
        if not (omode in cmode) then write(opufile, 'rp');
        write(opufile, 'c_close(status, h_');
        writeuname(opufile, unitname);
        writeln(opufile, ');');

        write(opufile, '    ');
        if not (omode in cmode) then write(opufile, 'rp');
        writeln(opufile, 'c_report_error(status);');

        if (omode = pcturbo) then begin
            write(opufile, '   ExitProc := e_');
            writeuname(opufile, unitname);
            writeln(opufile, ';');
        end;

        if omode in cmode then writeln(opufile, '    }')
                          else writeln(opufile, 'END;');
    end; {gen_close}

{______________________________________________________
}
begin {CLIENT_GENERATOR}

if fileuopen(opufile, cliuname, rewriting) then begin

    omode := clientumode;
    client := true;
    fragmentationuused := false;

    if omode=pils then clientugeneratorupils
    else if omode=vaxfor then clientugenufor
    else begin


        {* Generate module name and type, var, procedure inclusion *}
        if not (omode=monolith) then genumodule(opufile);

{       Special INTERFACE part for PCTurbo and MacTurbo:
}

        { For PCTurbo provide a variable to save the address of ExitProc }

        if (omode = pcturbo) then begin
            write(opufile, '    e_');           { Add a variable for close_x }
            writeuname(opufile, unitname);
            writeln(opufile, ':   pointer;');
        end;

        if (omode in [pcturbo, macturbo]) then begin

            { For MacTurbo and PCTurbo the global procedure headings
              must appearin the interface part.
              For MacTurbo we must generate the procedure heading
              formal parameter part ONLY in the INTERFACE section.
            }

            procunumber := 1;
            scan := blockptr;
            while scan <> nil do begin
                genuheader(opufile, scan, false, true);
                procunumber := procunumber + 1;
                scan := scan^.next;
            end {while};

            write       (opufile, 'PROCEDURE open_');
            writeuname  (opufile, unitname);
            writeln     (opufile, ';');

            write       (opufile, 'PROCEDURE close_');
            writeuname  (opufile, unitname);
            writeln     (opufile, ';');

            writeln     (opufile);
            writeln     (opufile, 'IMPLEMENTATION');
            writeln     (opufile);
        end;

{*      Generate procedures *}

        procunumber := 1;
        scan := blockptr;
        while scan <> nil do begin
                if omode in cmode   then genuclientublockuc(scan)
                                    else genuclientublockupas(scan);
                procunumber := procunumber + 1;
                scan := scan^.next;
        end;

        genuopen;
        genuclose;      { Added for all versions now 8-May-1989 }

{       Specials for PCTurbo and MacTurbo :  Autoinit.
}

        { Exit handler only in PCTurbo version. }

        if (omode in [pcturbo, macturbo]) then begin
            if (not runoptions[noautoinit].value) or (omode = pcturbo) then
                writeln(opufile,'BEGIN');

            if (omode = pcturbo) then begin
                write(opufile,'  e_');
                    writeuname(opufile, unitname);
                    writeln(opufile, ' := ExitProc;');
                write(opufile,'  ExitProc := @close_');
                    writeuname(opufile, unitname);
                    writeln(opufile, ';');
            end;

            if (not runoptions[noautoinit].value) then begin
                write(opufile,'  open_');
                    writeuname(opufile, unitname);
                    writeln(opufile, ';');
            end;

            writeln(opufile,'END.');
        end {if};

{*      Generate end of module *}

        if (omode in [vaxvms, vaxpas]) then
                writeln(opufile, 'END .')
        else if (omode = m6809) then
                writeln(opufile, 'MODEND .')
        else if (omode = cerncross) then
                writeln(opufile, '.');

    end {if not pils or FORTRAN};
    if fragmentationuused
        then writeln('Buffer fragmentation may be used for large parameters.');
    if not(fileuclose(opufile)) then
                error(cantuclsuclient);
end else
        error(cantuopnuclient);
end; {CLIENT_GENERATOR}



{*****************************************************************************

        S E R V E R     C O D E         G E N E R A T O R
        *************************************************

The server stub is produced according to

}
procedure serverugenerator( serverumode:        outputumode;
                                seruname:       astring);
var
        scan : ptrublockutable;


{       Generate server stub for one Procedure/function
        -----------------------------------------------
}
procedure genuserverublock(ptr: ptrublockutable);
{***
* On each block, the server allocates 'RPC_integer' variables to pack or
* unpack parameters, accoring to the maximum number (blk_nesting) needed
* at any one time.
****}
var
        a:              integer;
        scanlist:       ptruidlist;
        parucount:      integer;        { how far we are getting across line }

  procedure writeucamp;
  begin
      write(opufile, ', ');
      if (omode in cmode)
              and not (runoptions[byvalue].value and (scanlist^.attr = intok))
          then write(opufile, '&');
  end;

begin {gen_server_block}

  sizeusoufar := 0;         { No unmarshalling code generated yet }

  {* generate and print procedure header *}
  genuheader(opufile, ptr, false, true);

  {* print local vars and buffer allocation *}
  for a := 1 to ptr^.blkunesting do
  begin
      write(opufile, '    ');
      if omode in cmode then write(opufile, 'register rpc_integer ');
      write(opufile, 'rpc_', chr(ord('a') - 1 + a));
      if not (omode in cmode) then write(opufile, ' : rpc_integer');
      writeln(opufile, ';');
  end;

  if omode in cmode then writeln(opufile)
  else begin
      writeln(opufile, 'BEGIN');
      writeln(opufile, '  WITH rpc_p_buf^ DO BEGIN');
  end;

  {* generate UNpacking statements for IN and INOUT params *}
  if ptr^.list <> nil then genupack(opufile, ptr^.list, intok, false);

{ Generate call and bookeeping:
}
  if omode in cmode
    then writeln(opufile, '    c_turn(rpc_p_buf);')
    else begin {pascal}
        writeln(opufile, '    rpc_p_buf^.which := RETURN_MESSAGE;');
        writeln(opufile, '    rpc_p_buf^.m_index := RETURN_HEADER_LENGTH;');
    end; {if pascal}

  sizeusoufar := 0;         { No marshalling code generated yet }

  if (ptr^.butype = proctok) and ptr^.inuonly
  then if (runoptions[concurrent].value or ptr^.blkuconcurrent)
         then begin
            if (omode in cmode) then write(opufile, '    c')
                                else write(opufile, '    rpc');
            writeln(opufile, '_early_return(rpc_p_buf);')
         end else if ptr^.blkucast
         then   if omode in cmode
                then writeln(opufile, '    rpc_p_buf->m_status=1;')
                else writeln(opufile, '    m_status:=1;');
  write(opufile, '    ');

{   If function: generate return variable *}
  if ptr^.butype = functok then dopackusimple(opufile, ptr^.return, true);

{ Generate list of variables:
}
  writeuname(opufile, ptr^.name);
  if ptr^.list <> nil then
  begin
      write(opufile, '(');
      scanlist := ptr^.list;
      parucount := 1;
      repeat

{       Generate address of simple types and records in C if necessary:
}
          if (omode in cmode)
             and (scanlist^.idutype^.typubasicutype in
                                [chartok..longtok, recordtok])
             and not (runoptions[byvalue].value and (scanlist^.attr = intok))
          then write(opufile, '&');

{ If var is SEQUENCE or SUBSTRING or STRING put all names
}
          if (scanlist^.idutype^.typubasicutype = sequence) then
          begin
              write(opufile, 'a_');
              writeuname(opufile, scanlist^.name);
              parucount := parucount + 1;
              writeucamp;
              write(opufile, 'l_');
          end
          else if (scanlist^.idutype^.typubasicutype = substring) then
          begin
              write(opufile, 'a_');
              writeuname(opufile, scanlist^.name);
              writeucamp;
              write(opufile, 's_');
              writeuname(opufile, scanlist^.name);
              parucount := parucount + 2;
              writeucamp;
              write(opufile, 'l_');
          end
          else if (scanlist^.idutype^.typubasicutype = stringtok)
                 and not (omode in (cmode +
                        [vaxvms, vaxpas, m6809, pcturbo, macturbo])) then
          begin
              write(opufile, 'a_');
              writeuname(opufile, scanlist^.name);
              parucount := parucount + 1;
              write(opufile, ', l_');
          end;

{   Write the main parameter name:
}
          writeuname(opufile, scanlist^.name);

          parucount := parucount + 1;
          scanlist := scanlist^.next;
          if scanlist <> nil then begin
              write(opufile, ', ');
              if parucount > 3 then     { Line wrap around }
              begin
                  writeln(opufile);
                  write(opufile, '        ');
                  parucount := 0;
              end;
          end {if};
      until scanlist = nil;
      write(opufile, ')');
  end
  else if omode in cmode then write(opufile, '()');

  {* if function generates return variable *}
  if ptr^.butype = functok then write(opufile, ')');

  writeln(opufile, ';');

  {* generate packing statements for OUT and INOUT params*}
  if ptr^.list <> nil then genupack(opufile, ptr^.list, outok, true);

{* writeln(op_file, '    rpc_early_return(rpc_p_buf, m_index);');   oct 86 *}

  if omode in cmode then writeln(opufile, '    }')
  else begin
      writeln(opufile, '  END {with};');
      writeln(opufile, 'END;');
  end;
  writeln(opufile);
end; {gen_server_block}


{       Generate Main Server Procedure
        ------------------------------
}
procedure genujumpuproc;
var
        procunum : integer;

begin
  if omode in cmode then
  begin
      writeln(opufile, 'static rpc_message *rpc_p_buf;');
      writeln(opufile);
      scan := blockptr;
      procunum := 0;
      while scan <> nil do
      begin
          genuserverublock(scan);
          scan := scan^.next;
          procunum := procunum + 1;
      end;
  end;

  if omode in [vaxvms, vaxpas] then writeln(opufile, '[GLOBAL]');
  writok(opufile, proctok);
  write(opufile, 'r_');
  writeuname(opufile, unitname);
  if omode in cmode
      then writeln(opufile, '(rpc_a) rpc_message **rpc_a;')     { 71217 }
      else writeln(opufile, '(VAR rpc_p_buf : rpc_message_pointer);');
  if omode = m6809 then writeln(opufile, '    ENTRY;');

  if omode in cmode then writeln(opufile, '{   rpc_short request;')
  else begin
      writeln(opufile, 'VAR');
      writeln(opufile, '    request : rpc_short;');
  end;

   {* writeln(op_file, '    status: rpc_status;'); *}

{       All the procedures to service each procedure/function are
        now declared as local to the jump procedure. In this way they
        can use the parameter rpc_p_buf (TBL Oct 86).

        EXCEPT in Cmode: here the service functions are declared static
        and beforehand; rpc_p_buf is a static global that is set on entry
        to this procedure (Nici Aug 87).
}
  if not (omode in cmode) then
  begin
      scan := blockptr;
      procunum := 0;
      while scan <> nil do
      begin
          genuserverublock(scan);
          scan := scan^.next;
          procunum := procunum + 1;
      end;

      writeln(opufile, 'BEGIN');
      writeln(opufile, '  WITH rpc_p_buf^ DO BEGIN');
      write(opufile, '    m_index :');
  end
  else begin
      writeln(opufile);
      writeln(opufile, '    rpc_p_buf = *rpc_a;');              { 71217 }
      write(opufile, '    rpc_p_buf->m_index ');
  end;

  write(opufile, '= CALL_HEADER_LENGTH - ');
  if versionunum = 0 then writeln(opufile, '2;')
  else begin
      writeln(opufile, '4;');
      writeln(opufile, '    upk_short(rpc_p_buf, request);');
      if omode in cmode then
      begin
          writeln(opufile,
                '    if (request != 0 && request != ', versionunum:1, ')');
          writeln(opufile,
                '         rpc_p_buf->m_status = RPC_S_UNSUPPORTED_VERSION;');
          writeln(opufile,
                '    else {');
      end
      else begin
          writeln(opufile, '    IF (request<>0) AND (request<>', versionunum:1, ')');
          writeln(opufile, '    THEN m_status := RPC_S_UNSUPPORTED_VERSION');
          writeln(opufile, '    ELSE begin');
      end;
  end;
  writeln(opufile, '     upk_short(rpc_p_buf, request);');

  if omode in cmode then
  begin
      writeln(opufile, '     switch (request)');
      write(opufile, '     {   ');
  end
  else begin
      writeln(opufile, '     IF (request = 0) OR (request > ', procunum:1, ') THEN');
      writeln(opufile, '         m_status := RPC_S_BAD_PROCEDURE_NUMBER');
      writeln(opufile, '     ELSE CASE request OF');
      write(opufile, '        ');
  end;

  scan := blockptr;
  procunum := 1;
  while scan <> nil do
  begin
      if omode in cmode then write(opufile, 'case ');
      write(opufile, procunum:2, ' : r_');
      writeuname(opufile, scan^.name);
      if omode in cmode then write(opufile, '()');
      writeln(opufile, ';');
      procunum := procunum + 1;
      scan := scan^.next;
      if (omode in cmode) then
              writeln(opufile, '                  break;');
      write(opufile, '        ');
  end;

  if omode in cmode then
  begin
      writeln(opufile, 'default : rpc_p_buf->m_status = RPC_S_BAD_PROCEDURE_NUMBER;');
      writeln(opufile, '        }');
      if versionunum <> 0 then writeln(opufile, '     }');      { switch }
      writeln(opufile, '    }');                { if else }
  end
  else begin
      writeln(opufile, 'END {case};');
      if versionunum <> 0 then writeln(opufile, '    END {if};');
      writeln(opufile, '  END {with};');
      writeln(opufile, 'END;');
  end;
  writeln(opufile);
end; {GEN_JUMP_PROC}


{       Generate Code to Attach stub to RPCRTS
        --------------------------------------
}

procedure genuattach;
begin
  if omode in [vaxvms, vaxpas] then
      if runoptions[noautoinit].value
          then writeln(opufile,   '[GLOBAL]')
          else writeln(opufile,   '[GLOBAL, INITIALIZE]');

  writok(opufile, proctok);
  write(opufile, 'attach_');
  writeuname(opufile, unitname);
  if omode in cmode then writeln(opufile, '()')
                    else writeln(opufile, ';');

  if omode in cmode then
  begin
      writeln(opufile, '{   rpc_status      status;');
      writeln(opufile);
  end
  else begin
      writeln(opufile, 'VAR');
      writeln(opufile, '    status  : rpc_status;');
      writeln(opufile, '    service : rpc_name;');
      writeln(opufile, 'BEGIN');
      write(opufile, '    service := ');
      writeunameupadded(opufile, unitname, '''');
      writeln(opufile, ';');
  end;

  write(opufile, '    ');
  if not (omode in cmode) then write(opufile, 'rp');

  { For PCTurbo and MacTurbo use the @ operator to pass the stub pointer. }

  if (omode in [pcturbo, macturbo]) then
        write(opufile, 'c_attach_stub(status, @r_')
  else
        write(opufile, 'c_attach_stub(status, r_');

  writeuname(opufile, unitname);
  write(opufile, ', ');
  if omode in cmode then begin
                writeln(opufile); write(opufile, '        ');
                writeunameupadded(opufile, unitname, '"')
        end else
                write (opufile, 'service');
  write  (opufile, ', p_');
  writeuname(opufile, unitname);
  writeln(opufile, ');');

  write(opufile, '    ');
  if not (omode in cmode) then write(opufile, 'rp');
  writeln(opufile, 'c_report_error(status);');

  if omode in cmode
      then writeln(opufile, '    }')
      else writeln(opufile, 'END;');
end; {GEN_ATTACH}


{       Generate Code to Detach stub from RPCRTS
        ----------------------------------------
}

    procedure genudetach;
    begin
      if omode in [vaxvms, vaxpas] then writeln(opufile,   '[GLOBAL]');

      writok(opufile, proctok);
      write(opufile, 'detach_');
      writeuname(opufile, unitname);
      if omode in cmode then writeln(opufile, '()')
                        else writeln(opufile, ';');

      if omode in cmode then begin
          writeln(opufile, '{');
      end else begin
          writeln(opufile, 'BEGIN');
      end;

      write(opufile, '    ');
      if not (omode in cmode) then write(opufile, 'rp');

      write(opufile, 'c_detach_stub(p_');
      writeuname(opufile, unitname);
      writeln(opufile, ');');

      if omode in cmode
          then writeln(opufile, '    }')
          else writeln(opufile, 'END;');

    end; {gen_detach}


{       MAIN BLOCK:     GENERATE SERVER FILE
        ==========      ====================
}

begin {SERVER_GENERATOR}

  if fileuopen(opufile, seruname, rewriting) then begin

    omode := serverumode;
    client := false;
    fragmentationuused := false;

    if omode=pils then serverugeneratorupils
    else if omode=vaxfor then serverugenufor
    else begin

        {* Generate module name and type, var, procedure inclusion *}
        if not (omode in [monolith]) then genumodule(opufile);

{       Special INTERFACE part for PCTurbo and MacTurbo:
}

        if (omode in [pcturbo, macturbo]) then begin

            { For MacTurbo and PCTurbo the global procedure
              headings must appear in the interface part.
            }

            { Declare the attach procedure. }

            write       (opufile, 'PROCEDURE attach_');
            writeuname  (opufile, unitname);
            writeln     (opufile, ';');

            { For PCTurbo we must declare the main server procedure in the
              INTERFACE part in order to force generation of FAR model code.
              This is necessary because it will be called FAR by
              Call_Local_Stub.
            }

            if (omode = pcturbo) then begin
                write   (opufile, 'PROCEDURE r_');
                writeuname      (opufile, unitname);
                writeln (opufile, '(VAR rpc_p_buf : rpc_message_pointer);');
            end;

            writeln     (opufile);
            writeln     (opufile, 'IMPLEMENTATION');
            writeln     (opufile);
        end;

        {* Generate the jump table for the server *}
        genujumpuproc;

        {* Generate the attach procedure *}
        if not (omode in [m6809]) then genuattach;
                        {* this can't handle procedure parameters *}

        {* Generate end of module *}
        if (omode in [vaxvms, vaxpas, pcturbo, macturbo]) then
                writeln(opufile, 'END .')
        else if (omode = m6809) then
                writeln(opufile, 'MODEND .')
        else if (omode = cerncross) then
                writeln(opufile, '.');

    end {if not pils or vaxfor};
    if not(fileuclose(opufile)) then
                error(cantuclsuserver);
    if fragmentationuused
        then writeln('Buffer fragmentation may be used for large parameters.');
  end else
        error(cantuopnuserver);
end; {SERVER_GENERATOR}



{               Main Program
                ------------
}
begin {MAIN}

    inituglobal;                        {* Global initialization *}
    getuparameters;                     {* Get command line parameters *}
    parser;                             {* Parse input *}

    if not fileuclose(inpufile)         {* Close input file *}
        then error(cantuclsuinput);

    if not runoptions[version].value    {* Generate stub version number *}
    then begin
        versionunum := checksum + 1000;
        writeln('RPCC: generated stub version number is ', versionunum:1, '.');
    end;

    if runoptions[dtree].value          {* Debugging : print trees *}
    then begin
        printutabutypes;
        printutabublocks;
    end
    else if not (seruspec or cliuspec) then
        writeln('RPCC: no stubs specified - no code generated, hope that''s okay.');

    if errorfound > 0 then begin    {* Exit gracefully on error *}
           writeln('RPCC: ',errorfound:3,
                ' errors found, no code generated.');
           halt;
    end;

{       Generate client stubs and .ext files }

    if cliuspec then begin
        clientugenerator(cliumode, cliuname);
        extugenerator(cliumode, extuname)
    end;

{       Generate server stubs }

    if seruspec then begin
        serverugenerator(serumode, seruname);
    end;

end. {MAIN}
